@@ -40,8 +40,8 @@ function adaptLayoutToSize() {
     if (l_deb < log_level) {
         alert("[UI] adaptLayoutToSize");
     }
-    display_width = parseInt(scene.get_option('General', 'LastWidth'));
-    display_height = parseInt(scene.get_option('General', 'LastHeight'));
+    display_width = parseInt(scene.get_option('GUI', 'LastWidth'));
+    display_height = parseInt(scene.get_option('GUI', 'LastHeight'));
     alert("display "+display_width+" "+display_height);
     if (!scene.fullscreen && display_width && display_height) {
      scene.set_size(display_width, display_height);
@@ -1561,7 +1561,7 @@ function media(etype, uri, w, h) {
     child.setAttribute('width', w);
     child.setAttribute('height', h);
     if (etype == 'animation') {
-        child.setAttributeNS('http://gpac.io/svg-extensions', 'use-as-primary', 'false');
+        child.setAttributeNS('https://gpac.io/svg-extensions', 'use-as-primary', 'false');
     }
     return child;
 }
