@@ -56,11 +56,11 @@ GF_Err gf_filter_pck_merge_properties_filter(GF_FilterPacket *pck_src, GF_Filter
 	//restore internal props flags of packet
 	pck_dst->info.flags |= iflags;
 
-	if (!pck_src->props || pck_dst->is_dangling) {
+	if (!pck_src->props || (!pck_dst->pid && !pck_src->pid)) {
 		return GF_OK;
 	}
 	if (!pck_dst->props) {
-		pck_dst->props = gf_props_new(pck_dst->pid->filter);
+		pck_dst->props = gf_props_new(pck_dst->pid ? pck_dst->pid->filter : pck_src->pid->filter);
 
 		if (!pck_dst->props) return GF_OUT_OF_MEM;
 	}
@@ -129,7 +129,7 @@ static GF_FilterPacket *gf_filter_pck_new_alloc_internal(GF_FilterPid *pid, u32
 		closest = pck_enum_state.closest;
 	}
 
-	//stop allocating after a while - TODO we for sur can design a better algo...
+	//stop allocating after a while - TODO we for sure can design a better algo...
 	max_reservoir_size = pid->num_destinations ? 10 : 1;
 	//if pid is file, force 1 max
 	if (!pck && (pid->stream_type==GF_STREAM_FILE))
@@ -216,6 +216,14 @@ static GF_FilterPacket *gf_filter_pck_new_dangling_packet(GF_FilterPacket *cache
 			}
 		}
 		cached_pck->data_length = data_length;
+		if (cached_pck->props) {
+			GF_PropertyMap *props = cached_pck->props;
+			cached_pck->props=NULL;
+			gf_assert(props->reference_count);
+			if (safe_int_dec(&props->reference_count) == 0) {
+				gf_props_del(props);
+			}
+		}
 		return cached_pck;
 	}
 
@@ -343,7 +351,7 @@ static GF_FilterPacket *gf_filter_pck_new_clone_internal(GF_FilterPid *pid, GF_F
 			ref = ref->reference;
 		}
 	}
-	
+
 	if (max_ref>1) {
 		u8 *data_new;
 		if (dangling_packet) {
@@ -376,7 +384,7 @@ static GF_FilterPacket *gf_filter_pck_new_clone_internal(GF_FilterPid *pid, GF_F
 		dst->data_length = pck_source->data_length;
 		dst->reference = pck_source;
 		dst->pck = dst;
-		dst->is_dangling = 1;
+		dst->is_dangling = 2;
 
 		//cf gf_filter_pck_new_ref for these
 		safe_int_inc(&pck_source->reference_count);
@@ -401,6 +409,13 @@ GF_FilterPacket *gf_filter_pck_dangling_copy(GF_FilterPacket *pck_source, GF_Fil
 	return gf_filter_pck_new_clone_internal(NULL, pck_source, NULL, GF_FALSE, GF_TRUE, cached_pck);
 }
 
+GF_EXPORT
+GF_FilterPacket *gf_filter_pck_dangling_clone(GF_FilterPacket *pck_source, GF_FilterPacket *cached_pck)
+{
+	return gf_filter_pck_new_clone_internal(NULL, pck_source, NULL, GF_TRUE, GF_TRUE, cached_pck);
+}
+
+
 GF_EXPORT
 GF_FilterPacket *gf_filter_pck_new_clone(GF_FilterPid *pid, GF_FilterPacket *pck_source, u8 **data)
 {
@@ -430,6 +445,10 @@ GF_FilterPacket *gf_filter_pck_new_shared_internal(GF_FilterPid *pid, const u8 *
 {
 	GF_FilterPacket *pck;
 
+	if (!pid) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Attempt to allocate a packet on a NULL PID\n"));
+		return NULL;
+	}
 	if (PID_IS_INPUT(pid)) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Attempt to allocate a packet on an input PID in filter %s\n", pid->filter->name));
 		return NULL;
@@ -462,7 +481,6 @@ GF_EXPORT
 GF_FilterPacket *gf_filter_pck_new_shared(GF_FilterPid *pid, const u8 *data, u32 data_size, gf_fsess_packet_destructor destruct)
 {
 	return gf_filter_pck_new_shared_internal(pid, data, data_size, destruct, GF_FALSE);
-
 }
 
 GF_EXPORT
@@ -850,7 +868,11 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 #endif
 
 
-	if (!pck->src_filter) return GF_BAD_PARAM;
+	if (!pck->src_filter || (pck->src_filter->removed==1)) {
+		gf_filter_pck_discard(pck);
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Packet sent on already removed filter, ignoring\n"));
+		return GF_OK;
+	}
 
 #ifdef GPAC_MEMORY_TRACKING
 	if (pck->pid->filter->nb_process_since_reset)
@@ -863,8 +885,13 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 
 	if (PCK_IS_INPUT(pck)) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Attempt to dispatch input packet on output PID in filter %s\n", pck->pid->filter->name));
+		gf_filter_pck_discard(pck);
 		return GF_BAD_PARAM;
 	}
+	if (pid->not_connected) {
+		gf_filter_pck_discard(pck);
+		return GF_OK;
+	}
 
 	is_cmd_pck = (pck->info.flags & GF_PCK_CMD_MASK);
 
@@ -880,8 +907,6 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 
 	pid->filter->nb_pck_io++;
 
-	gf_rmt_begin(pck_send, GF_RMT_AGGREGATE);
-
 	cktype = ( pck->info.flags & GF_PCK_CKTYPE_MASK) >> GF_PCK_CKTYPE_POS;
 
 	//send from filter, update flags
@@ -959,7 +984,6 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			pck->pid->filter->session->nb_realloc_pck += (nb_reallocs - prev_nb_reallocs);
 		}
 #endif
-		gf_rmt_end();
 		return GF_PENDING_PACKET;
 	}
 	//now dispatched
@@ -1120,9 +1144,9 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		Bool post_task=GF_FALSE;
 		GF_FilterPacketInstance *inst;
 		GF_FilterPidInst *dst = gf_list_get(pck->pid->destinations, i);
-		if (!dst->filter || dst->filter->finalized || (dst->filter->removed==1) || !dst->filter->freg->process) continue;
+		if (!dst->filter || dst->filter->finalized || (dst->filter->removed==1) || !dst->filter->freg->process || dst->in_swap) continue;
 
-		if (dst->discard_inputs==1) {
+		if (dst->discard_inputs==GF_PIDI_DISCARD_ON) {
 			//in discard input mode, we drop all input packets but trigger reconfigure as they happen
 			if ((pck->info.flags & GF_PCKF_PROPS_CHANGED) && (dst->props != pck->pid_props)) {
 				//unassign old property list and set the new one
@@ -1144,7 +1168,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 				//in which a previously blacklisted filter (failing (re)configure for previous state) could
 				//now work, eg moving from formatA to formatB then back to formatA
 				gf_list_reset(dst->filter->blacklisted);
-				dst->discard_inputs = 2;
+				dst->discard_inputs = GF_PIDI_DISCARD_RCFG;
 				//and post a reconfigure task
 				gf_fs_post_task(dst->filter->session, gf_filter_pid_reconfigure_task_discard, dst->filter, (GF_FilterPid *)dst, "pidinst_reconfigure", NULL);
 				//keep packets, they will be trashed if we are still in discard when executing gf_filter_pid_reconfigure_task_discard
@@ -1152,6 +1176,14 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 				continue;
 			}
 		}
+		//ignore flush packets if destination requires full blocks and block is in progress
+		if (dst->requires_full_data_block && !dst->last_block_ended && (pck->info.flags & GF_PCKF_IS_FLUSH)) {
+			continue;
+		}
+		//stop forwarding clock packets when in EOS
+		if (dst->is_end_of_stream && cktype) {
+			continue;
+		}
 
 		inst = gf_fq_pop(pck->pid->filter->pcks_inst_reservoir);
 		if (!inst) {
@@ -1269,7 +1301,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 					if (inst->pck->pid_props) {
 						safe_int_inc(&inst->pck->pid_props->reference_count);
 					}
-					
+
 					gf_assert(pck->reference_count);
 					safe_int_dec(&pck->reference_count);
 					safe_int_inc(&inst->pck->reference_count);
@@ -1301,7 +1333,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 				safe_int64_add(&dst->buffer_duration, us_duration);
 			}
 			safe_int_inc(&dst->filter->pending_packets);
-//
+
 			gf_fq_add(dst->packets, inst);
 			post_task = GF_TRUE;
 		}
@@ -1315,7 +1347,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			}
 
 			//make sure we lock the tasks mutex before getting the packet count, otherwise we might end up with a wrong number of packets
-			//if one thread consumes one packet while the dispatching thread  (the caller here) is still upddating the state for that pid
+			//if one thread consumes one packet while the dispatching thread  (the caller here) is still updating the state for that pid
 			gf_mx_p(pid->filter->tasks_mx);
 			u32 nb_pck = gf_fq_count(dst->packets);
 			//update buffer occupancy before dispatching the task - if target pid is processed before we are done disptching his packet, pid buffer occupancy
@@ -1327,7 +1359,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			//otherwise we would have max_buffer_time=1ms (default) and a single AU dispatched would block unless speed is AU_DUR_ms/1ms ...
 			if (us_duration && pid->max_buffer_time && (pid->max_buffer_time<us_duration))
 				pid->max_buffer_time = us_duration;
-				
+
 			gf_mx_v(pid->filter->tasks_mx);
 
 			//post process task
@@ -1357,19 +1389,23 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		}
 		gf_filter_packet_destroy(pck);
 	}
-	gf_rmt_end();
 	return GF_OK;
 }
 
 GF_EXPORT
 GF_Err gf_filter_pck_send(GF_FilterPacket *pck)
 {
+	if (PCK_IS_INPUT(pck)) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Attempt to send a received packet in filter %s\n", pck->pid->filter->name));
+		return GF_BAD_PARAM;
+	}
+	if (!pck || !pck->pid) return GF_BAD_PARAM;
+
 	//dangling packet
 	if (pck->is_dangling) {
 		gf_filter_pck_discard(pck);
 		return GF_OK;
 	}
-	gf_assert(pck->pid);
 	return gf_filter_pck_send_internal(pck, GF_TRUE);
 }
 
@@ -1481,6 +1517,33 @@ static GF_Err gf_filter_pck_set_property_full(GF_FilterPacket *pck, u32 prop_4cc
 	}
 	//get true packet pointer
 	pck=pck->pck;
+
+	if (prop_4cc && value && pck->pid && pck->pid->filter->session->check_props) {
+		u32 ptype = gf_props_4cc_get_type(prop_4cc);
+		u8 c = prop_4cc>>24;
+		if ((c>='A') && (c<='Z')) {
+			if (gf_props_get_base_type(ptype) != gf_props_get_base_type(value->type)) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Assigning packet property %s of type %s in filter %s but expecting %s\n",
+					gf_props_4cc_get_name(prop_4cc),
+					gf_props_get_type_name(value->type),
+					pck->pid->filter->freg->name,
+					gf_props_get_type_name(ptype)
+				));
+				if (gf_sys_is_test_mode())
+					exit(5);
+			}
+			u32 flags = gf_props_4cc_get_flags(prop_4cc);
+			if (!(flags & GF_PROP_FLAG_PCK)) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Assigning packet property %s in filter %s but this is a PID property\n",
+					gf_props_4cc_get_name(prop_4cc),
+					pck->pid->filter->freg->name
+				));
+				if (gf_sys_is_test_mode())
+					exit(5);
+			}
+		}
+	}
+
 	hash = gf_props_hash_djb2(prop_4cc, prop_name ? prop_name : dyn_name);
 
 	if (!pck->props) {
@@ -1489,7 +1552,7 @@ static GF_Err gf_filter_pck_set_property_full(GF_FilterPacket *pck, u32 prop_4cc
 		gf_props_remove_property(pck->props, hash, prop_4cc, prop_name ? prop_name : dyn_name);
 	}
 	if (!value) return GF_OK;
-	
+
 	return gf_props_insert_property(pck->props, hash, prop_4cc, prop_name, dyn_name, value);
 }
 
@@ -1557,12 +1620,25 @@ static GFINLINE const GF_PropertyValue *pck_check_prop(GF_FilterPacket *pck, u32
 #else
 #define pck_check_prop(_a, _b, _str, c) c
 #endif
+
 GF_EXPORT
 const GF_PropertyValue *gf_filter_pck_get_property(GF_FilterPacket *_pck, u32 prop_4cc)
 {
 	//get true packet pointer
 	GF_FilterPacket *pck = _pck->pck;
 	if (!pck->props) return NULL;
+	if (pck->pid && pck->pid->filter->session->check_props && gf_props_4cc_get_type(prop_4cc)) {
+		u32 flags = gf_props_4cc_get_flags(prop_4cc);
+		if (!(flags & GF_PROP_FLAG_PCK)) {
+			GF_Filter *f = (_pck == pck) ? pck->pid->filter : ((GF_FilterPacketInstance*)_pck)->pid->filter;
+			GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Fetching packet property %s in filter %s but this is a PID property\n",
+				gf_props_4cc_get_name(prop_4cc),
+				f->freg->name
+			));
+			if (gf_sys_is_test_mode())
+				exit(5);
+		}
+	}
 	return pck_check_prop(_pck, prop_4cc, NULL, gf_props_get_property(pck->props, prop_4cc, NULL));
 }
 
@@ -1672,6 +1748,25 @@ GF_FilterSAPType gf_filter_pck_get_sap(GF_FilterPacket *pck)
 	return (GF_FilterSAPType) ( (pck->pck->info.flags & GF_PCK_SAP_MASK) >> GF_PCK_SAP_POS);
 }
 
+GF_EXPORT
+GF_Err gf_filter_pck_set_switch_frame(GF_FilterPacket *pck, Bool is_switch_frame)
+{
+	PCK_SETTER_CHECK("switch_frame")
+	pck->info.flags &= ~GF_PCKF_IS_SWITCH_FRAME;
+	if (is_switch_frame) {
+		pck->info.flags |= GF_PCKF_IS_SWITCH_FRAME;
+	}
+	return GF_OK;
+}
+
+GF_EXPORT
+Bool gf_filter_pck_get_switch_frame(GF_FilterPacket *pck)
+{
+	gf_assert(pck);
+	//get true packet pointer
+	return (Bool) (pck->pck->info.flags & GF_PCKF_IS_SWITCH_FRAME) ? GF_TRUE : GF_FALSE;
+}
+
 GF_EXPORT
 GF_Err gf_filter_pck_set_roll_info(GF_FilterPacket *pck, s16 roll_count)
 {
@@ -1867,7 +1962,7 @@ GF_Err gf_filter_pck_expand(GF_FilterPacket *pck, u32 nb_bytes_to_add, u8 **data
 		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Attempt to reallocate input packet on output PID in filter %s\n", pck->pid->filter->name));
 		return GF_BAD_PARAM;
 	}
-	if (! pck->src_filter) {
+	if (! pck->src_filter && (pck->is_dangling!=1)) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Attempt to reallocate an already sent packet in filter %s\n", pck->pid->filter->name));
 		return GF_BAD_PARAM;
 	}
@@ -1882,7 +1977,8 @@ GF_Err gf_filter_pck_expand(GF_FilterPacket *pck, u32 nb_bytes_to_add, u8 **data
 		pck->alloc_size = pck->data_length + nb_bytes_to_add;
 		pck->data = gf_realloc(pck->data, pck->alloc_size);
 #ifdef GPAC_MEMORY_TRACKING
-		pck->pid->filter->session->nb_realloc_pck++;
+		if (!pck->is_dangling)
+			pck->pid->filter->session->nb_realloc_pck++;
 #endif
 	}
 	pck->info.byte_offset = GF_FILTER_NO_BO;
