@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2022-2023
+ *			Copyright (c) Telecom ParisTech 2022-2024
  *					All rights reserved
  *
  *  This file is part of GPAC / compressed split bitstream aggregator filter
@@ -261,7 +261,7 @@ static GF_Err bsagg_transfer_param_array(GF_List *array_out, GF_List *array_in)
 
 static GF_Err vvc_hevc_rewrite_pid_config(BSAggCtx *ctx, BSAggOut *pctx)
 {
-	GF_Err e;
+	GF_Err e = GF_OK;
 	u32 i, count;
 	Bool is_vvc = GF_FALSE;
 	GF_HEVCConfig *hvcc_out = NULL;
@@ -961,7 +961,7 @@ static const GF_FilterCapability BSAggCaps[] =
 
 GF_FilterRegister BSAggRegister = {
 	.name = "bsagg",
-	GF_FS_SET_DESCRIPTION("Compressed layered bitstream aggregator")
+	GF_FS_SET_DESCRIPTION("Layered bitstream aggregator")
 	GF_FS_SET_HELP("This filter aggregates layers and sublayers into a single output PID.\n"
 	"\n"
 	"The filter supports AVC|H264, HEVC and VVC stream reconstruction, and is passthrough for other codec types.\n"
@@ -981,7 +981,8 @@ GF_FilterRegister BSAggRegister = {
 	.initialize = bs_agg_initialize,
 	.finalize = bs_agg_finalize,
 	.configure_pid = bs_agg_configure_pid,
-	.process = bs_agg_process
+	.process = bs_agg_process,
+	.hint_class_type = GF_FS_CLASS_STREAM
 };
 
 const GF_FilterRegister *bsagg_register(GF_FilterSession *session)
@@ -994,4 +995,3 @@ const GF_FilterRegister *bsagg_register(GF_FilterSession *session)
 	return NULL;
 }
 #endif
-
