@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2023
+ *			Copyright (c) Telecom ParisTech 2023-2024
  *					All rights reserved
  *
  *  This file is part of GPAC / WebCodec decoder filter
@@ -26,7 +26,7 @@
 #include <gpac/internal/media_dev.h>
 #include <gpac/constants.h>
 
-#if defined(GPAC_CONFIG_EMSCRIPTEN)
+#ifndef GPAC_DISABLE_WEBCODEC
 
 typedef struct
 {
@@ -48,6 +48,8 @@ typedef struct
 	char szCodec[RFC6381_CODEC_NAME_SIZE_MAX];
 } GF_WCDecCtx;
 
+#if defined(GPAC_CONFIG_EMSCRIPTEN)
+
 GF_EXPORT
 void wcdec_on_error(GF_WCDecCtx *ctx, int state, char *msg)
 {
@@ -66,7 +68,7 @@ void wcdec_on_error(GF_WCDecCtx *ctx, int state, char *msg)
 }
 
 EM_JS(int, wcdec_init, (int wc_ctx, int _codec_str, int width, int height, int sample_rate, int num_channels, int dsi, int dsi_size), {
-	let codec_str = _codec_str ? libgpac.UTF8ToString(_codec_str) : null;
+	let codec_str = _codec_str ? UTF8ToString(_codec_str) : null;
 	let config = {};
 	config.codec = codec_str;
 	let dec_class = null;
@@ -80,7 +82,7 @@ EM_JS(int, wcdec_init, (int wc_ctx, int _codec_str, int width, int height, int s
 		dec_class = AudioDecoder;
 	}
 	if (dsi_size) {
-		config.description = new Uint8Array(libgpac.HEAPU8.buffer, dsi, dsi_size);
+		config.description = new Uint8Array(HEAPU8.buffer, dsi, dsi_size);
 	}
 
 	if (typeof libgpac._to_webdec != 'function') {
@@ -91,11 +93,11 @@ EM_JS(int, wcdec_init, (int wc_ctx, int _codec_str, int width, int height, int s
           }
           return null;
 		};
-		libgpac._on_wcdec_error = libgpac.cwrap('wcdec_on_error', null, ['number', 'number', 'string']);
-		libgpac._on_wcdec_frame = libgpac.cwrap('wcdec_on_video', null, ['number', 'bigint', 'string', 'number', 'number']);
-		libgpac._on_wcdec_audio = libgpac.cwrap('wcdec_on_audio', null, ['number', 'bigint', 'string', 'number', 'number', 'number']);
-		libgpac._on_wcdec_flush = libgpac.cwrap('wcdec_on_flush', null, ['number']);
-		libgpac._on_wcdec_frame_copy = libgpac.cwrap('wcdec_on_frame_copy', null, ['number', 'number', 'number']);
+		libgpac._on_wcdec_error = cwrap('wcdec_on_error', null, ['number', 'number', 'string']);
+		libgpac._on_wcdec_frame = cwrap('wcdec_on_video', null, ['number', 'bigint', 'string', 'number', 'number']);
+		libgpac._on_wcdec_audio = cwrap('wcdec_on_audio', null, ['number', 'bigint', 'string', 'number', 'number', 'number']);
+		libgpac._on_wcdec_flush = cwrap('wcdec_on_flush', null, ['number']);
+		libgpac._on_wcdec_frame_copy = cwrap('wcdec_on_frame_copy', null, ['number', 'number', 'number']);
 	}
 	let c = libgpac._to_webdec(wc_ctx);
 	if (!c) {
@@ -256,7 +258,7 @@ EM_JS(int, wcdec_push_frame, (int wc_ctx, int buf, int buf_size, int key, u64 ts
 	const chunk = new EncodedVideoChunk({
 		timestamp: Number(ts),
 		type: key ? "key" : "delta",
-		data: new Uint8Array(libgpac.HEAPU8.buffer, buf, buf_size)
+		data: new Uint8Array(HEAPU8.buffer, buf, buf_size)
   });
   c.dec.decode(chunk);
 })
@@ -267,7 +269,7 @@ EM_JS(int, wcdec_push_audio, (int wc_ctx, int buf, int buf_size, int key, u64 ts
 	const chunk = new EncodedAudioChunk({
 		timestamp: Number(ts),
 		type: key ? "key" : "delta",
-		data: new Uint8Array(libgpac.HEAPU8.buffer, buf, buf_size)
+		data: new Uint8Array(HEAPU8.buffer, buf, buf_size)
   });
   c.dec.decode(chunk);
 })
@@ -277,7 +279,7 @@ EM_JS(int, wcdec_copy_frame, (int wc_ctx, int dst_pck, int buf, int buf_size), {
 	if (!c || !c._frame || !dst_pck) return;
 
 	//setup dst
-	let ab = new Uint8Array(libgpac.HEAPU8.buffer, buf, buf_size);
+	let ab = new Uint8Array(HEAPU8.buffer, buf, buf_size);
 	let frame = c._frame;
 	c._frame = null;
 	frame.copyTo(ab).then( layout => {
@@ -381,7 +383,7 @@ EM_JS(int, wcdec_copy_audio, (int wc_ctx, int buf, int buf_size, int plane_index
 	if (!c || !c._frame) return;
 
 	//setup dst
-	let ab = new Uint8Array(libgpac.HEAPU8.buffer, buf, buf_size);
+	let ab = new Uint8Array(HEAPU8.buffer, buf, buf_size);
 	c._frame.copyTo(ab, { planeIndex: plane_index });
 })
 
@@ -618,6 +620,17 @@ static GF_FilterCapability WCDecCapsA[] =
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_CODECID, GF_CODECID_RAW),
 };
 
+#else
+static GF_Err wcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove)
+{
+	return GF_NOT_SUPPORTED;
+}
+static GF_Err wcdec_process(GF_Filter *filter)
+{
+	return GF_NOT_SUPPORTED;
+}
+#endif
+
 static GF_FilterCapability WCDecCapsAV[] =
 {
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
@@ -635,7 +648,7 @@ static GF_FilterCapability WCDecCapsAV[] =
 
 static const GF_FilterArgs WCDecArgs[] =
 {
-	{ OFFS(queued), "Maximum number of packets to queue in webcodec instance", GF_PROP_UINT, "10", NULL, 0},
+	{ OFFS(queued), "maximum number of packets to queue in webcodec instance", GF_PROP_UINT, "10", NULL, 0},
 	{0}
 };
 
@@ -647,16 +660,22 @@ GF_FilterRegister GF_WCDecCtxRegister = {
 	SETCAPS(WCDecCapsAV),
 	.flags = GF_FS_REG_SINGLE_THREAD|GF_FS_REG_ASYNC_BLOCK,
 	.private_size = sizeof(GF_WCDecCtx),
+#if defined(GPAC_CONFIG_EMSCRIPTEN)
 	.initialize = wcdec_initialize,
 	.finalize = wcdec_finalize,
+	.process_event = wcdec_process_event,
+#endif
 	.configure_pid = wcdec_configure_pid,
 	.process = wcdec_process,
-	.process_event = wcdec_process_event,
+	.hint_class_type = GF_FS_CLASS_DECODER
 };
 
+#endif //GPAC_DISABLE_WEBCODEC
 
 const GF_FilterRegister *wcdec_register(GF_FilterSession *session)
 {
+#ifndef GPAC_DISABLE_WEBCODEC
+#if defined(GPAC_CONFIG_EMSCRIPTEN)
 	int has_webv_decode = EM_ASM_INT({
 		if (typeof VideoDecoder == 'undefined') return 0;
 		return 1;
@@ -678,6 +697,13 @@ const GF_FilterRegister *wcdec_register(GF_FilterSession *session)
 		GF_WCDecCtxRegister.nb_caps = sizeof(WCDecCapsV)/sizeof(GF_FilterCapability);
 	}
 	GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[WebDec] AudioDecoder %d - VideoDecoder %d\n", has_weba_decode, has_webv_decode));
-	return &GF_WCDecCtxRegister;
-}
+#else
+	if (!gf_opts_get_bool("temp", "gendoc"))
+		return NULL;
+	GF_WCDecCtxRegister.version = "! Warning: WebCodec NOT AVAILABLE IN THIS BUILD !";
 #endif
+	return &GF_WCDecCtxRegister;
+#else
+	return NULL;
+#endif
+}
