@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2023
+ *			Copyright (c) Telecom ParisTech 2017-2025
  *					All rights reserved
  *
  *  This file is part of GPAC / ffmpeg decode filter
@@ -241,11 +241,11 @@ static GF_Err ffdec_process_video(GF_Filter *filter, struct _gf_ffdec_ctx *ctx)
 		gf_filter_pid_drop_packet(ctx->in_pid);
 		return GF_OK;
 	}
-    //we don't own the codec and we're in end of stream, don't try to decode (the context might have been closed)
-    if (!pck && !ctx->owns_context) {
-        gf_filter_pid_set_eos(ctx->out_pid);
-        return GF_EOS;
-    }
+	//we don't own the codec and we're in end of stream, don't try to decode (the context might have been closed)
+	if (!pck && !ctx->owns_context) {
+		gf_filter_pid_set_eos(ctx->out_pid);
+		return GF_EOS;
+	}
 
 restart:
 
@@ -451,9 +451,9 @@ restart:
 		gf_filter_pck_set_sap(dst_pck, GF_FILTER_SAP_1);
 	}
 
-    //rewrite dts and pts to PTS value
-    gf_filter_pck_set_dts(dst_pck, out_cts);
-    gf_filter_pck_set_cts(dst_pck, out_cts);
+	//rewrite dts and pts to PTS value
+	gf_filter_pck_set_dts(dst_pck, out_cts);
+	gf_filter_pck_set_cts(dst_pck, out_cts);
 
 	ff_pfmt = ctx->decoder->pix_fmt;
 	if (ff_pfmt==AV_PIX_FMT_YUVJ420P) {
@@ -479,6 +479,11 @@ restart:
 		dst_stride[0] = 4*ctx->width;
 		pix_out = AV_PIX_FMT_RGBA;
 		break;
+	case GF_PIXEL_BGRA:
+		dst_planes[0] =  (uint8_t *)out_buffer;
+		dst_stride[0] = 4*ctx->width;
+		pix_out = AV_PIX_FMT_BGRA;
+		break;
 	case GF_PIXEL_YUV:
 	case GF_PIXEL_YUV_10:
 		dst_planes[0] =  (uint8_t *)out_buffer;
@@ -535,8 +540,13 @@ restart:
 
 	gf_filter_pck_set_seek_flag(dst_pck, GF_FALSE);
 
+#if (LIBAVFORMAT_VERSION_MAJOR < 62)
 	if (frame->interlaced_frame)
 		gf_filter_pck_set_interlaced(dst_pck, frame->top_field_first ? 2 : 1);
+#else
+	if (frame->flags & AV_FRAME_FLAG_INTERLACED)
+		gf_filter_pck_set_interlaced(dst_pck, frame->flags & AV_FRAME_FLAG_TOP_FIELD_FIRST ? 2 : 1);
+#endif
 
 	gf_filter_pck_send(dst_pck);
 
@@ -1119,9 +1129,9 @@ static GF_Err ffdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	else {
 		u32 codec_id, ff_codectag=0;
 
-        if (!ctx->owns_context) {
-            ctx->decoder = NULL;
-        }
+		if (!ctx->owns_context) {
+			ctx->decoder = NULL;
+		}
 		if (ctx->decoder) {
 			codec_id = ffmpeg_codecid_from_gpac(gpac_codecid, NULL);
 			//same codec, same config, don't reinit
@@ -1136,7 +1146,7 @@ static GF_Err ffdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 				}
 			}
 
-            
+
 			//we could further optimize by detecting we have the same codecid and injecting the extradata
 			//but this is not 100% reliable, and will require parsing AVC/HEVC config
 			//since this seems to work properly with decoder close/open, we keep it as is
@@ -1345,7 +1355,7 @@ reuse_codec_context:
 		}
 		//if SAR is given ignore sar detection
 		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_SAR);
-		if (prop) {
+		if (prop && (prop->value.frac.num>0)) {
 			ctx->sar.num = 0;
 			ctx->sar.den = 0;
 		} else if (ctx->sar.num) {
@@ -1502,19 +1512,20 @@ static const GF_FilterCapability FFDecodeCaps[] =
 GF_FilterRegister FFDecodeRegister = {
 	.name = "ffdec",
 	.version = LIBAVCODEC_IDENT,
-	GF_FS_SET_DESCRIPTION("FFMPEG decoder")
-	GF_FS_SET_HELP("This filter decodes audio and video streams using FFMPEG.\n"
-	"See FFMPEG documentation (https://ffmpeg.org/documentation.html) for more details.\n"
+	GF_FS_SET_DESCRIPTION("FFmpeg decoder")
+	GF_FS_SET_HELP("This filter decodes audio and video streams using FFmpeg.\n"
+	"See FFmpeg documentation (https://ffmpeg.org/documentation.html) for more details.\n"
 	"To list all supported decoders for your GPAC build, use `gpac -h ffdec:*`.\n"
 	"\n"
 	"Options can be passed from prompt using `--OPT=VAL`\n"
+	"Decoder flags can be passed directly as `:FLAGNAME`.\n"
 	"The default threading mode is to let libavcodec decide how many threads to use. To enforce single thread, use `--threads=1`\n"
 	"\n"
 	"# Codec Map\n"
-	"The [-ffcmap]() option allows specifying FFMPEG codecs for codecs not supported by GPAC.\n"
+	"The [-ffcmap]() option allows specifying FFmpeg codecs for codecs not supported by GPAC.\n"
 	"Each entry in the list is formatted as `GID@name` or `GID@+name`, with:\n"
 	"- GID: 4CC or 32 bit identifier of codec ID, as indicated by `gpac -i source inspect:full`\n"
-	"- name: FFMPEG codec name\n"
+	"- name: FFmpeg codec name\n"
 	"- `+': is set and extra data is set and formatted as an ISOBMFF box, removes box header\n"
 	"\n"
 	"EX gpac -i source.mp4 --ffcmap=BKV1@binkvideo vout\n"
@@ -1531,8 +1542,8 @@ GF_FilterRegister FFDecodeRegister = {
 	.flags = GF_FS_REG_META|GF_FS_REG_BLOCK_MAIN,
 	//use middle priorty, so that hardware decs/other native impl in gpac can take over if needed
 	//don't use lowest one since we use this for scalable codecs
-	.priority = 128
-
+	.priority = 128,
+	.hint_class_type = GF_FS_CLASS_DECODER
 };
 
 
