@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2018-2023
+ *			Copyright (c) Telecom ParisTech 2018-2025
  *					All rights reserved
  *
  *  This file is part of GPAC / ffmpeg encode filter
@@ -47,6 +47,7 @@ typedef struct _gf_ffenc_ctx
 	char *c;
 	Bool ls, rld;
 	u32 pfmt;
+	s32 round;
 	GF_Fraction fintra;
 	Bool rc;
 
@@ -66,8 +67,9 @@ typedef struct _gf_ffenc_ctx
 
 	u32 nb_frames_out, nb_frames_in;
 	u64 time_spent;
+	u64 orig_cts_plus_one;
 
-	Bool low_delay;
+	u32 low_delay_mode;
 
 	GF_Err (*process)(GF_Filter *filter, struct _gf_ffenc_ctx *ctx);
 	//gpac one
@@ -76,6 +78,8 @@ typedef struct _gf_ffenc_ctx
 	u32 flush_done;
 	//frame used by both video and audio encoder
 	AVFrame *frame;
+	//ffmpeg codecid forced by output
+	u32 forced_ffcid;
 
 	//encoding buffer - we allocate ENC_BUF_ALLOC_SAFE+WxH for the video (some image codecs in ffmpeg require more than WxH for headers), ENC_BUF_ALLOC_SAFE+nb_ch*samplerate for the audio
 	//this should be enough to hold any lossless compression formats
@@ -112,6 +116,7 @@ typedef struct _gf_ffenc_ctx
 
 	GF_BitStream *sdbs;
 
+	GF_FilterPacket *reconfig_from_pck;
 	Bool reconfig_pending;
 	Bool infmt_negotiate;
 	Bool remap_ts;
@@ -193,6 +198,7 @@ static GF_Err ffenc_initialize(GF_Filter *filter)
 
 	if (!ctx->c) return GF_OK;
 
+	ctx->forced_ffcid = AV_CODEC_ID_NONE;
 	//first look by name, to handle cases such as "aac" vs "vo_aacenc"
 	ctx->force_codec = avcodec_find_encoder_by_name(ctx->c);
 	if (ctx->force_codec) {
@@ -264,13 +270,19 @@ static void ffenc_copy_pid_props(GF_FFEncodeCtx *ctx)
 		}
 	} else {
 		gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_CODECID, &PROP_UINT(ctx->codecid) );
+		if (ctx->codecid==GF_CODECID_FFMPEG) {
+			if (ctx->encoder)
+				gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_META_DEMUX_CODEC_ID, &PROP_UINT(ctx->encoder->codec->id) );
+			else if (ctx->forced_ffcid)
+				gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_META_DEMUX_CODEC_ID, &PROP_UINT(ctx->forced_ffcid) );
+		}
 	}
 	gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_ISOM_SUBTYPE, NULL);
 	gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_PROFILE_LEVEL, NULL);
 
 	ctx->gen_dsi = GF_FALSE;
 	switch (ctx->codecid) {
-	//reframe all these codecs for proper ISOBMFF+DSI formating
+	//reframe all these codecs for proper ISOBMFF+DSI formatting
 	case GF_CODECID_AVC:
 	case GF_CODECID_HEVC:
 	case GF_CODECID_VVC:
@@ -334,6 +346,17 @@ static void ffenc_copy_pid_props(GF_FFEncodeCtx *ctx)
 }
 
 
+static GFINLINE void ffenc_set_deps(GF_FilterPacket *dst_pck, AVPacket *pkt)
+{
+	//reset dependency flags to unknown
+	u8 flags = 0;
+#if LIBAVCODEC_VERSION_MAJOR >= 58
+	if (pkt->flags & AV_PKT_FLAG_DISPOSABLE) {
+		flags = 0x8;
+	}
+#endif
+	gf_filter_pck_set_dependency_flags(dst_pck, flags);
+}
 
 static u64 ffenc_get_cts(GF_FFEncodeCtx *ctx, GF_FilterPacket *pck)
 {
@@ -496,9 +519,25 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		force_intra = 2;
 	}
 
+	//don't repeat encoder reconfiguration if we already forced one
+	if (force_intra == 2) {
+		if (ctx->reconfig_from_pck == pck) {
+			force_intra = 1;
+			ctx->reconfig_from_pck = NULL;
+		} else {
+			ctx->reconfig_from_pck = pck;
+		}
+	}
+
 	//check if we need to force a closed gop
 	if (pck && (ctx->fintra.den && (ctx->fintra.num>0)) && !ctx->force_reconfig) {
 		u64 cts = ffenc_get_cts(ctx, pck);
+		//if we have a first encoded frame, use it as our anchor point and reset
+		//we only do this on first setup
+		if (ctx->orig_cts_plus_one) {
+			cts = ctx->orig_cts_plus_one - 1;
+			ctx->orig_cts_plus_one = 0;
+		}
 		if (!ctx->fintra_setup) {
 			ctx->fintra_setup = GF_TRUE;
 			ctx->orig_ts = cts;
@@ -518,8 +557,13 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 			}
 		}
 	}
-	if (!ctx->nb_frames_in)
+
+	if (!ctx->nb_frames_in) {
 		force_intra = 0;
+		//remember timing of first encoded frame
+		if (pck)
+			ctx->orig_cts_plus_one = ffenc_get_cts(ctx, pck) + 1;
+	}
 
 	if (force_intra) {
 		if (ctx->args_updated) {
@@ -588,12 +632,25 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		}
 		if (pck) {
 			ilaced = gf_filter_pck_get_interlaced(pck);
+#if (LIBAVFORMAT_VERSION_MAJOR < 62)
 			if (!ilaced) {
 				ctx->frame->interlaced_frame = 0;
 			} else {
 				ctx->frame->interlaced_frame = 1;
 				ctx->frame->top_field_first = (ilaced==2) ? 1 : 0;
 			}
+#else
+			if (!ilaced) {
+				ctx->frame->flags &= ~AV_FRAME_FLAG_INTERLACED;
+			} else {
+				ctx->frame->flags |= AV_FRAME_FLAG_INTERLACED;
+				if (ilaced==2) {
+					ctx->frame->flags |= AV_FRAME_FLAG_TOP_FIELD_FIRST;
+				} else {
+					ctx->frame->flags &= ~AV_FRAME_FLAG_TOP_FIELD_FIRST;
+				}
+			}
+#endif
 			ctx->frame->pts = ffenc_get_cts(ctx, pck);
 			ctx->frame->_avf_dur = gf_filter_pck_get_duration(pck);
 		}
@@ -706,7 +763,7 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 				ctx->reconfig_pending = GF_FALSE;
 				ctx->force_reconfig = GF_FALSE;
 				GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[FFEnc] codec flush done, triggering reconfiguration\n"));
-				avcodec_close(ctx->encoder);
+				avcodec_free_context(&ctx->encoder);
 				ctx->encoder = NULL;
 				ctx->setup_failed = 0;
 				e = ffenc_configure_pid_ex(filter, ctx->in_pid, GF_FALSE, GF_TRUE);
@@ -739,25 +796,6 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		return GF_OK;
 	}
 
-	ctx->nb_frames_out++;
-	if (ctx->init_cts_setup) {
-		ctx->init_cts_setup = GF_FALSE;
-		if (ctx->frame->pts != pkt->pts) {
-			//check shift in PTS - most of the time this is 0 (ffmpeg does not restamp video pts)
-			ctx->ts_shift = (s64) ctx->cts_first_frame_plus_one - 1 - (s64) pkt->pts;
-
-			//check shift in DTS
-			ctx->ts_shift += (s64) ctx->cts_first_frame_plus_one - 1 - (s64) pkt->dts;
-		}
-
-		//if ts_shift>0, this means we have a skip
-		if (ctx->ts_shift) {
-			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_DELAY, &PROP_LONGSINT( -ctx->ts_shift ) );
-		} else {
-			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_DELAY, NULL);
-		}
-	}
-
 	src_pck = NULL;
 	count = gf_list_count(ctx->src_packets);
 	for (i=0; i<count; i++) {
@@ -772,6 +810,30 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		src_pck = NULL;
 	}
 
+	ctx->nb_frames_out++;
+	if (ctx->init_cts_setup) {
+		ctx->init_cts_setup = GF_FALSE;
+		if (ctx->frame->pts != pkt->pts) {
+			//first frame out is not first frame in (SAP 2)
+			if (src_pck && (gf_list_find(ctx->src_packets, src_pck)>0) && (pkt->dts<0)) {
+				ctx->ts_shift = - (s64) pkt->dts;
+			} else {
+				//check shift in PTS - most of the time this is 0 (ffmpeg does not restamp video pts)
+				ctx->ts_shift = (s64) ctx->cts_first_frame_plus_one - 1 - (s64) pkt->pts;
+
+				//check shift in DTS
+				ctx->ts_shift += (s64) ctx->cts_first_frame_plus_one - 1 - (s64) pkt->dts;
+			}
+		}
+
+		//if ts_shift>0, this means we have a skip
+		if (ctx->ts_shift) {
+			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_DELAY, &PROP_LONGSINT( -ctx->ts_shift ) );
+		} else {
+			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_DELAY, NULL);
+		}
+	}
+
 	offset = 0;
 	to_copy = size = pkt->size;
 
@@ -889,16 +951,13 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 	//since we send the output to our reframers we should be fine
 	if (pkt->flags & AV_PKT_FLAG_KEY) {
 		gf_filter_pck_set_sap(dst_pck, GF_FILTER_SAP_1);
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[FFEnc] frame %d is SAP\n", ctx->nb_frames_out));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[FFEnc] frame %d is SAP\n", ctx->nb_frames_out-1));
 	}
 	else
 		gf_filter_pck_set_sap(dst_pck, 0);
 
-#if LIBAVCODEC_VERSION_MAJOR >= 58
-	if (pkt->flags & AV_PKT_FLAG_DISPOSABLE) {
-		gf_filter_pck_set_dependency_flags(dst_pck, 0x8);
-	}
-#endif
+	ffenc_set_deps(dst_pck, pkt);
+
 	gf_filter_pck_send(dst_pck);
 
 	av_packet_free_side_data(pkt);
@@ -1335,6 +1394,7 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		gf_filter_pck_set_sap(dst_pck, 0);
 
 	gf_filter_pck_set_duration(dst_pck, (u32) pkt->duration);
+	ffenc_set_deps(dst_pck, pkt);
 
 	gf_filter_pck_send(dst_pck);
 
@@ -1400,6 +1460,11 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 		prop = gf_filter_pid_caps_query(pid, GF_PROP_PID_CODECID);
 		if (prop) {
 			ctx->codecid = prop->value.uint;
+			if (ctx->codecid==GF_CODECID_FFMPEG) {
+				prop = gf_filter_pid_caps_query(pid, GF_PROP_PID_META_DEMUX_CODEC_ID);
+				if (!prop) return GF_NOT_SUPPORTED;
+				ctx->forced_ffcid = prop->value.uint;
+			}
 		} else if (!ctx->codecid && ctx->c) {
 			ctx->codecid = gf_codecid_parse(ctx->c);
 			if (!ctx->codecid) {
@@ -1414,6 +1479,13 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 	}
 
 	if (!ctx->codecid && !desired_codec) {
+		if (gf_filter_is_dynamic(filter)) {
+			ctx->in_pid = pid;
+			if (!ctx->out_pid) {
+				ctx->out_pid = gf_filter_pid_new(filter);
+			}
+			return GF_OK;
+		}
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] No codecid specified\n" ));
 		return GF_BAD_PARAM;
 	}
@@ -1432,7 +1504,10 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 	}
 
 	if (ctx->codecid) {
-		codec_id = ffmpeg_codecid_from_gpac(ctx->codecid, &ff_codectag);
+		if (ctx->forced_ffcid)
+			codec_id = ctx->forced_ffcid;
+		else
+			codec_id = ffmpeg_codecid_from_gpac(ctx->codecid, &ff_codectag);
 		if (codec_id) {
 			if (desired_codec && desired_codec->id==codec_id)
 				codec = desired_codec;
@@ -1664,6 +1739,26 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			ctx->infmt_negotiate = GF_TRUE;
 		} else {
 			ctx->infmt_negotiate = GF_FALSE;
+			u32 downsample_w=0, downsample_h=0;
+			if ((ctx->round==1) || (ctx->round==-1))
+				gf_pixel_get_downsampling(ffmpeg_pixfmt_to_gpac(ctx->pixel_fmt, GF_FALSE), &downsample_w, &downsample_h);
+			else if (ctx->round>0)
+				downsample_h = downsample_w = ctx->round;
+			else if (ctx->round<0)
+				downsample_h = downsample_w = (u32) -ctx->round;
+
+			if (downsample_w && (ctx->width % downsample_w)) {
+				u32 w = (ctx->width/downsample_w) * downsample_w;
+				if (ctx->round>0) w+=downsample_w;
+				gf_filter_pid_negotiate_property(ctx->in_pid, GF_PROP_PID_WIDTH, &PROP_UINT(w) );
+				ctx->infmt_negotiate = GF_TRUE;
+			}
+			if (downsample_h && (ctx->height % downsample_h)) {
+				u32 h = (ctx->height/downsample_h) * downsample_h;
+				if (ctx->round>0) h+=downsample_h;
+				gf_filter_pid_negotiate_property(ctx->in_pid, GF_PROP_PID_HEIGHT, &PROP_UINT(h) );
+				ctx->infmt_negotiate = GF_TRUE;
+			}
 		}
 	} else {
 		u32 change_input_sr = 0;
@@ -1779,7 +1874,7 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 		ctx->encoder->width = ctx->width;
 		ctx->encoder->height = ctx->height;
 		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_SAR);
-		if (prop) {
+		if (prop && (prop->value.frac.num>0)) {
 			ctx->encoder->sample_aspect_ratio.num = prop->value.frac.num;
 			ctx->encoder->sample_aspect_ratio.den = prop->value.frac.den;
 		} else {
@@ -1845,12 +1940,16 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			}
 		}
 
-		if (ctx->low_delay) {
+		if (ctx->low_delay_mode==1) {
 			av_dict_set(&ctx->options, "profile", "baseline", 0);
 			av_dict_set(&ctx->options, "preset", "ultrafast", 0);
 			av_dict_set(&ctx->options, "tune", "zerolatency", 0);
 			if (ctx->codecid==GF_CODECID_AVC) {
-				av_dict_set(&ctx->options, "x264opts", "no-mbtree:sliced-threads:sync-lookahead=0", 0);
+				if (av_opt_find((void*)&codec->priv_class, "x264-params", NULL, 0, 0) != NULL) {
+					av_dict_set(&ctx->options, "x264-params", "no-mbtree=1:sliced-threads=1:sync-lookahead=0", 0);
+				} else {
+					av_dict_set(&ctx->options, "x264opts", "no-mbtree:sliced-threads:sync-lookahead=0", 0);
+				}
 			}
 #if LIBAVCODEC_VERSION_MAJOR >= 58
 			ctx->encoder->flags |= AV_CODEC_FLAG_LOW_DELAY;
@@ -2101,15 +2200,26 @@ static GF_Err ffenc_update_arg(GF_Filter *filter, const char *arg_name, const GF
 
 	if (!strcmp(arg_name, "global_header"))	return GF_OK;
 	else if (!strcmp(arg_name, "local_header"))	return GF_OK;
-	else if (!strcmp(arg_name, "low_delay"))	ctx->low_delay = GF_TRUE;
+	//activate opts for low delay
+	else if (!strcmp(arg_name, "flags")
+		&& arg_val && arg_val->value.string && strstr(arg_val->value.string, "low_delay")
+		&& !ctx->low_delay_mode
+	)
+		ctx->low_delay_mode = 1;
+	//activate opts for low delay
+	else if (!strcmp(arg_name, "low_delay")) {
+		ctx->low_delay_mode = 1;
+		gf_filter_report_meta_option(filter, "low_delay", 1, NULL);
+	}
 	//remap some options
 	else if (!strcmp(arg_name, "bitrate") || !strcmp(arg_name, "rate"))	arg_name = "b";
 //	else if (!strcmp(arg_name, "gop")) arg_name = "g";
 	//disable low delay if these options are set
-	else if (!strcmp(arg_name, "x264opts")) ctx->low_delay = GF_FALSE;
-	else if (!strcmp(arg_name, "profile")) ctx->low_delay = GF_FALSE;
-	else if (!strcmp(arg_name, "preset")) ctx->low_delay = GF_FALSE;
-	else if (!strcmp(arg_name, "tune")) ctx->low_delay = GF_FALSE;
+	else if (!strcmp(arg_name, "x264opts")) ctx->low_delay_mode = 2;
+	else if (!strcmp(arg_name, "x264-params")) ctx->low_delay_mode = 2;
+	else if (!strcmp(arg_name, "profile")) ctx->low_delay_mode = 2;
+	else if (!strcmp(arg_name, "preset")) ctx->low_delay_mode = 2;
+	else if (!strcmp(arg_name, "tune")) ctx->low_delay_mode = 2;
 
 	if (!strcmp(arg_name, "g") || !strcmp(arg_name, "gop"))
 		ctx->gop_size = arg_val->value.string ? atoi(arg_val->value.string) : 25;
@@ -2175,50 +2285,90 @@ static GF_Err ffenc_update_arg(GF_Filter *filter, const char *arg_name, const GF
 static Bool ffenc_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 {
 	GF_FFEncodeCtx *ctx = gf_filter_get_udta(filter);
-	if (evt->base.type==GF_FEVT_ENCODE_HINTS) {
-		if (evt->encode_hints.gen_dsi_only) {
+	if (evt->base.type==GF_FEVT_TRANSPORT_HINTS) {
+		if (evt->transport_hints.flags & GF_TRANSPORT_HINTS_SAW_ENCODER) {
+			// this is a pass-through event, ignore it
+			return GF_FALSE;
+		}
+
+		if (evt->transport_hints.gen_dsi_only) {
 			ctx->generate_dsi_only = GF_TRUE;
 		}
-		else if ((ctx->fintra.num<0) && evt->encode_hints.intra_period.den && evt->encode_hints.intra_period.num) {
-			ctx->fintra = evt->encode_hints.intra_period;
+		//change in fintra
+		else if (ctx->fintra.num * evt->transport_hints.seg_duration.den != ctx->fintra.den * evt->transport_hints.seg_duration.num) {
+			ctx->fintra = evt->transport_hints.seg_duration;
+			ctx->fintra_setup = GF_FALSE;
 
 			if (!ctx->rc || (gf_list_count(ctx->src_packets) && !ctx->force_reconfig)) {
 				ctx->reconfig_pending = GF_TRUE;
 				ctx->force_reconfig = GF_TRUE;
 			}
 		}
+
+		//send the event upstream (in case any other filter is interested in it)
+		GF_FilterEvent new_evt = *evt;
+		new_evt.base.on_pid = ctx->in_pid;
+		new_evt.transport_hints.flags |= GF_TRANSPORT_HINTS_SAW_ENCODER;
+		gf_filter_pid_send_event(ctx->in_pid, &new_evt);
 		return GF_TRUE;
 	}
+	else if (evt->base.type==GF_FEVT_STOP) {
+		ctx->nb_frames_in = ctx->nb_frames_out = 0;
+	}
 	return GF_FALSE;
 }
 
+static GF_Err ffenc_reconfigure_output(GF_Filter *filter, GF_FilterPid *pid)
+{
+	const GF_PropertyValue *p;
+	GF_FFEncodeCtx *ctx = gf_filter_get_udta(filter);
+	if (ctx->out_pid != pid) return GF_BAD_PARAM;
+
+	p = gf_filter_pid_caps_query(pid, GF_PROP_PID_CODECID);
+	if (p) ctx->codecid = p->value.uint;
+
+	ctx->forced_ffcid = AV_CODEC_ID_NONE;
+	if (ctx->codecid==GF_CODECID_FFMPEG) {
+		p = gf_filter_pid_caps_query(pid, GF_PROP_PID_META_DEMUX_CODEC_ID);
+		if (p) ctx->forced_ffcid = p->value.uint;
+	}
+	return ffenc_configure_pid_ex(filter, ctx->in_pid, GF_FALSE, GF_FALSE);
+}
+
 static const GF_FilterCapability FFEncodeCaps[] =
 {
-	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
+	CAP_UINT(GF_CAPS_INPUT_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_RAW),
 	CAP_BOOL(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
 	CAP_UINT(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_RAW),
 	CAP_BOOL(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_TILE_BASE, GF_TRUE),
 	//some video encoding dumps in unframe mode, we declare the pid property at runtime
 	{0},
-	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
+	CAP_UINT(GF_CAPS_INPUT_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
 	CAP_BOOL(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_RAW),
 	CAP_UINT(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_RAW),
+	{0},
+	CAP_UINT(GF_CAPFLAG_RECONFIG, GF_PROP_PID_CODECID, 0),
+	CAP_UINT(GF_CAPFLAG_RECONFIG, GF_PROP_PID_META_DEMUX_CODEC_ID, 0),
 };
 
 GF_FilterRegister FFEncodeRegister = {
 	.name = "ffenc",
 	.version=LIBAVCODEC_IDENT,
-	GF_FS_SET_DESCRIPTION("FFMPEG encoder")
-	GF_FS_SET_HELP("This filter encodes audio and video streams using FFMPEG.\n"
-		"See FFMPEG documentation (https://ffmpeg.org/documentation.html) for more details.\n"
+	GF_FS_SET_DESCRIPTION("FFmpeg encoder")
+	GF_FS_SET_HELP("This filter encodes audio and video streams using FFmpeg.\n"
+		"See FFmpeg documentation (https://ffmpeg.org/documentation.html) for more details.\n"
 		"To list all supported encoders for your GPAC build, use `gpac -h ffenc:*`.\n"
 		"\n"
 		"The filter will try to resolve the codec name in [-c]() against a libavcodec codec name (e.g. `libx264`) and use it if found.\n"
 		"If not found, it will consider the name to be a GPAC codec name and find a codec for it. In that case, if no pixel format is given, codecs will be enumerated to find a matching pixel format.\n"
 		"\n"
 		"Options can be passed from prompt using `--OPT=VAL` (global options) or appending `::OPT=VAL` to the desired encoder filter.\n"
+		"Encoder flags can be passed directly as `:FLAGNAME`.\n"
+		"\n"
+		"Note\n"
+		"Setting the `:low_delay` flag will set by default `profile=baseline`, `preset=ultrafast` and `tune=zerolatency` options as well as `x264-params` for AVC|H264. If one or more of these options are set as filter arguments, no defaulting is used for all these options.\n"
 		"\n"
 		"The filter will look for property `TargetRate` on input PID to set the desired bitrate per PID.\n"
 		"\n"
@@ -2243,9 +2393,11 @@ GF_FilterRegister FFEncodeRegister = {
 	.process = ffenc_process,
 	.process_event = ffenc_process_event,
 	.update_arg = ffenc_update_arg,
+	.reconfigure_output = ffenc_reconfigure_output,
 	.flags = GF_FS_REG_META | GF_FS_REG_TEMP_INIT | GF_FS_REG_BLOCK_MAIN,
 	//use middle priority in case we have other encoders
-	.priority = 128
+	.priority = 128,
+	.hint_class_type = GF_FS_CLASS_ENCODER
 };
 
 #define OFFS(_n)	#_n, offsetof(GF_FFEncodeCtx, _n)
@@ -2259,6 +2411,12 @@ static const GF_FilterArgs FFEncodeArgs[] =
 	{ OFFS(ls), "log stats", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(rc), "reset encoder when forcing intra frame (some encoders might not support intra frame forcing)", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(rld), "force reloading of encoder when arguments are updated", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT|GF_FS_ARG_UPDATE},
+	{ OFFS(round), "round video up or down\n"
+	"- 0: no rounding\n"
+	"- 1: round up to match codec YUF format requirements\n"
+	"- -1: round down to match codec YUF format requirements\n"
+	"- other: round to lower (negative value) or higher (positive value), for example CTU size"
+	, GF_PROP_SINT, "1", NULL, GF_FS_ARG_HINT_EXPERT|GF_FS_ARG_UPDATE},
 
 	{ "*", -1, "any possible options defined for AVCodecContext and sub-classes. see `gpac -hx ffenc` and `gpac -hx ffenc:*`", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_META},
 	{0}
