@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2018-2023
+ *			Copyright (c) Telecom ParisTech 2018-2025
  *					All rights reserved
  *
  *  This file is part of GPAC / MPEG-2 TS mux filter
@@ -80,6 +80,11 @@ enum
 	TEMI_TC64_ALWAYS,
 };
 
+GF_OPT_ENUM (GF_TSMuxInputDescriptorAction,
+	IN_TEMI_DROP=0,
+	IN_TEMI_FWD,
+	IN_TEMI_NTP,
+);
 
 typedef struct
 {
@@ -105,6 +110,7 @@ typedef struct
 	s32 subs_sidx;
 	Bool keepts;
 	GF_Fraction cdur;
+	GF_TSMuxInputDescriptorAction temi_fwd;
 
 	//internal
 	GF_FilterPid *opid;
@@ -121,6 +127,12 @@ typedef struct
 	Bool init_buffering;
 	u32 last_log_time;
 	Bool pmt_update_pending;
+	Bool cdur_overwrite;
+
+	//SCTE-35 data is conveyed by packet properties
+	GF_M2TS_Mux_Stream *scte35_stream;
+	u8 *scte35_payload;
+	u32 scte35_size;
 
 	u32 dash_mode;
 	Bool init_dash;
@@ -133,6 +145,8 @@ typedef struct
 
 	char dash_file_name[GF_MAX_PATH];
 	char idx_file_name[GF_MAX_PATH];
+	char llhas_template[GF_MAX_PATH];
+
 
 	//dash indexing
 	u32 nb_sidx_entries, nb_sidx_alloc;
@@ -149,10 +163,10 @@ typedef struct
 	Double start_range;
 
 	struct __tsmx_pid *ref_pid;
-	Bool wait_llhls_flush;
+	Bool wait_llhas_flush;
 
-	u32 llhls;
-	Bool next_is_llhls_start;
+	u32 llhas_mode;
+	Bool next_is_llhas_start;
 	u32 frag_num;
 
 	u32 frag_offset, frag_size, frag_duration;
@@ -170,9 +184,11 @@ typedef struct
 	u32 id, delay, timescale;
 	u64 offset, init_val;
 	char *url;
-	Bool ntp, use_init_val;
+	u32 ntp_mode;
+	Bool use_init_val;
 	u32 mode_64bits;
 	u64 cts_at_init_val_plus_one;
+	u64 ntp_init_ts, ntp_init_cts;
 } TEMIDesc;
 
 enum
@@ -182,8 +198,8 @@ enum
 	M2TS_EODS_FOUND,
 	//forced end of DASH segment (found empty packet with EODS set)
 	M2TS_EODS_FORCED,
-	//end of LL-HLS part
-	M2TS_EODS_LLHLS,
+	//end of LLHAS part
+	M2TS_EODS_LLHAS,
 };
 
 typedef struct __tsmx_pid
@@ -224,18 +240,18 @@ typedef struct __tsmx_pid
 	u8 *pck_data_buf;
 
 	u32 suspended;
-	u64 llhls_dts_init;
+	u64 llhas_dts_init;
 	Bool is_sparse;
 } M2Pid;
 
-static GF_Err tsmux_format_af_descriptor(GF_BitStream *bs, u32 timeline_id, u64 timecode, u32 timescale, u32 mode_64bits, u64 ntp, const char *temi_url, u32 temi_delay, u32 *last_url_time)
+static GF_Err tsmux_format_af_descriptor(GF_BitStream *bs, Bool is_realtime, u32 timeline_id, u64 timecode, u32 timescale, u32 mode_64bits, u64 ntp, const char *temi_url, u32 temi_delay, u32 *last_url_time, Bool is_reload, Bool is_splicing, GF_Fraction *announce, Bool is_paused, Bool is_discontinuity)
 {
 	u32 len;
-	u32 last_time;
-	if (ntp) {
+	u32 last_time=0;
+	if (ntp && is_realtime) {
 		last_time = 1000*(ntp>>32);
 		last_time += 1000*(ntp&0xFFFFFFFF)/0xFFFFFFFF;
-	} else {
+	} else if (timescale) {
 		last_time = (u32) (1000*timecode/timescale);
 	}
 	if (temi_url && (!*last_url_time || (last_time - *last_url_time + 1 >= temi_delay)) ) {
@@ -246,12 +262,16 @@ static GF_Err tsmux_format_af_descriptor(GF_BitStream *bs, u32 timeline_id, u64
 		gf_bs_write_int(bs,	GF_M2TS_AFDESC_LOCATION_DESCRIPTOR, 8);
 		gf_bs_write_int(bs,	len, 8);
 
-		gf_bs_write_int(bs,	0, 1); //force_reload
-		gf_bs_write_int(bs,	0, 1); //is_announcement
-		gf_bs_write_int(bs,	0, 1); //splicing_flag
-		gf_bs_write_int(bs,	0, 1); //use_base_temi_url
+		gf_bs_write_int(bs,	is_reload, 1); //force_reload
+		gf_bs_write_int(bs,	announce ? 1 : 0, 1); //is_announcement
+		gf_bs_write_int(bs,	is_splicing, 1); //splicing_flag
+		gf_bs_write_int(bs,	temi_url[0] ? 0 : 1, 1); //use_base_temi_url
 		gf_bs_write_int(bs,	0xFF, 5); //reserved
 		gf_bs_write_int(bs,	timeline_id, 7); //timeline_id
+		if (announce) {
+			gf_bs_write_u32(bs, announce->den);
+			gf_bs_write_u32(bs, announce->num);
+		}
 
 		if (strlen(temi_url)) {
 			char *url = (char *)temi_url;
@@ -266,8 +286,8 @@ static GF_Err tsmux_format_af_descriptor(GF_BitStream *bs, u32 timeline_id, u64
 			}
 			gf_bs_write_u8(bs, (u32) strlen(url)); //url_path_len
 			gf_bs_write_data(bs, url, (u32) strlen(url) ); //url
-			gf_bs_write_u8(bs, 0); //nb_addons
 		}
+		gf_bs_write_u8(bs, 0); //nb_addons
 		//rewrite len
 		end = gf_bs_get_position(bs);
 		len = (u32) (end - start - 2);
@@ -287,7 +307,6 @@ static GF_Err tsmux_format_af_descriptor(GF_BitStream *bs, u32 timeline_id, u64
 			while (timecode > 0xFFFFFFFFUL) {
 				timecode -= 0xFFFFFFFFUL;
 			}
-
 		}
 
 		len = 3; //3 bytes flags
@@ -303,9 +322,9 @@ static GF_Err tsmux_format_af_descriptor(GF_BitStream *bs, u32 timeline_id, u64
 		gf_bs_write_int(bs,	ntp ? 1 : 0, 1); //has_ntp
 		gf_bs_write_int(bs,	0, 1); //has_ptp
 		gf_bs_write_int(bs,	0, 2); //has_timecode
-		gf_bs_write_int(bs,	0, 1); //force_reload
-		gf_bs_write_int(bs,	0, 1); //paused
-		gf_bs_write_int(bs,	0, 1); //discontinuity
+		gf_bs_write_int(bs,	is_reload, 1); //force_reload
+		gf_bs_write_int(bs,	is_paused, 1); //paused
+		gf_bs_write_int(bs,	is_discontinuity, 1); //discontinuity
 		gf_bs_write_int(bs,	0xFF, 7); //reserved
 		gf_bs_write_int(bs,	timeline_id, 8); //timeline_id
 		if (timescale) {
@@ -397,6 +416,27 @@ static void tsmux_check_mpd_start_time(GF_TSMuxCtx *ctx, GF_FilterPacket *pck)
 	}
 }
 
+static u32 tsmux_stream_process_scte35(GF_M2TS_Mux *muxer, GF_M2TS_Mux_Stream *stream)
+{
+	if (stream->table_needs_update) { /* generate table payload */
+		GF_TSMuxCtx *ctx = ((M2Pid*)stream->ifce->input_udta)->ctx;
+		gf_m2ts_mux_table_update(stream, GF_M2TS_TABLE_ID_SCTE35_SPLICE_INFO, stream->program->number,
+			ctx->scte35_payload+3, ctx->scte35_size-3, // remove redundancy since payload already contains an entire SCTE 35 splice_info_section
+			GF_FALSE, GF_FALSE);
+		ctx->scte35_size = 0;
+		gf_free(ctx->scte35_payload);
+		ctx->scte35_payload = NULL;
+
+		stream->table_needs_update = GF_FALSE;
+		stream->table_needs_send = GF_TRUE;
+	}
+	if (stream->table_needs_send)
+		return 1;
+	if (stream->refresh_rate_ms)
+		return 1;
+	return 0;
+}
+
 static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 {
 	u32 cversion;
@@ -453,6 +493,7 @@ static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 					tspid->ctx->wait_dash_flush = GF_TRUE;
 				tspid->ctx->dash_seg_num = p->value.uint;
 				tspid->ctx->dash_file_name[0] = 0;
+				tspid->ctx->llhas_template[0] = 0;
 				tsmux_check_mpd_start_time(tspid->ctx, pck);
 			}
 
@@ -467,6 +508,10 @@ static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 						strcpy(tspid->ctx->dash_file_name, p->value.string);
 						tspid->ctx->dash_file_switch = GF_TRUE;
 					}
+					p = gf_filter_pck_get_property(pck, GF_PROP_PCK_LLHAS_TEMPLATE);
+					if (p) {
+						strcpy(tspid->ctx->llhas_template, p->value.string);
+					}
 					return GF_OK;
 				}
 				else if (tspid->ctx->last_is_eods_flush) {
@@ -474,6 +519,9 @@ static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 					if (p2) {
 						strcpy(tspid->ctx->dash_file_name, p2->value.string);
 						tspid->ctx->next_is_start = GF_TRUE;
+						p2 = gf_filter_pck_get_property(pck, GF_PROP_PCK_LLHAS_TEMPLATE);
+						if (p2)
+							strcpy(tspid->ctx->llhas_template, p2->value.string);
 					}
 				}
 			}
@@ -525,20 +573,20 @@ static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 		if (tspid->ctx->ref_pid) {
 			dts = gf_filter_pck_get_dts(pck);
 
-			if (!tspid->llhls_dts_init) {
-				tspid->llhls_dts_init = dts+1;
+			if (!tspid->llhas_dts_init) {
+				tspid->llhas_dts_init = dts+1;
 				if (tspid->ctx->ref_pid == tspid) {
 					if (gf_filter_pck_get_sap(pck))
 						tspid->ctx->frag_has_intra = GF_TRUE;
 				}
 			} else {
-				u64 dur = dts - tspid->llhls_dts_init + 1;
+				u64 dur = dts - tspid->llhas_dts_init + 1;
 				if (gf_timestamp_greater_or_equal(dur, ifce->timescale, tspid->ctx->cdur.num, tspid->ctx->cdur.den)) {
 					if (tspid->ctx->ref_pid == tspid) {
-						tspid->ctx->wait_llhls_flush = GF_TRUE;
+						tspid->ctx->wait_llhas_flush = GF_TRUE;
 						tspid->ctx->frag_duration = (u32) dur;
 					}
-					tspid->has_seen_eods = M2TS_EODS_LLHLS;
+					tspid->has_seen_eods = M2TS_EODS_LLHAS;
 					return GF_OK;
 				}
 				if (tspid->ctx->ref_pid == tspid) {
@@ -578,12 +626,13 @@ static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 			es_pck.cts = 0;
 		}
 
+		//prepare for adaptation field descriptors
+		if (!tspid->temi_af_bs) tspid->temi_af_bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+		else gf_bs_reassign_buffer(tspid->temi_af_bs, tspid->af_data, tspid->af_data_alloc);
+
 		if (tspid->temi_descs) {
 			u32 i, count=gf_list_count(tspid->temi_descs);
 
-			if (!tspid->temi_af_bs) tspid->temi_af_bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
-			else gf_bs_reassign_buffer(tspid->temi_af_bs, tspid->af_data, tspid->af_data_alloc);
-
 			for (i=0; i<count; i++) {
 				TEMIDesc *temi = gf_list_get(tspid->temi_descs, i);
 				u64 ntp=0;
@@ -620,20 +669,105 @@ static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 						tc += temi->offset;
 				}
 
-				if (temi->ntp) {
-					u32 sec, frac;
-					gf_net_get_ntp(&sec, &frac);
-					ntp = sec;
-					ntp <<= 32;
-					ntp |= frac;
+				if (temi->ntp_mode==1) {
+					ntp = gf_net_get_ntp_ts();
 				}
-				tsmux_format_af_descriptor(tspid->temi_af_bs, temi->id, tc, timescale, temi->mode_64bits, ntp, temi->url, temi->delay, &tspid->last_temi_url);
+				else if (temi->ntp_mode==2) {
+					u64 pck_ts = 0;
+					if ((s64) cts + tspid->media_delay >= 0)
+						pck_ts = gf_timestamp_rescale(cts + tspid->media_delay, ifce->timescale, 1000000);
+
+					if (!temi->ntp_init_ts) {
+						temi->ntp_init_ts = gf_net_get_ntp_ts();
+						temi->ntp_init_cts = pck_ts;
+					}
+
+					s64 diff_usec = pck_ts;
+					diff_usec -= temi->ntp_init_cts;
+					ntp = gf_net_ntp_add_usec(temi->ntp_init_ts, (s32) diff_usec);
+				}
+				tsmux_format_af_descriptor(tspid->temi_af_bs, tspid->ctx->realtime, temi->id, tc, timescale, temi->mode_64bits, ntp, temi->url, temi->delay, &tspid->last_temi_url, GF_FALSE, GF_FALSE, NULL, GF_FALSE, GF_FALSE);
 			}
-			gf_bs_get_content_no_truncate(tspid->temi_af_bs, &tspid->af_data, &es_pck.mpeg2_af_descriptors_size, &tspid->af_data_alloc);
-			es_pck.mpeg2_af_descriptors = tspid->af_data;
 		}
+
+		u32 pidx = 0;
+		//rewrite temi descriptors present in source
+		while (tspid->ctx->temi_fwd) {
+			u32 p4cc;
+			const char *pname;
+			const GF_PropertyValue *p = gf_filter_pck_enum_properties(pck, &pidx, &p4cc, &pname);
+			if (!p) break;
+			if (pname && !strncmp(pname, "temi_l", 6)) {
+				GF_BitStream *bs = gf_bs_new(p->value.data.ptr, p->value.data.size, GF_BITSTREAM_READ);
+				while (1) {
+					u8 achar =  gf_bs_read_u8(bs);
+					if (!achar) break;
+				}
+				GF_Fraction time;
+				u32 timeline_id  = atoi(pname+7);
+				const char *temi_url = p->value.data.ptr;
+				u8 is_announce = gf_bs_read_int(bs, 1);
+				u8 is_splicing = gf_bs_read_int(bs, 1);
+				u8 is_reload = gf_bs_read_int(bs, 1);
+				gf_bs_read_int(bs, 5);
+				if (is_announce) {
+					time.den = gf_bs_read_u32(bs);
+					time.num = gf_bs_read_u32(bs);
+				}
+				Bool invalid = gf_bs_is_overflow(bs);
+				gf_bs_del(bs);
+				//rewrite
+				u32 last_t=0;
+				if (!invalid)
+					tsmux_format_af_descriptor(tspid->temi_af_bs, tspid->ctx->realtime, timeline_id, 0, 0, 0, 0, temi_url, 0, &last_t, is_splicing, is_reload, is_announce ? &time : NULL, GF_FALSE, GF_FALSE);
+			}
+			else if (pname && !strncmp(pname, "temi_t", 6)) {
+				u32 timeline_id = atoi(pname+7);
+				GF_BitStream *bs = gf_bs_new(p->value.data.ptr, p->value.data.size, GF_BITSTREAM_READ);
+
+				u32 timescale = gf_bs_read_u32(bs);
+				u64 timestamp = gf_bs_read_u64(bs);
+				/*u64 media_pts = */gf_bs_read_u64(bs);
+				u8 is_reload = gf_bs_read_int(bs, 1);
+				u8 is_paused = gf_bs_read_int(bs, 1);
+				u8 is_discontinuity = gf_bs_read_int(bs, 1);
+				u8 has_ntp = gf_bs_read_int(bs, 1);
+				gf_bs_read_int(bs, 4);
+
+				u64 ntp = 0;
+				if (has_ntp) {
+					ntp = gf_bs_read_u64(bs);
+					if (tspid->ctx->temi_fwd==IN_TEMI_NTP)
+						ntp = gf_net_get_ntp_ts();
+				}
+				Bool invalid = gf_bs_is_overflow(bs);
+				gf_bs_del(bs);
+
+				u32 last_t=0;
+				if (!invalid)
+					tsmux_format_af_descriptor(tspid->temi_af_bs, tspid->ctx->realtime, timeline_id, timestamp, timescale, 0, ntp, NULL, 0, &last_t, GF_FALSE, is_reload, NULL, is_paused, is_discontinuity);
+			}
+		}
+
+		gf_bs_get_content_no_truncate(tspid->temi_af_bs, &tspid->af_data, &es_pck.mpeg2_af_descriptors_size, &tspid->af_data_alloc);
+		if (es_pck.mpeg2_af_descriptors_size)
+			es_pck.mpeg2_af_descriptors = tspid->af_data;
+		else
+			es_pck.mpeg2_af_descriptors = NULL;
+
 		es_pck.cts += tspid->max_media_skip + tspid->media_delay;
 
+		p = gf_filter_pck_get_property_str(pck, "scte35");
+		if (p) {
+			gf_assert(tspid->ctx->scte35_stream);
+			gf_assert(!tspid->ctx->scte35_payload);
+			tspid->ctx->scte35_payload = gf_malloc(p->value.data.size);
+			memcpy(tspid->ctx->scte35_payload, p->value.data.ptr, p->value.data.size);
+			tspid->ctx->scte35_size = p->value.data.size;
+			tspid->ctx->scte35_stream->table_needs_update = GF_TRUE;
+			tspid->ctx->scte35_stream->table_needs_send = GF_TRUE;
+		}
+
 		if (tspid->nb_repeat_last) {
 			es_pck.cts += tspid->nb_repeat_last * ifce->timescale * tspid->ctx->repeat_img / 1000;
 		}
@@ -656,11 +790,10 @@ static GF_Err tsmux_esi_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 				//we don't have reliable dts - double the diff should make sure we don't try to adjust too often
 				diff = cts_diff = 2*(es_pck.dts - es_pck.cts);
 				diff = gf_timestamp_rescale(diff, tspid->esi.timescale, 1000000);
-				gf_assert(tspid->prog->cts_offset <= diff);
+
+				GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[M2TSMux] Packet CTS "LLU" is less than packet DTS "LLU", adjusting all CTS by %d / %d (prev offset "LLU" us)\n", es_pck.cts, es_pck.dts, cts_diff, tspid->esi.timescale, tspid->prog->cts_offset));
+
 				tspid->prog->cts_offset += (u32) diff;
-
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[M2TSMux] Packet CTS "LLU" is less than packet DTS "LLU", adjusting all CTS by %d / %d!\n", es_pck.cts, es_pck.dts, cts_diff, tspid->esi.timescale));
-
 				es_pck.cts += cts_diff;
 			}
 			if (tspid->esi.stream_type!=GF_STREAM_VISUAL) {
@@ -940,7 +1073,12 @@ static Bool tsmux_setup_esi(GF_TSMuxCtx *ctx, GF_M2TS_Mux_Program *prog, M2Pid *
 	p = gf_filter_pid_get_property_str(tspid->ipid, "M2TSRA");
 	if (p) {
 		if ((p->type==GF_PROP_UINT) || (p->type==GF_PROP_4CC)) m2ts_ra = p->value.uint;
-		else if (p->type==GF_PROP_STRING) m2ts_ra = gf_4cc_parse(p->value.string);
+		else if (p->type==GF_PROP_STRING) {
+			if (!stricmp(p->value.string, "srt"))
+				m2ts_ra = GF_M2TS_RA_STREAM_SRT;
+			else
+				m2ts_ra = gf_4cc_parse(p->value.string);
+		}
 	}
 	if (m2ts_ra != tspid->esi.ra_code) {
 		changed = GF_TRUE;
@@ -977,7 +1115,7 @@ static void tsmux_setup_temi(GF_TSMuxCtx *ctx, M2Pid *tspid)
 		u32 temi_delay=1000;
 		u64 temi_offset=0;
 		u32 temi_timescale=0;
-		Bool temi_ntp=GF_FALSE;
+		u32 temi_ntp=0;
 		u32 temi_64bits=TEMI_TC64_AUTO;
 		Bool temi_use_init_val=GF_FALSE;
 		u64 temi_init_val = 0;
@@ -1001,7 +1139,10 @@ static void tsmux_setup_temi(GF_TSMuxCtx *ctx, M2Pid *tspid)
 				service_id = atoi(temi_cfg+2);
 				break;
 			case 'N':
-				temi_ntp = GF_TRUE;
+				temi_ntp = 1;
+				break;
+			case 'n':
+				temi_ntp = 2;
 				break;
 			case 'D':
 				temi_delay = atoi(temi_cfg+2);
@@ -1064,7 +1205,7 @@ static void tsmux_setup_temi(GF_TSMuxCtx *ctx, M2Pid *tspid)
 				temi->url = gf_strdup(temi_cfg);
 				temi->id = temi_id+1;
 			}
-			temi->ntp = temi_ntp;
+			temi->ntp_mode = temi_ntp;
 			temi->offset = temi_offset;
 			temi->delay = temi_delay;
 			temi->timescale = temi_timescale;
@@ -1133,16 +1274,19 @@ static GF_Err tsmux_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_SERVICE_ID);
 	service_id = p ? p->value.uint : ctx->sid;
 
-	sname = ctx->name;
-	pname = ctx->provider;
-	p = gf_filter_pid_get_property(pid, GF_PROP_PID_SERVICE_NAME);
-	if (p) sname = p->value.string;
-	p = gf_filter_pid_get_property(pid, GF_PROP_PID_SERVICE_PROVIDER);
-	if (p) pname = p->value.string;
-
 	if (!ctx->opid) {
 		ctx->opid = gf_filter_pid_new(filter);
 		gf_filter_pid_set_name(ctx->opid, "ts_mux");
+		//we are muxing, prevent m2tsdmx from trying to probe the file
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_FILEPATH, NULL);
+
+		if (!ctx->cdur_overwrite) {
+			p = gf_filter_pid_get_property(pid, GF_PROP_PID_DASH_FDUR);
+			if (p && p->value.frac.den) {
+				ctx->cdur = p->value.frac;
+				ctx->cdur_overwrite = GF_TRUE;
+			}
+		}
 	}
 	//set output properties at init or reconfig
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DECODER_CONFIG, NULL);
@@ -1153,7 +1297,6 @@ static GF_Err tsmux_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_TIMESCALE, &PROP_UINT(90000));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_NO_TS_LOOP, &PROP_BOOL(GF_TRUE));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DASH_MODE, NULL);
-
 	mux_assign_mime_file_ext(pid, ctx->opid, M2TS_FILE_EXTS, M2TS_MIMES, "ts");
 
 	p = gf_filter_pid_get_info(pid, GF_PROP_PID_DASH_MODE, &pe);
@@ -1178,8 +1321,8 @@ static GF_Err tsmux_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 				ctx->force_seg_sync = GF_TRUE;
 		}
 	}
-	p = gf_filter_pid_get_info(pid, GF_PROP_PID_LLHLS, &pe);
-	ctx->llhls = p ? p->value.uint : 0;
+	p = gf_filter_pid_get_info(pid, GF_PROP_PID_LLHAS_MODE, &pe);
+	ctx->llhas_mode = p ? p->value.uint : GF_LLHAS_NONE;
 
 	gf_filter_release_property(pe);
 
@@ -1209,7 +1352,7 @@ static GF_Err tsmux_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 		}
 	}
 
-	if (ctx->llhls && (!ctx->ref_pid || (streamtype == GF_STREAM_VISUAL)) ) {
+	if (ctx->llhas_mode && (!ctx->ref_pid || (streamtype == GF_STREAM_VISUAL)) ) {
 		ctx->ref_pid = tspid;
 	}
 
@@ -1255,7 +1398,15 @@ static GF_Err tsmux_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 
 		prog = gf_m2ts_mux_program_add(ctx->mux, service_id, pmt_id, ctx->pmt_rate, pcr_offset, ctx->mpeg4, ctx->pmt_version, ctx->disc, first_pts_val);
 
+		pe = NULL;
+		sname = ctx->name;
+		pname = ctx->provider;
+		p = gf_filter_pid_get_info(pid, GF_PROP_PID_SERVICE_NAME, &pe);
+		if (p) sname = p->value.string;
+		p = gf_filter_pid_get_info(pid, GF_PROP_PID_SERVICE_PROVIDER, &pe);
+		if (p) pname = p->value.string;
 		if (sname) gf_m2ts_mux_program_set_name(prog, sname, pname);
+		gf_filter_release_property(pe);
 
 		p = gf_filter_pid_get_property(tspid->ipid, GF_PROP_PID_DASH_SPARSE);
 		if (p && p->value.boolean) ctx->keepts = GF_TRUE;
@@ -1264,6 +1415,51 @@ static GF_Err tsmux_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 			prog->force_first_pts = GF_FILTER_NO_TS;
 		}
 
+		p = gf_filter_pid_get_property(tspid->ipid, GF_PROP_PID_SCTE35_PID);
+		if (p && !tspid->ctx->scte35_stream) {
+			M2Pid *m2pid;
+			GF_SAFEALLOC(m2pid, M2Pid);
+			if (!m2pid) return GF_OUT_OF_MEM;
+
+			m2pid->sid = service_id; // belongs to the same service
+			m2pid->ipid = pid;       // attach to parent pid
+			m2pid->ctx = ctx;
+			m2pid->codec_id = GF_CODECID_SCTE35;
+			m2pid->is_sparse = GF_TRUE;
+			m2pid->prog = prog;
+
+			m2pid->esi.stream_type = GF_STREAM_METADATA;
+			m2pid->esi.codecid = GF_CODECID_SCTE35;
+			m2pid->esi.timescale = 90000;
+			m2pid->esi.caps = GF_ESI_STREAM_WITHOUT_MPEG4_SYSTEMS|GF_ESI_STREAM_SPARSE;
+			//m2pid->esi.input_ctrl = tsmux_esi_ctrl;
+			m2pid->esi.input_udta = m2pid;
+
+			gf_list_add(tspid->ctx->pids, m2pid);
+
+			int scte35_pid = gf_m2ts_mux_program_get_pmt_pid(m2pid->prog);
+			scte35_pid += 1 + gf_m2ts_mux_program_get_stream_count(m2pid->prog);
+			m2pid->ctx->scte35_stream = gf_m2ts_program_stream_add(m2pid->prog, &m2pid->esi, scte35_pid, GF_FALSE, GF_FALSE, GF_FALSE);
+
+			GF_M2TSDescriptor *scte35_desc;
+			GF_SAFEALLOC(scte35_desc, GF_M2TSDescriptor);
+			if (!scte35_desc) return GF_OUT_OF_MEM;
+			scte35_desc->tag = GF_M2TS_REGISTRATION_DESCRIPTOR;
+			scte35_desc->data = gf_strdup("CUEI");
+			if (!scte35_desc->data) return GF_OUT_OF_MEM;
+			scte35_desc->data_len = 4;
+			gf_list_add(m2pid->prog->loop_descriptors, scte35_desc);
+
+			m2pid->ctx->scte35_stream->process = tsmux_stream_process_scte35;
+
+			m2pid->prog->pmt->table_needs_update = GF_TRUE;
+			m2pid->ctx->pmt_update_pending = GF_TRUE;
+			m2pid->ctx->update_mux = GF_TRUE;
+		}
+		if (codec_id == GF_CODECID_SCTE35) {
+			streamtype = GF_STREAM_METADATA; // necessary when importing from NHML
+		}
+
 		GF_LOG(GF_LOG_INFO, GF_LOG_CONTAINER, ("[M2TSMux] Setting up program ID %d - send rates: PSI %d ms PCR every %d ms max - PCR offset %d\n", service_id, ctx->pmt_rate, ctx->max_pcr, ctx->pcr_offset));
 	}
 	if (tspid->esi.stream_type != streamtype)
@@ -1587,7 +1783,7 @@ static void tsmux_insert_sidx(GF_TSMuxCtx *ctx, Bool final_flush)
 	tsidx->offset = (ctx->nb_sidx_entries>1) ? 0 : ctx->nb_pck_first_sidx;
 }
 
-static void tsmux_flush_frag_hls(GF_TSMuxCtx *ctx, Bool is_last)
+static void tsmux_flush_frag_llhas(GF_TSMuxCtx *ctx, Bool is_last)
 {
 	GF_FilterEvent evt;
 
@@ -1653,6 +1849,9 @@ static GF_Err tsmux_process(GF_Filter *filter)
 			p = gf_filter_pck_get_property(pck, GF_PROP_PCK_FILENAME);
 			if (p)
 				strcpy(tspid->ctx->dash_file_name, p->value.string);
+			p = gf_filter_pck_get_property(pck, GF_PROP_PCK_LLHAS_TEMPLATE);
+			if (p)
+				strcpy(tspid->ctx->llhas_template, p->value.string);
 			p = gf_filter_pck_get_property(pck, GF_PROP_PCK_IDXFILENAME);
 			if (p)
 				strcpy(tspid->ctx->idx_file_name, p->value.string);
@@ -1667,12 +1866,12 @@ static GF_Err tsmux_process(GF_Filter *filter)
 	}
 
 
-	if (ctx->wait_dash_flush || ctx->wait_llhls_flush) {
+	if (ctx->wait_dash_flush || ctx->wait_llhas_flush) {
 		//we are waiting for all packets to be flushed
 		if (ctx->pending_packets)
 			return GF_OK;
 
-		Bool is_llhls_flush = GF_FALSE;
+		Bool is_llhas_flush = GF_FALSE;
 		Bool is_eods_flush = GF_FALSE;
 		u32 i, done=0, count = gf_list_count(ctx->pids);
 		for (i=0; i<count; i++) {
@@ -1685,26 +1884,26 @@ static GF_Err tsmux_process(GF_Filter *filter)
 				M2Pid *tspid = gf_list_get(ctx->pids, i);
 				if (tspid->has_seen_eods==M2TS_EODS_FORCED)
 					is_eods_flush = GF_TRUE;
-				else if (tspid->has_seen_eods==M2TS_EODS_LLHLS) {
-					is_llhls_flush = GF_TRUE;
+				else if (tspid->has_seen_eods==M2TS_EODS_LLHAS) {
+					is_llhas_flush = GF_TRUE;
 				}
 				tspid->has_seen_eods = 0;
-				tspid->llhls_dts_init = 0;
+				tspid->llhas_dts_init = 0;
 			}
 			ctx->wait_dash_flush = GF_FALSE;
 			ctx->mux->force_pat = GF_TRUE;
 
-			if (is_llhls_flush) {
-				ctx->wait_llhls_flush = GF_FALSE;
-				ctx->next_is_llhls_start = GF_TRUE;
+			if (is_llhas_flush) {
+				ctx->wait_llhas_flush = GF_FALSE;
+				ctx->next_is_llhas_start = GF_TRUE;
 
-				tsmux_flush_frag_hls(ctx, GF_FALSE);
+				tsmux_flush_frag_llhas(ctx, GF_FALSE);
 			} else {
 				ctx->next_is_start = ctx->dash_file_switch;
 				ctx->dash_file_switch = GF_FALSE;
 
-				if (ctx->llhls) {
-					tsmux_flush_frag_hls(ctx, GF_TRUE);
+				if (ctx->llhas_mode) {
+					tsmux_flush_frag_llhas(ctx, GF_TRUE);
 					ctx->frag_offset = 0;
 				}
 
@@ -1776,20 +1975,31 @@ static GF_Err tsmux_process(GF_Filter *filter)
 				gf_filter_pck_set_property(pck, GF_PROP_PCK_FILENAME, &PROP_STRING(ctx->dash_file_name) ) ;
 			if (ctx->dash_seg_start.den)
 				gf_filter_pck_set_property(pck, GF_PROP_PCK_MPD_SEGSTART, &PROP_FRAC64(ctx->dash_seg_start) ) ;
+			if (ctx->llhas_template[0])
+				gf_filter_pck_set_property(pck, GF_PROP_PCK_LLHAS_TEMPLATE, &PROP_STRING(ctx->llhas_template) ) ;
 
 			ctx->dash_file_name[0] = 0;
+			ctx->llhas_template[0] = 0;
 			ctx->next_is_start = GF_FALSE;
-			if (ctx->llhls>1) {
-				ctx->frag_num=1;
-				gf_filter_pck_set_property(pck, GF_PROP_PCK_HLS_FRAG_NUM, &PROP_UINT(ctx->frag_num));
+			if (ctx->llhas_mode>GF_LLHAS_BYTERANGES) {
+				ctx->frag_num=0;
+				u32 fnum = ctx->frag_num;
+				//we'll need to redo all LLHLS tests
+				if (gf_sys_is_test_mode() && (ctx->llhas_mode == GF_LLHAS_PARTS))
+					fnum++;
+				gf_filter_pck_set_property(pck, GF_PROP_PCK_LLHAS_FRAG_NUM, &PROP_UINT(fnum));
 			}
 		}
-		else if (ctx->next_is_llhls_start) {
-			if (ctx->llhls>1) {
+		else if (ctx->next_is_llhas_start) {
+			if (ctx->llhas_mode>GF_LLHAS_BYTERANGES) {
+				u32 fnum = ctx->frag_num;
 				ctx->frag_num++;
-				gf_filter_pck_set_property(pck, GF_PROP_PCK_HLS_FRAG_NUM, &PROP_UINT(ctx->frag_num));
+				//we'll need to redo all LLHLS tests
+				if (gf_sys_is_test_mode() && (ctx->llhas_mode == GF_LLHAS_PARTS))
+					fnum++;
+				gf_filter_pck_set_property(pck, GF_PROP_PCK_LLHAS_FRAG_NUM, &PROP_UINT(fnum));
 			}
-			ctx->next_is_llhls_start = GF_FALSE;
+			ctx->next_is_llhas_start = GF_FALSE;
 		}
 
 		pck_ts = gf_m2ts_get_ts_clock_90k(ctx->mux);
@@ -1813,7 +2023,7 @@ static GF_Err tsmux_process(GF_Filter *filter)
 		ctx->nb_pck_in_file += nb_pck_in_pack;
 		nb_pck_in_call += nb_pck_in_pack;
 		nb_pck_in_pack = 0;
-		if (ctx->llhls)
+		if (ctx->llhas_mode)
 			ctx->frag_size += osize;
 
 		if (is_pack_flush)
@@ -1826,7 +2036,7 @@ static GF_Err tsmux_process(GF_Filter *filter)
 			break;
 	}
 
-	if (ctx->wait_dash_flush || ctx->wait_llhls_flush) {
+	if (ctx->wait_dash_flush || ctx->wait_llhas_flush) {
 		u32 i, done=0, count = gf_list_count(ctx->pids);
 		for (i=0; i<count; i++) {
 			M2Pid *tspid = gf_list_get(ctx->pids, i);
@@ -1906,7 +2116,7 @@ static GF_Err tsmux_process(GF_Filter *filter)
 		gf_filter_ask_rt_reschedule(filter, 0);
 	}
 	//PMT update management is still under progress...
-	ctx->pmt_update_pending = 0;
+	ctx->pmt_update_pending = GF_FALSE;
 
 	return GF_OK;
 }
@@ -1919,11 +2129,18 @@ static GF_Err tsmux_initialize(GF_Filter *filter)
 	ctx->mux = gf_m2ts_mux_new(ctx->rate, ctx->pat_rate, ctx->realtime);
 	ctx->mux->flush_pes_at_rap = ctx->flush_rap;
 
-	if (gf_sys_is_test_mode() && ctx->pcr_init<0)
+	if (gf_sys_is_test_mode() && (ctx->pcr_init==-1))
 		ctx->pcr_init = 1000000;
 
 	gf_m2ts_mux_use_single_au_pes_mode(ctx->mux, ctx->pes_pack);
-	if (ctx->pcr_init>=0) gf_m2ts_mux_set_initial_pcr(ctx->mux, (u64) ctx->pcr_init);
+	if (ctx->pcr_init != -1) {
+		u64 pcr_init;
+		if (ctx->pcr_init>=0)
+			pcr_init = (u64) ctx->pcr_init;
+		else
+			pcr_init = GF_M2TS_MAX_PCR + ctx->pcr_init;
+		gf_m2ts_mux_set_initial_pcr(ctx->mux, pcr_init);
+	}
 	gf_m2ts_mux_set_pcr_max_interval(ctx->mux, ctx->max_pcr);
 	gf_m2ts_mux_enable_pcr_only_packets(ctx->mux, ctx->pcr_only);
 
@@ -2021,10 +2238,14 @@ static const GF_FilterCapability TSMuxCaps[] =
 	//for MPEG-H audio MHAS we need to insert sync packets
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_MHAS),
 
+	//for AC4 we need bitstream framing
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_AC4),
+
 	//static output cap file extension
 	CAP_UINT(GF_CAPS_OUTPUT_STATIC,  GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
 	CAP_STRING(GF_CAPS_OUTPUT_STATIC, GF_PROP_PID_FILE_EXT, M2TS_FILE_EXTS),
 	CAP_STRING(GF_CAPS_OUTPUT_STATIC, GF_PROP_PID_MIME, M2TS_MIMES),
+	CAP_UINT(GF_CAPS_OUTPUT_STATIC|GF_CAPFLAG_PRESENT|GF_CAPFLAG_OPTIONAL, GF_PROP_PID_TIMESCALE, 0),
 	{0},
 
 	//for now don't accept files as input, although we could store them as items, to refine
@@ -2044,7 +2265,9 @@ static const GF_FilterCapability TSMuxCaps[] =
 	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_SMPTE_VC1),
 	//we don't accept MPEG-H audio without MHAS
 	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_MPHA),
-	//no RAW support for now$
+	//we don't accept AC4 audio without framing
+	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_AC4),
+	//no RAW support for now
 	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_RAW),
 	{0},
 
@@ -2088,7 +2311,7 @@ static const GF_FilterArgs TSMuxArgs[] =
 	"- copy: uses BIFS PES but removes timestamps in BIFS SL and only carries PES timestamps", GF_PROP_UINT, "off", "off|on|copy", GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(flush_rap), "force flushing mux program when RAP is found on video, and injects PAT and PMT before the next video PES begin", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(pcr_only), "enable PCR-only TS packets", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
-	{ OFFS(pcr_init), "set initial PCR value for the programs. A negative value means random value is picked", GF_PROP_LSINT, "-1", NULL, 0},
+	{ OFFS(pcr_init), "set initial PCR value for the programs. -1 means random value is picked, other negative value means offset to maximum PCR", GF_PROP_LSINT, "-1", NULL, 0},
 	{ OFFS(sid), "set service ID for the program", GF_PROP_UINT, "0", NULL, 0},
 	{ OFFS(name), "set service name for the program", GF_PROP_STRING, NULL, NULL, 0},
 	{ OFFS(provider), "set service provider name for the program", GF_PROP_STRING, NULL, NULL, 0},
@@ -2099,7 +2322,11 @@ static const GF_FilterArgs TSMuxArgs[] =
 	{ OFFS(subs_sidx), "number of subsegments per sidx (negative value disables sidx)", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(keepts), "keep cts/dts untouched and adjust PCR accordingly, used to keep TS unmodified when dashing", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(cdur), "chunk duration for fragmentation modes", GF_PROP_FRACTION, "-1/1", NULL, GF_FS_ARG_HINT_HIDE},
-
+	{ OFFS(temi_fwd), "input TEMI properties when remuwing\n"
+		"- drop: remove input descriptors\n"
+		"- fwd: forward input descriptors\n"
+		"- ntp: forward input descriptors after NTP rewriting"
+		, GF_PROP_UINT, "fwd", "drop|fwd|ntp", GF_FS_ARG_HINT_EXPERT},
 	{0}
 };
 
@@ -2139,6 +2366,7 @@ GF_FilterRegister TSMuxRegister = {
 		"  - `Y`: use 64 bit signaling only.\n"
 		"  - `N`: use 32 bit signaling only and wrap around timecode value.\n"
 		"- N: insert NTP timestamp in TEMI timeline descriptor\n"
+		"- n: insert NTP timestamp using NTP for first packet than incrementing based on media timestamp (for non real-time)\n"
 		"- ID_OR_URL: If number, indicate the TEMI ID to use for external timeline. Otherwise, give the URL to insert\n"
 		"  \n"
 		"EX temi=\"url\"\n"
@@ -2153,6 +2381,8 @@ GF_FilterRegister TSMuxRegister = {
 		"Inserts an external TEMI with ID 4 and timescale 30000, NTP injection and carousel of 500 ms in the video stream of all programs.\n"
 		"\n"
 		"Warning: multipliers (k,m,g) are not supported in TEMI options.\n"
+		"\n"
+		"When input TEMI properties are found, they can be removed using [-temi_fwd](). When rewritten, any NTP information present is rewritten to the current NTP.\n"
 		"# Adaptive Streaming\n"
 		"In DASH and HLS mode:\n"
 		"- the PCR is always initialized at 0, and [-flush_rap]() is automatically set.\n"
@@ -2160,11 +2390,20 @@ GF_FilterRegister TSMuxRegister = {
 		"- `pes_pack=none` is forced since some demultiplexers have issues with non-aligned ADTS PES.\n"
 		"\n"
 		"The filter watches the property `FileNumber` on incoming packets to create new files, or new segments in DASH mode.\n"
+		"# Custom streams\n"
 		"The filter will look for property `M2TSRA` set on the input stream.\n"
 		"The value can either be a 4CC or a string, indicating the MP2G-2 TS Registration tag for unknown media types.\n"
+		"The value `SRT ` (alias: `srt`, `SRT`) will inject an SRT header with frame number increasing at each packet and start time 0.\n"
+		"EX gpac -i source.srt:#M2TSRA='SRT ' -o mux.ts\n"
+		"This will inject the content of the source SRT as a PES data stream, removing any markup.\n"
+		"EX gpac -i source.srt:stxtmod=sbtt:#M2TSRA='SRT ' -o mux.ts\n"
+		"This will inject the content of the source SRT as a PES data stream, keeping the markup.\n"
 		"\n"
 		"# Notes\n"
 		"In LATM mux mode, the decoder configuration is inserted at the given [-repeat_rate]() or `CarouselRate` PID property if defined.\n"
+		"\n"
+		"By default text streams are embeded using HLS ID3 schemes, use `M2TSRA` property to use raw private PES.\n"
+		"WebVTT header and TX3G formatting are removed, only the text data is injected.\n"
 	)
 	.private_size = sizeof(GF_TSMuxCtx),
 	.args = TSMuxArgs,
@@ -2175,6 +2414,7 @@ GF_FilterRegister TSMuxRegister = {
 	.configure_pid = tsmux_configure_pid,
 	.process = tsmux_process,
 	.process_event = tsmux_process_event,
+	.hint_class_type = GF_FS_CLASS_MULTIPLEXER
 };
 
 
