@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2023
+ *			Copyright (c) Telecom ParisTech 2017-2025
  *					All rights reserved
  *
  *  This file is part of GPAC / generic FILE output filter
@@ -28,30 +28,30 @@
 #include <gpac/constants.h>
 #include <gpac/xml.h>
 #include <gpac/network.h>
+#include <gpac/mpd.h>
 
 #ifndef GPAC_DISABLE_FOUT
 
-enum
-{
+GF_OPT_ENUM (GF_FileOutConcatMode,
 	FOUT_CAT_NONE = 0,
 	FOUT_CAT_AUTO,
-	FOUT_CAT_ALL
-};
+	FOUT_CAT_ALL,
+);
 
-enum
-{
+GF_OPT_ENUM (GF_FileOutOverwriteMode,
 	FOUT_OW_YES = 0,
 	FOUT_OW_NO,
-	FOUT_OW_ASK
-};
+	FOUT_OW_ASK,
+);
 
 typedef struct
 {
 	//options
 	Double start, speed;
 	char *dst, *mime, *ext;
-	Bool append, dynext, redund, noinitraw, force_null;
-	u32 cat, ow;
+	Bool append, dynext, redund, noinitraw, force_null, atomic;
+	GF_FileOutConcatMode cat;
+	GF_FileOutOverwriteMode ow;
 	u32 mvbk;
 	s32 max_cache_segs;
 
@@ -65,6 +65,7 @@ typedef struct
 	GF_FilterCapability in_caps[2];
 	char szExt[10];
 	char szFileName[GF_MAX_PATH];
+	char *llhas_template;
 
 	Bool patch_blocks;
 	Bool is_null;
@@ -76,13 +77,15 @@ typedef struct
 
 	FILE *hls_chunk;
 
-	u32 max_segs;
+	u32 max_segs, llhas_mode;
 	GF_List *past_files;
 
 	Bool gfio_pending;
 
 	u64 last_file_size;
 	Bool use_rel;
+	Bool use_move;
+	char *llhls_file_name;
 
 #ifdef GPAC_HAS_FD
 	Bool no_fd;
@@ -93,25 +96,38 @@ typedef struct
 #ifdef WIN32
 #include <io.h>
 #include <fcntl.h>
-
-#else //WIN32
-
-#ifdef GPAC_HAS_FD
-#include <unistd.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #endif
 
-#endif
+#define ATOMIC_SUFFIX	".gftmp"
 
 static void fileout_close_hls_chunk(GF_FileOutCtx *ctx, Bool final_flush)
 {
 	if (!ctx->hls_chunk) return;
 	gf_fclose(ctx->hls_chunk);
 	ctx->hls_chunk = NULL;
+	if (!ctx->llhls_file_name) return;
+
+	char szName[GF_MAX_PATH];
+	strcpy(szName, ctx->llhls_file_name);
+	strcat(szName, ATOMIC_SUFFIX);
+	gf_file_delete(ctx->llhls_file_name);
+	gf_file_move(szName, ctx->llhls_file_name);
+	gf_free(ctx->llhls_file_name);
+	ctx->llhls_file_name = NULL;
 }
 
-static GF_Err fileout_open_close(GF_FileOutCtx *ctx, const char *filename, const char *ext, u32 file_idx, Bool explicit_overwrite, char *file_suffix)
+static void fileout_check_close(GF_FileOutCtx *ctx)
+{
+	if (!ctx->use_move) return;
+	char szName[GF_MAX_PATH];
+	strcpy(szName, ctx->szFileName);
+	strcat(szName, ATOMIC_SUFFIX);
+	gf_file_delete(ctx->szFileName);
+	gf_file_move(szName, ctx->szFileName);
+	ctx->use_move = GF_FALSE;
+}
+
+static GF_Err fileout_open_close(GF_FileOutCtx *ctx, const char *filename, const char *ext, u32 file_idx, Bool explicit_overwrite, char *file_suffix, Bool check_no_open)
 {
 	if (!ctx->is_std) {
 #ifdef GPAC_HAS_FD
@@ -126,6 +142,7 @@ static GF_Err fileout_open_close(GF_FileOutCtx *ctx, const char *filename, const
 			gf_fclose(ctx->file);
 			fileout_close_hls_chunk(ctx, GF_FALSE);
 		}
+		fileout_check_close(ctx);
 	}
 	ctx->file = NULL;
 #ifdef GPAC_HAS_FD
@@ -139,91 +156,113 @@ static GF_Err fileout_open_close(GF_FileOutCtx *ctx, const char *filename, const
 	else if (!strcmp(filename, "stdout")) ctx->is_std = GF_TRUE;
 	else ctx->is_std = GF_FALSE;
 
+	if (!strcmp(filename, "null") || !strcmp(filename, "/dev/null"))
+		ext = NULL;
+
 	if (ctx->is_std) {
 		ctx->file = stdout;
+		ctx->nb_write = 0;
 #ifdef WIN32
 		_setmode(_fileno(stdout), _O_BINARY);
 #endif
+		return GF_OK;
+	}
 
+	char szName[GF_MAX_PATH];
+	char szFinalName[GF_MAX_PATH];
+	Bool append = ctx->append;
+	Bool is_gfio=GF_FALSE;
+	const char *url = filename;
+
+	if (!strncmp(filename, "gfio://", 7)) {
+		url = gf_fileio_translate_url(filename);
+		is_gfio=GF_TRUE;
+	}
+
+	if (ctx->dynext) {
+		const char *has_ext = gf_file_ext_start(url);
+
+		strcpy(szFinalName, url);
+		if (!has_ext && ext) {
+			strcat(szFinalName, ".");
+			strcat(szFinalName, ext);
+		}
 	} else {
-		char szFinalName[GF_MAX_PATH];
-		Bool append = ctx->append;
-		Bool is_gfio=GF_FALSE;
-		const char *url = filename;
+		strcpy(szFinalName, url);
+	}
 
-		if (!strncmp(filename, "gfio://", 7)) {
-			url = gf_fileio_translate_url(filename);
-			is_gfio=GF_TRUE;
-		}
-
-		if (ctx->dynext) {
-			const char *has_ext = gf_file_ext_start(url);
-
-			strcpy(szFinalName, url);
-			if (!has_ext && ext) {
-				strcat(szFinalName, ".");
-				strcat(szFinalName, ext);
-			}
+	if (ctx->use_templates) {
+		GF_Err e;
+		gf_assert(ctx->dst);
+		if (!strcmp(filename, ctx->dst)) {
+			strcpy(szName, szFinalName);
+			e = gf_filter_pid_resolve_file_template(ctx->pid, szName, szFinalName, file_idx, file_suffix);
 		} else {
-			strcpy(szFinalName, url);
+			char szFileName[GF_MAX_PATH];
+			strcpy(szFileName, szFinalName);
+			strcpy(szName, ctx->dst);
+			e = gf_filter_pid_resolve_file_template_ex(ctx->pid, szName, szFinalName, file_idx, file_suffix, szFileName);
 		}
-
-		if (ctx->use_templates) {
-			GF_Err e;
-			char szName[GF_MAX_PATH];
-			gf_assert(ctx->dst);
-			if (!strcmp(filename, ctx->dst)) {
-				strcpy(szName, szFinalName);
-				e = gf_filter_pid_resolve_file_template(ctx->pid, szName, szFinalName, file_idx, file_suffix);
-			} else {
-				char szFileName[GF_MAX_PATH];
-				strcpy(szFileName, szFinalName);
-				strcpy(szName, ctx->dst);
-				e = gf_filter_pid_resolve_file_template_ex(ctx->pid, szName, szFinalName, file_idx, file_suffix, szFileName);
-			}
-			if (e) {
-				return ctx->error = e;
-			}
+		if (e) {
+			return ctx->error = e;
 		}
+	}
 
-		if (!gf_file_exists(szFinalName)) append = GF_FALSE;
+	if (!gf_file_exists(szFinalName)) append = GF_FALSE;
 
-		if (!strcmp(szFinalName, ctx->szFileName) && (ctx->cat==FOUT_CAT_AUTO))
-			append = GF_TRUE;
+	if (!strcmp(szFinalName, ctx->szFileName) && (ctx->cat==FOUT_CAT_AUTO))
+		append = GF_TRUE;
 
-		if (!append && (ctx->ow!=FOUT_OW_YES) && gf_file_exists(szFinalName)) {
-			char szRes[21];
-			s32 res;
+	if (!append && (ctx->ow!=FOUT_OW_YES) && gf_file_exists(szFinalName)) {
+		char szRes[21];
+		s32 res;
 
-			if (ctx->ow==FOUT_OW_ASK) {
-				fprintf(stderr, "File %s already exists - override (y/n/a) ?:", szFinalName);
-				res = scanf("%20s", szRes);
-				if (!res || (szRes[0] == 'n') || (szRes[0] == 'N')) {
-					return ctx->error = GF_IO_ERR;
-				}
-				if ((szRes[0] == 'a') || (szRes[0] == 'A')) ctx->ow = GF_TRUE;
-			} else {
+		if (ctx->ow==FOUT_OW_ASK) {
+			fprintf(stderr, "File %s already exists - override (y/n/a) ?:", szFinalName);
+			res = scanf("%20s", szRes);
+			if (!res || (szRes[0] == 'n') || (szRes[0] == 'N')) {
 				return ctx->error = GF_IO_ERR;
 			}
+			if ((szRes[0] == 'a') || (szRes[0] == 'A')) ctx->ow = FOUT_OW_YES;
+		} else {
+			return ctx->error = GF_IO_ERR;
 		}
-
-		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[FileOut] opening output file %s\n", szFinalName));
-#ifdef GPAC_HAS_FD
-		if (!ctx->no_fd && !is_gfio && !append && !gf_opts_get_bool("core", "no-fd")
-			&& (!ctx->original_url || strncmp(ctx->original_url, "gfio://", 7))
-		) {
-			//make sure output dir exists
-			gf_fopen(szFinalName, "mkdir");
-			ctx->fd = open(szFinalName, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH );
-		} else
-#endif
-			ctx->file = gf_fopen_ex(szFinalName, ctx->original_url, append ? "a+b" : "w+b", GF_FALSE);
-
-		if (!strcmp(szFinalName, ctx->szFileName) && !append && ctx->nb_write && !explicit_overwrite) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_MMIO, ("[FileOut] re-opening in write mode output file %s, content overwrite (use `cat` option to enable append)\n", szFinalName));
-		}
-		strcpy(ctx->szFileName, szFinalName);
 	}
+
+	if (check_no_open && (ctx->llhas_mode==GF_LLHAS_SUBSEG)) {
+		strcpy(ctx->szFileName, szFinalName);
+		ctx->nb_write = 0;
+		return GF_OK;
+	}
+
+	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[FileOut] opening output file %s\n", szFinalName));
+
+	ctx->use_move = GF_FALSE;
+	if (ctx->atomic && !append && !is_gfio && (!ctx->original_url || strncmp(ctx->original_url, "gfio://", 7))) {
+		ctx->use_move = GF_TRUE;
+	}
+	strcpy(szName, szFinalName);
+	if (ctx->use_move) {
+		strcat(szName, ATOMIC_SUFFIX);
+	}
+
+#ifdef GPAC_HAS_FD
+	if (!ctx->no_fd && !is_gfio && !append && !gf_opts_get_bool("core", "no-fd")
+		&& (!ctx->original_url || strncmp(ctx->original_url, "gfio://", 7))
+	) {
+		//make sure output dir exists
+		gf_fopen(szFinalName, "mkdir");
+
+		ctx->fd = gf_fd_open(szName, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH );
+	} else
+#endif
+		ctx->file = gf_fopen_ex(szName, ctx->original_url, append ? "a+b" : "w+b", GF_FALSE);
+
+	if (!strcmp(szFinalName, ctx->szFileName) && !append && ctx->nb_write && !explicit_overwrite) {
+		GF_LOG(GF_LOG_WARNING, GF_LOG_MMIO, ("[FileOut] re-opening in write mode output file %s, content overwrite (use `cat` option to enable append)\n", szFinalName));
+	}
+	strcpy(ctx->szFileName, szFinalName);
+
 	ctx->nb_write = 0;
 	if (!ctx->file
 #ifdef GPAC_HAS_FD
@@ -246,7 +285,7 @@ static void fileout_setup_file(GF_FileOutCtx *ctx, Bool explicit_overwrite)
 	ext = gf_filter_pid_get_property(ctx->pid, GF_PROP_PID_FILE_EXT);
 
 	if (p && p->value.string) {
-		fileout_open_close(ctx, p->value.string, (ext && ctx->dynext) ? ext->value.string : NULL, 0, explicit_overwrite, NULL);
+		fileout_open_close(ctx, p->value.string, (ext && ctx->dynext) ? ext->value.string : NULL, 0, explicit_overwrite, NULL, GF_FALSE);
 		return;
 	}
 	if (!dst) {
@@ -266,19 +305,16 @@ static void fileout_setup_file(GF_FileOutCtx *ctx, Bool explicit_overwrite)
 		}
 	}
 	if (ctx->dynext) {
-		p = gf_filter_pid_get_property(ctx->pid, GF_PROP_PCK_FILENUM);
-		if (!p) {
-			if (ext && ext->value.string) {
-				fileout_open_close(ctx, dst, ext->value.string, 0, explicit_overwrite, NULL);
-			}
+		if (ext && ext->value.string) {
+			fileout_open_close(ctx, dst, ext->value.string, 0, explicit_overwrite, NULL, GF_FALSE);
 		}
 	} else if (ctx->dst) {
-		fileout_open_close(ctx, ctx->dst, NULL, 0, explicit_overwrite, NULL);
+		fileout_open_close(ctx, ctx->dst, NULL, 0, explicit_overwrite, NULL, GF_FALSE);
 	} else {
 		p = gf_filter_pid_get_property(ctx->pid, GF_PROP_PID_FILEPATH);
 		if (!p) p = gf_filter_pid_get_property(ctx->pid, GF_PROP_PID_URL);
 		if (p && p->value.string)
-			fileout_open_close(ctx, p->value.string, NULL, 0, explicit_overwrite, NULL);
+			fileout_open_close(ctx, p->value.string, NULL, 0, explicit_overwrite, NULL, GF_FALSE);
 	}
 }
 static GF_Err fileout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove)
@@ -287,7 +323,7 @@ static GF_Err fileout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 	GF_FileOutCtx *ctx = (GF_FileOutCtx *) gf_filter_get_udta(filter);
 	if (is_remove) {
 		ctx->pid = NULL;
-		fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL);
+		fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL, GF_FALSE);
 		return GF_OK;
 	}
 	gf_filter_pid_check_caps(pid);
@@ -320,6 +356,8 @@ static GF_Err fileout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 		if (ctx->max_segs && !ctx->past_files)
 			ctx->past_files = gf_list_new();
 	}
+	p = gf_filter_pid_get_property(ctx->pid, GF_PROP_PID_LLHAS_MODE);
+	ctx->llhas_mode = p ? p->value.uint : GF_LLHAS_NONE;
 
 #ifdef GPAC_HAS_FD
 	//disable fd for mp2t since we only dispatch small blocks - todo check this for other streams ?
@@ -424,7 +462,7 @@ static void fileout_finalize(GF_Filter *filter)
 
 	fileout_close_hls_chunk(ctx, GF_TRUE);
 
-	fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL);
+	fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL, GF_FALSE);
 	if (ctx->gfio_ref)
 		gf_fileio_open_url((GF_FileIO *)ctx->gfio_ref, NULL, "unref", &e);
 
@@ -435,6 +473,8 @@ static void fileout_finalize(GF_Filter *filter)
 		}
 		gf_list_del(ctx->past_files);
 	}
+	if (ctx->llhas_template) gf_free(ctx->llhas_template);
+	if (ctx->llhls_file_name) gf_free(ctx->llhls_file_name);
 }
 
 static GF_Err fileout_process(GF_Filter *filter)
@@ -482,7 +522,7 @@ restart:
 					evt.seg_size.media_range_start = ctx->offset_at_seg_start;
 #ifdef GPAC_HAS_FD
 					if (ctx->fd>=0) {
-						evt.seg_size.media_range_end = lseek(ctx->fd, 0, SEEK_CUR);
+						evt.seg_size.media_range_end = lseek_64(ctx->fd, 0, SEEK_CUR);
 					} else
 #endif
 					if (ctx->file) {
@@ -497,7 +537,7 @@ restart:
 					gf_filter_pid_send_event(ctx->pid, &evt);
 				}
 			}
-			fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL);
+			fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL, GF_FALSE);
 			return GF_EOS;
 		}
 		return GF_OK;
@@ -568,7 +608,7 @@ restart:
 				evt.seg_size.media_range_start = ctx->offset_at_seg_start;
 #ifdef GPAC_HAS_FD
 				if (ctx->fd>=0) {
-					evt.seg_size.media_range_end = lseek(ctx->fd, 0, SEEK_CUR);
+					evt.seg_size.media_range_end = lseek_64(ctx->fd, 0, SEEK_CUR);
 				} else
 #endif
 				if (ctx->file) {
@@ -608,8 +648,6 @@ restart:
 		}
 		//filename change at packet start, open new file
 		if (!fname) fname = gf_filter_pck_get_property(pck, GF_PROP_PCK_FILENAME);
-		if (!fname) fname = gf_filter_pck_get_property(pck, GF_PROP_PID_OUTPATH);
-		if (!ext) ext = gf_filter_pck_get_property(pck, GF_PROP_PID_FILE_EXT);
 		if (fname) name = fname->value.string;
 
 		fsuf = gf_filter_pck_get_property(pck, GF_PROP_PCK_FILESUF);
@@ -621,13 +659,13 @@ restart:
 			Bool use_rel = GF_FALSE;
 			if (ctx->dst) {
 				use_rel = ctx->use_rel;
-				rel = gf_filter_pck_get_property(pck, GF_PROP_PID_FILE_REL);
+				rel = gf_filter_pck_get_property(pck, GF_PROP_PCK_FILE_REL);
 				if (rel && rel->value.boolean) use_rel = GF_TRUE;
 			}
 			if (use_rel) {
 				name = gf_url_concatenate(ctx->dst, name);
 			}
-			fileout_open_close(ctx, name, ext ? ext->value.string : NULL, fnum ? fnum->value.uint : 0, explicit_overwrite, fsuf ? fsuf->value.string : NULL);
+			fileout_open_close(ctx, name, ext ? ext->value.string : NULL, fnum ? fnum->value.uint : 0, explicit_overwrite, fsuf ? fsuf->value.string : NULL, GF_TRUE);
 
 			if (use_rel) {
 				gf_free((char*) name);
@@ -647,6 +685,12 @@ restart:
 			ctx->gfio_pending = GF_TRUE;
 		}
 
+		fname = gf_filter_pck_get_property(pck, GF_PROP_PCK_LLHAS_TEMPLATE);
+		if (fname) {
+			if (ctx->llhas_template) gf_free(ctx->llhas_template);
+			ctx->llhas_template = gf_strdup(fname->value.string);
+		}
+
 		if (ctx->max_segs) {
 			while (gf_list_count(ctx->past_files)>ctx->max_segs) {
 				char *url = gf_list_pop_front(ctx->past_files);
@@ -659,15 +703,36 @@ restart:
 			}
 		}
 	}
-
-	p = gf_filter_pck_get_property(pck, GF_PROP_PCK_HLS_FRAG_NUM);
+	p = gf_filter_pck_get_property(pck, GF_PROP_PCK_LLHAS_FRAG_NUM);
 	if (p) {
-		char szHLSChunk[GF_MAX_PATH+21];
-		snprintf(szHLSChunk, GF_MAX_PATH+20, "%s.%d", ctx->szFileName, p->value.uint);
+#ifndef GPAC_DISABLE_MPD
+		char *llhas_chunkname = gf_mpd_resolve_subnumber(ctx->llhas_template, ctx->szFileName, p->value.uint);
 		//for now we only use buffered IO for hls chunks, too small to really benefit from direct write
-		if (ctx->hls_chunk) gf_fclose(ctx->hls_chunk);
-		ctx->hls_chunk = gf_fopen_ex(szHLSChunk, ctx->original_url, "w+b", GF_FALSE);
+		fileout_close_hls_chunk(ctx, GF_FALSE);
+
+		if (ctx->use_move) {
+			ctx->llhls_file_name = gf_strdup(llhas_chunkname);
+			gf_dynstrcat(&llhas_chunkname, ATOMIC_SUFFIX, NULL);
+		}
+
+		//we need a gf_url_concatenate to register target gfio, so simulate one
+		//this works because file subparts are always in the same dir as segment file
+		if (ctx->original_url && !strncmp(ctx->original_url, "gfio://", 7)) {
+			const char *rad = strrchr(llhas_chunkname, '/');
+			if (!rad) rad = strrchr(llhas_chunkname, '\\');
+			if (!rad) rad = llhas_chunkname;
+			else rad += 1;
+			//no need to check return: if failure, following gf_fopen_ex will also fail
+			gf_fileio_open_url(gf_fileio_from_url(ctx->original_url), rad, "url", &e);
+		}
+		ctx->hls_chunk = gf_fopen_ex(llhas_chunkname, ctx->original_url, "w+b", GF_FALSE);
+
 		ctx->gfio_pending = GF_TRUE;
+		gf_free(llhas_chunkname);
+#else
+		gf_filter_setup_failure(filter, GF_NOT_SUPPORTED);
+		return GF_NOT_SUPPORTED;
+#endif
 	}
 
 check_gfio:
@@ -688,15 +753,19 @@ check_gfio:
 	}
 
 
-	pck_data = gf_filter_pck_get_data(pck, &pck_size);
+	Bool main_valid = 0;
 	if (ctx->file
 #ifdef GPAC_HAS_FD
 		|| (ctx->fd>=0)
 #endif
-	) {
+	)
+		main_valid = GF_TRUE;
+
+	pck_data = gf_filter_pck_get_data(pck, &pck_size);
+	if (main_valid || ctx->hls_chunk) {
 		GF_FilterFrameInterface *hwf = gf_filter_pck_get_frame_interface(pck);
 		if (pck_data) {
-			if (ctx->patch_blocks && gf_filter_pck_get_seek_flag(pck)) {
+			if (ctx->patch_blocks && gf_filter_pck_get_seek_flag(pck) && main_valid) {
 				u64 bo = gf_filter_pck_get_byte_offset(pck);
 				if (ctx->is_std) {
 					GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[FileOut] Cannot patch file, output is stdout\n"));
@@ -713,8 +782,8 @@ check_gfio:
 #ifdef GPAC_HAS_FD
 						if (ctx->fd>=0) {
 							nb_write = (u32) write(ctx->fd, pck_data, pck_size);
-							cur_w = lseek(ctx->fd, 0, SEEK_CUR);
-							lseek(ctx->fd, pos, SEEK_SET);
+							cur_w = lseek_64(ctx->fd, 0, SEEK_CUR);
+							lseek_64(ctx->fd, pos, SEEK_SET);
 						} else
 #endif
 						{
@@ -742,7 +811,7 @@ check_gfio:
 
 #ifdef GPAC_HAS_FD
 								if (ctx->fd>=0) {
-									lseek(ctx->fd, cur_r - move_bytes, SEEK_SET);
+									lseek_64(ctx->fd, cur_r - move_bytes, SEEK_SET);
 									nb_write = (u32) read(ctx->fd, block, (size_t) move_bytes);
 								} else
 #endif
@@ -758,7 +827,7 @@ check_gfio:
 
 #ifdef GPAC_HAS_FD
 								if (ctx->fd>=0) {
-									lseek(ctx->fd, cur_w - move_bytes, SEEK_SET);
+									lseek_64(ctx->fd, cur_w - move_bytes, SEEK_SET);
 									nb_write = (u32) write(ctx->fd, block, (size_t) move_bytes);
 								} else
 #endif
@@ -780,9 +849,9 @@ check_gfio:
 
 #ifdef GPAC_HAS_FD
 					if (ctx->fd>=0) {
-						lseek(ctx->fd, bo, SEEK_SET);
+						lseek_64(ctx->fd, bo, SEEK_SET);
 						nb_write = (u32) write(ctx->fd, pck_data, pck_size);
-						lseek(ctx->fd, pos, SEEK_SET);
+						lseek_64(ctx->fd, pos, SEEK_SET);
 					} else
 #endif
 					{
@@ -797,18 +866,21 @@ check_gfio:
 					}
 				}
 			} else {
-#ifdef GPAC_HAS_FD
-				if (ctx->fd>=0) {
-					nb_write = (u32) write(ctx->fd, pck_data, pck_size);
-				} else
-#endif
-					nb_write = (u32) gf_fwrite(pck_data, pck_size, ctx->file);
+				if (main_valid) {
 
-				if (nb_write!=pck_size) {
-					GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[FileOut] Write error, wrote %d bytes but had %d to write\n", nb_write, pck_size));
-					e = GF_IO_ERR;
+#ifdef GPAC_HAS_FD
+					if (ctx->fd>=0) {
+						nb_write = (u32) write(ctx->fd, pck_data, pck_size);
+					} else
+#endif
+						nb_write = (u32) gf_fwrite(pck_data, pck_size, ctx->file);
+
+					if (nb_write!=pck_size) {
+						GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[FileOut] Write error, wrote %d bytes but had %d to write\n", nb_write, pck_size));
+						e = GF_IO_ERR;
+					}
+					ctx->nb_write += nb_write;
 				}
-				ctx->nb_write += nb_write;
 
 				if (ctx->hls_chunk) {
 					nb_write = (u32) gf_fwrite(pck_data, pck_size, ctx->hls_chunk);
@@ -816,9 +888,11 @@ check_gfio:
 						GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[FileOut] Write error, wrote %d bytes but had %d to write\n", nb_write, pck_size));
 						e = GF_IO_ERR;
 					}
+					if (!main_valid)
+						ctx->nb_write += nb_write;
 				}
 			}
-		} else if (hwf) {
+		} else if (hwf && main_valid) {
 			u32 w, h, stride, stride_uv, pf;
 			u32 nb_planes, uv_height;
 			p = gf_filter_pid_get_property(ctx->pid, GF_PROP_PID_WIDTH);
@@ -864,6 +938,8 @@ check_gfio:
 					}
 				}
 			}
+		} else if (!main_valid && pck_size) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[FileOut] output file handle is not opened, discarding %d bytes\n", pck_size));
 		} else {
 			GF_LOG(GF_LOG_WARNING, GF_LOG_MMIO, ("[FileOut] No data associated with packet, cannot write\n"));
 		}
@@ -875,12 +951,12 @@ check_gfio:
 		if (ctx->dash_mode) {
 #ifdef GPAC_HAS_FD
 			if (ctx->fd>=0) {
-				ctx->last_file_size = lseek(ctx->fd, 0, SEEK_CUR);
+				ctx->last_file_size = lseek_64(ctx->fd, 0, SEEK_CUR);
 			} else
 #endif
 				ctx->last_file_size = gf_ftell(ctx->file);
 		}
-		fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL);
+		fileout_open_close(ctx, NULL, NULL, 0, GF_FALSE, NULL, GF_FALSE);
 	}
 	pck = gf_filter_pid_get_packet(ctx->pid);
 	if (pck)
@@ -894,6 +970,8 @@ check_gfio:
 	return e;
 }
 
+GF_Err gf_fileio_file_delete(const char *fileName, const char *parent_gfio);
+
 static Bool fileout_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 {
 	if (evt->base.type==GF_FEVT_FILE_DELETE) {
@@ -902,7 +980,15 @@ static Bool fileout_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[FileOut] null delete (file name was %s)\n", evt->file_del.url));
 		} else {
 			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[FileOut] delete file %s\n", evt->file_del.url));
-			if (ctx->use_rel) {
+
+			char *gfio_sep = NULL;
+			if (!strncmp(evt->file_del.url, "gfio://", 7)) gfio_sep = strchr(evt->file_del.url, '@');
+			if (gfio_sep) {
+				gfio_sep[0] = 0;
+				gf_fileio_file_delete(gfio_sep+1, evt->file_del.url);
+				gfio_sep[0] = '@';
+			}
+			else if (ctx->use_rel) {
 				char *fname = gf_url_concatenate(ctx->dst, evt->file_del.url);
 				gf_file_delete(fname);
 				gf_free(fname);
@@ -955,6 +1041,7 @@ static const GF_FilterArgs FileOutArgs[] =
 	{ OFFS(noinitraw), "do not produce initial segment", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_HIDE},
 	{ OFFS(max_cache_segs), "maximum number of segments cached per HAS quality when recording live sessions (0 means no limit)", GF_PROP_SINT, "0", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(force_null), "force no output regardless of file name", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(atomic), "use atomic file write for non append modes", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(use_rel), "packet filename use relative names (only set by dasher)", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_HIDE},
 	{0}
 };
@@ -975,6 +1062,9 @@ GF_FilterRegister FileOutRegister = {
 		"The output file name can use gpac templating mechanism, see `gpac -h doc`."
 		"The filter watches the property `FileNumber` on incoming packets to create new files.\n"
 		"\n"
+		"By default output files are created directly, which may lead to issues if concurrent programs attempt to access them.\n"
+		"By enabling [-atomic](), files will be created in target destination folder with the `"ATOMIC_SUFFIX"` suffix and move to their final name upon close.\n"
+		"\n"
 		"# Discard sink mode\n"
 		"When the destination is `null`, the filter is a sink dropping all input packets.\n"
 		"In this case it accepts ANY type of input PID, not just file ones.\n"
@@ -998,7 +1088,8 @@ GF_FilterRegister FileOutRegister = {
 	.finalize = fileout_finalize,
 	.configure_pid = fileout_configure_pid,
 	.process = fileout_process,
-	.process_event = fileout_process_event
+	.process_event = fileout_process_event,
+	.hint_class_type = GF_FS_CLASS_NETWORK_IO
 };
 
 
