@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Romain Bouqueau, Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2018-2023
+ *			Copyright (c) Telecom ParisTech 2018-2025
  *					All rights reserved
  *
  *  This file is part of GPAC / AV1 IVF/OBU/annexB reframer filter
@@ -47,6 +47,8 @@ typedef enum {
 	IVF,
 	//input is raw VPX
 	RAW_VPX,
+	//input is IAMF
+	IAMF,
 	UNSUPPORTED
 } AV1BitstreamSyntax;
 
@@ -86,6 +88,7 @@ typedef struct
 
 	Bool is_av1;
 	Bool is_vp9;
+	Bool is_iamf;
 	u32 codecid;
 	u32 num_frames;
 	GF_VPConfig *vp_cfg;
@@ -100,6 +103,7 @@ typedef struct
 	u32 index_alloc_size, index_size;
 
 	AV1State state;
+	IAMFState iamfstate;
 	u32 dsi_crc;
 
 	Bool pts_from_file;
@@ -108,6 +112,8 @@ typedef struct
 
 	u32 clli_crc, mdcv_crc;
 	Bool copy_props;
+
+	GF_SEILoader *sei_loader;
 } GF_AV1DmxCtx;
 
 
@@ -135,6 +141,8 @@ GF_Err av1dmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove
 		ctx->opid = gf_filter_pid_new(filter);
 		gf_filter_pid_copy_properties(ctx->opid, ctx->ipid);
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, NULL);
+		if (ctx->sei_loader)
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_SEI_LOADED, &PROP_BOOL(GF_TRUE) );
 	}
 
 	//if source has no timescale, recompute time
@@ -162,7 +170,16 @@ GF_Err av1dmx_check_format(GF_Filter *filter, GF_AV1DmxCtx *ctx, GF_BitStream *b
 	if (!ctx->state.config)
 		ctx->state.config = gf_odf_av1_cfg_new();
 
-	ctx->is_av1 = ctx->is_vp9 = GF_FALSE;
+	if (!ctx->iamfstate.config) {
+		ctx->iamfstate.config = gf_odf_iamf_cfg_new();
+		if (!ctx->iamfstate.config) return GF_OUT_OF_MEM;
+	}
+
+	ctx->is_av1 = ctx->is_vp9 = ctx->is_iamf = GF_FALSE;
+	if (ctx->sei_loader) {
+		gf_sei_loader_del(ctx->sei_loader);
+		ctx->sei_loader = NULL;
+	}
 	ctx->codecid = 0;
 	if (ctx->vp_cfg) gf_odf_vp_cfg_del(ctx->vp_cfg);
 	ctx->vp_cfg = NULL;
@@ -173,6 +190,14 @@ GF_Err av1dmx_check_format(GF_Filter *filter, GF_AV1DmxCtx *ctx, GF_BitStream *b
 	}
 
 	ctx->pts_from_file = GF_FALSE;
+	if (gf_media_probe_iamf(bs)) {
+		ctx->bsmode = IAMF;
+		ctx->is_iamf = GF_TRUE;
+		ctx->codecid = GF_CODECID_IAMF;
+		if (last_obu_end) (*last_obu_end) = (u32) gf_bs_get_position(bs);
+
+		return GF_OK;
+	}
 	if (gf_media_probe_ivf(bs)) {
 		u32 width = 0, height = 0;
 		u32 codec_fourcc = 0, timebase_den = 0, timebase_num = 0, num_frames = 0;
@@ -185,6 +210,8 @@ GF_Err av1dmx_check_format(GF_Filter *filter, GF_AV1DmxCtx *ctx, GF_BitStream *b
 		case GF_4CC('A', 'V', '0', '1'):
 			ctx->is_av1 = GF_TRUE;
 			ctx->codecid = GF_CODECID_AV1;
+			if (!ctx->sei_loader) ctx->sei_loader = gf_sei_loader_new();
+			gf_sei_init_from_av1(ctx->sei_loader, &ctx->state);
 			break;
 		case GF_4CC('V', 'P', '9', '0'):
 			ctx->is_vp9 = GF_TRUE;
@@ -292,6 +319,8 @@ GF_Err av1dmx_check_format(GF_Filter *filter, GF_AV1DmxCtx *ctx, GF_BitStream *b
 	ctx->is_av1 = GF_TRUE;
 	ctx->state.unframed = GF_TRUE;
 	ctx->codecid = GF_CODECID_AV1;
+	if (!ctx->sei_loader) ctx->sei_loader = gf_sei_loader_new();
+	gf_sei_init_from_av1(ctx->sei_loader, &ctx->state);
 	return GF_OK;
 }
 
@@ -322,7 +351,8 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	GF_Err e;
 	GF_BitStream *bs;
 	u64 duration, cur_dur, last_cdur, file_size, max_pts, last_pts, probe_size=0;
-	AV1State av1state;
+	AV1State *av1state=NULL;
+	IAMFState *iamfstate=NULL;
 	const char *filepath=NULL;
 	const GF_PropertyValue *p;
 	if (!ctx->opid || ctx->timescale || ctx->file_loaded) return;
@@ -345,7 +375,7 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		} else {
 			p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_DOWN_SIZE);
 			if (!p || (p->value.longuint > 20000000)) {
-				GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[AV1/VP9] Source file larger than 20M, skipping indexing\n"));
+				GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[AV1/VP9/IAMF] Source file larger than 20M, skipping indexing\n"));
 				if (!gf_sys_is_test_mode())
 					probe_size = 20000000;
 			} else {
@@ -364,6 +394,19 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	}
 
 	ctx->index_size = 0;
+	switch (ctx->bsmode) {
+	case IAMF:
+		GF_SAFEALLOC(iamfstate, IAMFState);
+		if (!iamfstate) {
+			return;
+		}
+		break;
+	default:
+		GF_SAFEALLOC(av1state, AV1State);
+		if (!av1state) {
+			return;
+		}
+	}
 
 	bs = gf_bs_from_file(stream, GF_BITSTREAM_READ);
 #ifndef GPAC_DISABLE_LOG
@@ -375,9 +418,18 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		gf_bs_seek(bs, ctx->file_hdr_size);
 	}
 	file_size = gf_bs_available(bs);
-	gf_av1_init_state(&av1state);
-	av1state.skip_frames = GF_TRUE;
-	av1state.config = gf_odf_av1_cfg_new();
+
+	switch (ctx->bsmode) {
+	case IAMF:
+		gf_iamf_init_state(iamfstate);
+		iamfstate->config = gf_odf_iamf_cfg_new();
+		if (!iamfstate->config) return;
+		break;
+	default:
+		gf_av1_init_state(av1state);
+		av1state->skip_frames = GF_TRUE;
+		av1state->config = gf_odf_av1_cfg_new();
+	}
 
 	max_pts = last_pts = 0;
 	duration = 0;
@@ -390,19 +442,25 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		if (probe_size && (frame_start>probe_size))
 			break;
 
-		gf_av1_reset_state(&av1state, GF_FALSE);
+		switch (ctx->bsmode) {
+		case IAMF:
+			gf_iamf_reset_state(iamfstate, GF_FALSE);
+			break;
+		default:
+			gf_av1_reset_state(av1state, GF_FALSE);
+		}
 
 		/*we process each TU and extract only the necessary OBUs*/
 		switch (ctx->bsmode) {
 		case OBUs:
-			e = aom_av1_parse_temporal_unit_from_section5(bs, &av1state);
+			e = aom_av1_parse_temporal_unit_from_section5(bs, av1state);
 			break;
 		case AnnexB:
-			e = aom_av1_parse_temporal_unit_from_annexb(bs, &av1state);
+			e = aom_av1_parse_temporal_unit_from_annexb(bs, av1state);
 			break;
 		case IVF:
 			if (ctx->is_av1) {
-				e = aom_av1_parse_temporal_unit_from_ivf(bs, &av1state);
+				e = aom_av1_parse_temporal_unit_from_ivf(bs, av1state);
 			} else {
 				u64 frame_size;
 				e = gf_media_parse_ivf_frame_header(bs, &frame_size, &pts);
@@ -411,6 +469,10 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		 		pts *= ctx->cur_fps.den;
 			}
 			break;
+		case IAMF:
+			e = aom_iamf_parse_temporal_unit(bs, iamfstate);
+			is_sap = GF_TRUE;
+			break;
 		default:
 			e = GF_NOT_SUPPORTED;
 		}
@@ -430,7 +492,7 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 			duration += ctx->cur_fps.den;
 			cur_dur += ctx->cur_fps.den;
 		}
-		if (av1state.frame_state.key_frame)
+		if (ctx->bsmode != IAMF && av1state->frame_state.key_frame)
 		 	is_sap = GF_TRUE;
 
 		//only index at I-frame start
@@ -450,18 +512,28 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		probe_size = gf_bs_get_position(bs);
 	gf_bs_del(bs);
 	gf_fclose(stream);
-	gf_odf_av1_cfg_del(av1state.config);
-	gf_av1_reset_state(&av1state, GF_TRUE);
+	switch (ctx->bsmode) {
+	case IAMF:
+		if (iamfstate->config) gf_odf_iamf_cfg_del(iamfstate->config);
+		gf_iamf_reset_state(iamfstate, GF_TRUE);
+		gf_free(iamfstate);
+		break;
+	default:
+		if (av1state->config) gf_odf_av1_cfg_del(av1state->config);
+		gf_av1_reset_state(av1state, GF_TRUE);
+		gf_free(av1state);
+	}
 
 	if (!ctx->duration.num || (ctx->duration.num  * ctx->cur_fps.num != duration * ctx->duration.den)) {
 		if (probe_size) {
 			duration *= file_size / probe_size;
 		}
 		ctx->duration.num = (s32) duration;
-		if (probe_size) ctx->duration.num = -ctx->duration.num;
 		ctx->duration.den = ctx->cur_fps.num;
 
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DURATION, & PROP_FRAC64(ctx->duration));
+		if (probe_size)
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DURATION_AVG, &PROP_BOOL(GF_TRUE) );
 
 		if (ctx->duration.num && (!gf_sys_is_test_mode() || gf_opts_get_bool("temp", "force_indexing"))) {
 			file_size *= 8 * ctx->duration.den;
@@ -559,39 +631,6 @@ static GFINLINE void av1dmx_update_cts(GF_AV1DmxCtx *ctx)
 	}
 }
 
-static void av1dmx_set_mdcv(GF_AV1DmxCtx *ctx, GF_FilterPid *pid, GF_FilterPacket *pck)
-{
-	u32 i;
-	u64 val;
-	u8 rw_mdcv[24];
-	GF_BitStream *bs_r = gf_bs_new(ctx->state.mdcv_data, 24, GF_BITSTREAM_READ);
-	GF_BitStream *bs_w = gf_bs_new(rw_mdcv, 24, GF_BITSTREAM_WRITE);
-
-	//3x{display_primaries_x, display_primaries_y} + whitePoint_x + whitePoint_y
-	//translate from AV1 representation 0.16 float to MPEG in increments of 0.00002 (1/50000)
-	for (i=0; i<8; i++) {
-		val = gf_bs_read_u16(bs_r);
-		val = (50000 * val) / 65536;
-		gf_bs_write_u16(bs_w, (u32) val);
-	}
-	//max_display_mastering_luminance: 24.8 fixed point in AV1 vs increments of 0.0001 (1/10000) candelas per square metre in MPEG
-	val = gf_bs_read_u32(bs_r);
-	val = (10000 * val) / 256;
-	gf_bs_write_u32(bs_w, (u32) val);
-
-	//min_display_mastering_luminance: 18.14 fixed point in AV1 vs increments of 0.0001 (1/10000) candelas per square metre in MPEG
-	val = gf_bs_read_u32(bs_r);
-	val = (10000 * val) / 16384;
-	gf_bs_write_u32(bs_w, (u32) val);
-	gf_bs_del(bs_r);
-	gf_bs_del(bs_w);
-
-	if (pid) {
-		gf_filter_pid_set_property(pid, GF_PROP_PID_MASTER_DISPLAY_COLOUR, &PROP_DATA(rw_mdcv, 24));
-	} else if (pck) {
-		gf_filter_pck_set_property(pck, GF_PROP_PID_MASTER_DISPLAY_COLOUR, &PROP_DATA(rw_mdcv, 24));
-	}
-}
 static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 {
 	u8 *dsi;
@@ -599,6 +638,12 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 
 	//no config or no config change
 	if (ctx->is_av1 && !gf_list_count(ctx->state.frame_state.header_obus)) return;
+	if (ctx->is_iamf && (!ctx->iamfstate.frame_state.pre_skip_is_finalized || !gf_list_count(ctx->iamfstate.frame_state.descriptor_obus))) return;
+
+	if (ctx->is_iamf) {
+		ctx->cur_fps.num = ctx->iamfstate.sample_rate;
+		ctx->cur_fps.den = ctx->iamfstate.num_samples_per_frame;
+	}
 
 	if (!ctx->opid) {
 		if (ctx->bsmode==UNSUPPORTED) return;
@@ -639,6 +684,31 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 			gf_free(dsi);
 			return;
 		}
+	} else if (ctx->is_iamf) {
+		//IAMF Descriptors changed, compute dsi
+
+		//Clear any old configOBUs - these will be repopulated from the descriptor OBUs below.
+		while (gf_list_count(ctx->iamfstate.config->configOBUs)) {
+			GF_IamfObu *a = (GF_IamfObu*) gf_list_pop_back(ctx->iamfstate.config->configOBUs);
+			if (a->raw_obu_bytes) gf_free(a->raw_obu_bytes);
+			gf_free(a);
+		}
+		ctx->iamfstate.config->configOBUs_size = 0;
+
+		dsi = NULL;
+		dsi_size = 0;
+
+		while (gf_list_count(ctx->iamfstate.frame_state.descriptor_obus)) {
+			GF_IamfObu *a = (GF_IamfObu*) gf_list_get(ctx->iamfstate.frame_state.descriptor_obus, 0);
+			gf_list_add(ctx->iamfstate.config->configOBUs, a);
+			ctx->iamfstate.config->configOBUs_size += (u32) a->obu_length;
+			gf_list_rem(ctx->iamfstate.frame_state.descriptor_obus, 0);
+		}
+
+		gf_odf_iamf_cfg_write(ctx->iamfstate.config, &dsi, &dsi_size);
+
+		// Compute the CRC of the entire iacb box.
+		crc = gf_crc_32(dsi, (u32) dsi_size);
 	}
 
 	if ((crc == ctx->dsi_crc) && !ctx->copy_props) {
@@ -652,12 +722,23 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	ctx->copy_props = GF_FALSE;
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, NULL);
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STREAM_TYPE, & PROP_UINT(GF_STREAM_VISUAL));
+
+	if (ctx->is_iamf) {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STREAM_TYPE, & PROP_UINT(GF_STREAM_AUDIO));
+		if(ctx->iamfstate.pre_skip > 0) {
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DELAY,  &PROP_LONGSINT(- (s64)ctx->iamfstate.pre_skip));
+		}
+	} else {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STREAM_TYPE, & PROP_UINT(GF_STREAM_VISUAL));
+	}
+	if (ctx->sei_loader)
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_SEI_LOADED, &PROP_BOOL(GF_TRUE) );
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, & PROP_UINT(ctx->codecid));
 	if (!ctx->timescale) {
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_TIMESCALE, & PROP_UINT(ctx->cur_fps.num));
 	}
+
 	//if we have a FPS prop, use it
 	if (!gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_FPS))
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_FPS, & PROP_FRAC(ctx->cur_fps));
@@ -700,13 +781,17 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_RANGE, & PROP_BOOL(ctx->state.color_range) );
 
 
-		if (ctx->state.clli_valid) {
-			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CONTENT_LIGHT_LEVEL, &PROP_DATA(ctx->state.clli_data, 4));
-			ctx->clli_crc = gf_crc_32(ctx->state.clli_data, 4);
+		if (ctx->state.sei.clli_valid) {
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CONTENT_LIGHT_LEVEL, &PROP_DATA(ctx->state.sei.clli_data, 4));
+			ctx->clli_crc = gf_crc_32(ctx->state.sei.clli_data, 4);
+			ctx->state.sei.clli_valid = 0;
 		}
-		if (ctx->state.mdcv_valid) {
-			av1dmx_set_mdcv(ctx, ctx->opid, NULL);
-			ctx->mdcv_crc = gf_crc_32(ctx->state.mdcv_data, 24);
+		if (ctx->state.sei.mdcv_valid) {
+			u8 rw_mdcv[24];
+			gf_av1_format_mdcv_to_mpeg(ctx->state.sei.mdcv_data, rw_mdcv);
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_MASTER_DISPLAY_COLOUR, &PROP_DATA(rw_mdcv, 24));
+			ctx->mdcv_crc = gf_crc_32(ctx->state.sei.mdcv_data, 24);
+			ctx->state.sei.mdcv_valid = 0;
 		}
 
 	}
@@ -729,7 +814,7 @@ GF_Err av1dmx_parse_ivf(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	u64 frame_size = 0, pts = GF_FILTER_NO_TS;
 	GF_FilterPacket *pck;
 	u64 pos=0, pos_ivf_hdr=0;
-	u8 *output;
+	u8 *output=NULL;
 
 	if (ctx->bsmode==IVF) {
 		pos_ivf_hdr = gf_bs_get_position(ctx->bs);
@@ -787,9 +872,8 @@ GF_Err av1dmx_parse_ivf(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	}
 
 	gf_bs_seek(ctx->bs, pos);
-	gf_bs_read_data(ctx->bs, output, pck_size);
 
-	if (output[0] & 0x80)
+	if (gf_bs_read_data(ctx->bs, output, pck_size) && (output[0] & 0x80))
 		gf_filter_pck_set_sap(pck, GF_FILTER_SAP_1);
 	else
 		gf_filter_pck_set_sap(pck, GF_FILTER_SAP_NONE);
@@ -940,11 +1024,22 @@ static GF_Err av1dmx_parse_flush_sample(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	if (!ctx->opid)
 		return GF_NON_COMPLIANT_BITSTREAM;
 
-	if (gf_bs_get_size(ctx->state.bs))
+	if (ctx->is_iamf) {
+		if (ctx->iamfstate.temporal_unit_obus) {
+			gf_free(ctx->iamfstate.temporal_unit_obus);
+			ctx->iamfstate.temporal_unit_obus = NULL;
+		}
+		gf_bs_get_content_no_truncate(ctx->iamfstate.bs, &ctx->iamfstate.temporal_unit_obus, &pck_size, &ctx->iamfstate.temporal_unit_obus_alloc);
+	} else if (ctx->state.bs && gf_bs_get_size(ctx->state.bs)) {
 		gf_bs_get_content_no_truncate(ctx->state.bs, &ctx->state.frame_obus, &pck_size, &ctx->state.frame_obus_alloc);
+	}
 
 	if (!pck_size) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] no frame OBU, skipping OBU\n"));
+		if (ctx->is_iamf) {
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] no IAMF OBUs making up a temporal unit, skipping OBUs\n"));
+		} else {
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] no frame OBU, skipping OBU\n"));
+		}
 		return GF_OK;
 	}
 
@@ -956,13 +1051,26 @@ static GF_Err av1dmx_parse_flush_sample(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 
 	gf_filter_pck_set_cts(pck, ctx->cts);
 	gf_filter_pck_set_sap(pck, ctx->state.frame_state.key_frame ? GF_FILTER_SAP_1 : 0);
+	gf_filter_pck_set_switch_frame(pck, ctx->state.frame_state.switch_frame);
 
-	memcpy(output, ctx->state.frame_obus, pck_size);
+	if (ctx->is_iamf) {
+		memcpy(output, ctx->iamfstate.temporal_unit_obus, pck_size);
+		if (ctx->iamfstate.audio_roll_distance != 0) {
+			gf_filter_pck_set_roll_info(pck, ctx->iamfstate.audio_roll_distance);
+			gf_filter_pck_set_sap(pck, GF_FILTER_SAP_4);
+		}
+		if (ctx->iamfstate.frame_state.num_samples_to_trim_at_end > 0) {
+			u64 trimmed_duration = ctx->iamfstate.num_samples_per_frame - ctx->iamfstate.frame_state.num_samples_to_trim_at_end;
+			gf_filter_pck_set_duration(pck, (u32) trimmed_duration);
+		}
+	} else {
+		memcpy(output, ctx->state.frame_obus, pck_size);
+	}
 
 	if (ctx->deps) {
 		u8 flags = 0;
 		//dependsOn
-		flags = ( ctx->state.frame_state.key_frame) ? 2 : 1;
+		flags = (ctx->state.frame_state.key_frame || ctx->state.frame_state.switch_frame) ? 2 : 1;
 		flags <<= 2;
 		//dependedOn
 	 	flags |= ctx->state.frame_state.refresh_frame_flags ? 1 : 2;
@@ -972,23 +1080,14 @@ static GF_Err av1dmx_parse_flush_sample(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	 	gf_filter_pck_set_dependency_flags(pck, flags);
 	}
 
-	if (ctx->state.clli_valid) {
-		u32 crc = gf_crc_32(ctx->state.clli_data, 4);
-		if (crc != ctx->clli_crc) {
-			gf_filter_pck_set_property(pck, GF_PROP_PID_CONTENT_LIGHT_LEVEL, &PROP_DATA(ctx->state.clli_data, 4));
-		}
-	}
-	if (ctx->state.mdcv_valid) {
-		u32 crc = gf_crc_32(ctx->state.mdcv_data, 24);
-		if (crc != ctx->mdcv_crc) {
-			av1dmx_set_mdcv(ctx, NULL, pck);
-		}
-	}
+	if (ctx->sei_loader)
+		gf_sei_load_from_state(ctx->sei_loader, pck);
 
 	gf_filter_pck_send(pck);
 
 	av1dmx_update_cts(ctx);
 	gf_av1_reset_state(&ctx->state, GF_FALSE);
+	gf_iamf_reset_state(&ctx->iamfstate, GF_FALSE);
 
 	return GF_OK;
 
@@ -1066,8 +1165,47 @@ GF_Err av1dmx_parse_av1(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		return GF_OK;
 	}
 
-	return av1dmx_parse_flush_sample(filter, ctx);
+	e = av1dmx_parse_flush_sample(filter, ctx);
+	ctx->state.sei.clli_valid = ctx->state.sei.mdcv_valid = 0;
+	return e;
+}
 
+GF_Err av1dmx_parse_iamf(GF_Filter *filter, GF_AV1DmxCtx *ctx)
+{
+	GF_Err e = GF_OK;
+
+	//first TU loaded !
+	u64 start = gf_bs_get_position(ctx->bs);
+	if (ctx->iamfstate.frame_state.found_full_temporal_unit) {
+		e = GF_OK;
+	} else {
+		e = aom_iamf_parse_temporal_unit(ctx->bs, &ctx->iamfstate);
+		if (e==GF_BUFFER_TOO_SMALL) {
+			gf_iamf_reset_state(&ctx->iamfstate, GF_FALSE);
+			gf_bs_seek(ctx->bs, start);
+		}
+	}
+
+	//check pid state
+	av1dmx_check_pid(filter, ctx);
+
+	if (e) {
+		if (e!=GF_EOS && e!=GF_BUFFER_TOO_SMALL) {
+			av1dmx_parse_flush_sample(filter, ctx);
+		}
+		return e;
+	}
+
+	if (!ctx->opid) {
+		if (ctx->iamfstate.frame_state.pre_skip_is_finalized) {
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[AV1Dmx] output pid not configured (no IAMF Descriptors yet?), skipping OBUs\n"));
+		}
+		gf_iamf_reset_state(&ctx->iamfstate, GF_FALSE);
+		return GF_OK;
+	}
+
+	e = av1dmx_parse_flush_sample(filter, ctx);
+	return e;
 }
 
 GF_Err av1dmx_process_buffer(GF_Filter *filter, GF_AV1DmxCtx *ctx, const char *data, u32 data_size, Bool is_copy)
@@ -1083,14 +1221,16 @@ GF_Err av1dmx_process_buffer(GF_Filter *filter, GF_AV1DmxCtx *ctx, const char *d
 		gf_bs_set_logger(ctx->bs, av1dmx_bs_log, ctx);
 #endif
 
-	//check ivf vs obu vs annexB
+	//check ivf vs obu vs annexB vs iamf
 	e = av1dmx_check_format(filter, ctx, ctx->bs, &last_obu_end);
 	if (e==GF_BUFFER_TOO_SMALL) return GF_OK;
 	else if (e) return e;
 
 	while (gf_bs_available(ctx->bs)) {
 
-		if (ctx->is_vp9) {
+		if (ctx->is_iamf) {
+			e = av1dmx_parse_iamf(filter, ctx);
+		} else if (ctx->is_vp9) {
 			e = av1dmx_parse_vp9(filter, ctx);
 		} else if (ctx->is_av1) {
 			e = av1dmx_parse_av1(filter, ctx);
@@ -1245,6 +1385,7 @@ static GF_Err av1dmx_initialize(GF_Filter *filter)
 	gf_av1_init_state(&ctx->state);
 	if (ctx->temporal_delim)
 		ctx->state.keep_temporal_delim = GF_TRUE;
+	gf_iamf_init_state(&ctx->iamfstate);
 
 	return GF_OK;
 }
@@ -1262,6 +1403,13 @@ static void av1dmx_finalize(GF_Filter *filter)
 	if (ctx->buffer) gf_free(ctx->buffer);
 
 	if (ctx->vp_cfg) gf_odf_vp_cfg_del(ctx->vp_cfg);
+
+	gf_iamf_reset_state(&ctx->iamfstate, GF_TRUE);
+	if (ctx->iamfstate.config) gf_odf_iamf_cfg_del(ctx->iamfstate.config);
+	if (ctx->iamfstate.bs) gf_bs_del(ctx->iamfstate.bs);
+	if (ctx->iamfstate.temporal_unit_obus) gf_free(ctx->iamfstate.temporal_unit_obus);
+	if (ctx->sei_loader)
+		gf_sei_loader_del(ctx->sei_loader);
 }
 
 static const char * av1dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeScore *score)
@@ -1273,27 +1421,36 @@ static const char * av1dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 	lt = gf_log_get_tool_level(GF_LOG_CODING);
 	gf_log_set_tool_level(GF_LOG_CODING, GF_LOG_QUIET);
 
-	res = gf_media_probe_ivf(bs);
-	if (res) {
+	if (gf_media_probe_iamf(bs)) {
+		res = GF_TRUE;
+		*score = GF_FPROBE_SUPPORTED;
+		mime = "audio/iamf";
+	} else if (gf_media_probe_ivf(bs)) {
+		res = GF_TRUE;
 		*score = GF_FPROBE_SUPPORTED;
 		mime = "video/x-ivf";
+	} else if (gf_media_aom_probe_annexb(bs)) {
+		res = GF_TRUE;
+		*score = GF_FPROBE_SUPPORTED;
 	} else {
 		res = gf_media_aom_probe_annexb(bs);
 		if (res) {
 			*score = GF_FPROBE_SUPPORTED;
 		} else {
-			AV1State state;
+			AV1State *av1_state;
 			Bool has_seq_header = GF_FALSE;
 			GF_Err e;
 			u32 nb_units = 0;
+			GF_SAFEALLOC(av1_state, AV1State);
+			if (!av1_state) return NULL;
 
-			gf_av1_init_state(&state);
-			state.config = gf_odf_av1_cfg_new();
+			gf_av1_init_state(av1_state);
+			av1_state->config = gf_odf_av1_cfg_new();
 			while (gf_bs_available(bs)) {
-				e = aom_av1_parse_temporal_unit_from_section5(bs, &state);
+				e = aom_av1_parse_temporal_unit_from_section5(bs, av1_state);
 				if ((e==GF_OK) || (nb_units && (e==GF_BUFFER_TOO_SMALL) ) ) {
-					if (!nb_units || gf_list_count(state.frame_state.header_obus) || gf_list_count(state.frame_state.frame_obus)) {
-						if (gf_list_count(state.frame_state.header_obus)) {
+					if (!nb_units || gf_list_count(av1_state->frame_state.header_obus) || gf_list_count(av1_state->frame_state.frame_obus)) {
+						if (gf_list_count(av1_state->frame_state.header_obus)) {
 							has_seq_header = GF_TRUE;
 						}
 						nb_units++;
@@ -1310,7 +1467,7 @@ static const char * av1dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 				}
 				//very large frame
 				else if (!nb_units && (e==GF_BUFFER_TOO_SMALL)) {
-					if (gf_list_count(state.frame_state.header_obus) && state.width && state.height) {
+					if (gf_list_count(av1_state->frame_state.header_obus) && av1_state->width && av1_state->height) {
 						res = GF_TRUE;
 						*score = GF_FPROBE_MAYBE_SUPPORTED;
 					}
@@ -1318,15 +1475,16 @@ static const char * av1dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 				} else {
 					break;
 				}
-				gf_av1_reset_state(&state, GF_FALSE);
+				gf_av1_reset_state(av1_state, GF_FALSE);
 				if (nb_units>2) {
 					res = GF_TRUE;
 					*score = GF_FPROBE_SUPPORTED;
 					break;
 				}
 			}
-			gf_odf_av1_cfg_del(state.config);
-			gf_av1_reset_state(&state, GF_TRUE);
+			gf_odf_av1_cfg_del(av1_state->config);
+			gf_av1_reset_state(av1_state, GF_TRUE);
+			gf_free(av1_state);
 		}
 	}
 
@@ -1349,12 +1507,23 @@ static const GF_FilterCapability AV1DmxCaps[] =
 	CAP_UINT(GF_CAPS_OUTPUT_STATIC, GF_PROP_PID_CODECID, GF_CODECID_VP10),
 	CAP_BOOL(GF_CAPS_OUTPUT_STATIC_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
 	{0},
-	CAP_UINT(GF_CAPS_INPUT,GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
-	CAP_UINT(GF_CAPS_INPUT,GF_PROP_PID_CODECID, GF_CODECID_AV1),
-	CAP_UINT(GF_CAPS_INPUT,GF_PROP_PID_CODECID, GF_CODECID_VP8),
-	CAP_UINT(GF_CAPS_INPUT,GF_PROP_PID_CODECID, GF_CODECID_VP9),
-	CAP_UINT(GF_CAPS_INPUT,GF_PROP_PID_CODECID, GF_CODECID_VP10),
-	CAP_BOOL(GF_CAPS_INPUT,GF_PROP_PID_UNFRAMED, GF_TRUE),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_AV1),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_VP8),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_VP9),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_VP10),
+	CAP_BOOL(GF_CAPS_INPUT, GF_PROP_PID_UNFRAMED, GF_TRUE),
+	{0},
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
+	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_FILE_EXT, "obu|iamf"),
+	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_MIME, "audio/iamf"),
+	CAP_UINT(GF_CAPS_OUTPUT_STATIC, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
+	CAP_UINT(GF_CAPS_OUTPUT_STATIC, GF_PROP_PID_CODECID, GF_CODECID_IAMF),
+	CAP_BOOL(GF_CAPS_OUTPUT_STATIC_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
+	{0},
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_IAMF),
+	CAP_BOOL(GF_CAPS_INPUT, GF_PROP_PID_UNFRAMED, GF_TRUE),
 };
 
 #define OFFS(_n)	#_n, offsetof(GF_AV1DmxCtx, _n)
@@ -1379,8 +1548,9 @@ static const GF_FilterArgs AV1DmxArgs[] =
 
 GF_FilterRegister AV1DmxRegister = {
 	.name = "rfav1",
-	GF_FS_SET_DESCRIPTION("AV1/IVF/VP9 reframer")
-	GF_FS_SET_HELP("This filter parses AV1 OBU, AV1 AnnexB or IVF with AV1 or VP9 files/data and outputs corresponding visual PID and frames.")
+	GF_FS_SET_DESCRIPTION("AV1/IVF/VP9/IAMF reframer")
+	GF_FS_SET_HELP("This filter parses AV1 OBU, AV1 AnnexB or IVF with AV1 or VP9 files/data and outputs corresponding visual PID and frames. "
+		       "It also parses IAMF OBU and outputs corresponding temporal units containing audio frames and parameter blocks.")
 	.private_size = sizeof(GF_AV1DmxCtx),
 	.args = AV1DmxArgs,
 	.initialize = av1dmx_initialize,
@@ -1389,7 +1559,8 @@ GF_FilterRegister AV1DmxRegister = {
 	.configure_pid = av1dmx_configure_pid,
 	.process = av1dmx_process,
 	.probe_data = av1dmx_probe_data,
-	.process_event = av1dmx_process_event
+	.process_event = av1dmx_process_event,
+	.hint_class_type = GF_FS_CLASS_FRAMING
 };
 
 
@@ -1403,4 +1574,3 @@ const GF_FilterRegister *rfav1_register(GF_FilterSession *session)
 	return NULL;
 }
 #endif // #if !defined(GPAC_DISABLE_AV_PARSERS) && !defined(GPAC_DISABLE_RFAV1)
-
