@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2023
+ *			Copyright (c) Telecom ParisTech 2000-2025
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -232,8 +232,12 @@ GF_Box *gf_isom_audio_sample_get_audio_codec_cfg_box(GF_AudioSampleEntryBox *ptr
 	case GF_ISOM_BOX_TYPE_AC3:
 	case GF_ISOM_BOX_TYPE_EC3:
 		return (GF_Box *)mpga->cfg_ac3;
+	case GF_ISOM_BOX_TYPE_AC4:
+		return (GF_Box *)mpga->cfg_ac4;
 	case GF_ISOM_BOX_TYPE_OPUS:
 		return (GF_Box *)mpga->cfg_opus;
+	case GF_ISOM_BOX_TYPE_IAMF:
+		return (GF_Box *)mpga->cfg_iamf;
 	case GF_ISOM_BOX_TYPE_MHA1:
 	case GF_ISOM_BOX_TYPE_MHA2:
 		return (GF_Box *)mpga->cfg_mha;
@@ -302,7 +306,7 @@ GF_3GPConfig *gf_isom_3gp_config_get(GF_ISOFile *the_file, u32 trackNumber, u32
 	GF_3GPConfig *config, *res;
 	GF_TrackBox *trak;
 	GF_SampleEntryBox *entry;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !StreamDescriptionIndex) return NULL;
 
 	config = NULL;
@@ -340,7 +344,7 @@ GF_AC3Config *gf_isom_ac3_config_get(GF_ISOFile *the_file, u32 trackNumber, u32
 	GF_AC3Config *res;
 	GF_TrackBox *trak;
 	GF_MPEGAudioSampleEntryBox *entry;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !StreamDescriptionIndex) return NULL;
 
 	entry = (GF_MPEGAudioSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
@@ -355,13 +359,36 @@ GF_AC3Config *gf_isom_ac3_config_get(GF_ISOFile *the_file, u32 trackNumber, u32
 	return res;
 }
 
+GF_EXPORT
+GF_AC4Config *gf_isom_ac4_config_get(GF_ISOFile *the_file, u32 trackNumber, u32 StreamDescriptionIndex)
+{
+	GF_AC4Config *res;
+	GF_TrackBox *trak;
+	GF_MPEGAudioSampleEntryBox *entry;
+	trak = gf_isom_get_track_box(the_file, trackNumber);
+	if (!trak || !StreamDescriptionIndex) return NULL;
+
+	entry = (GF_MPEGAudioSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
+	if (!entry) return NULL;
+	if (!entry->cfg_ac4) return NULL;
+	if (entry->internal_type != GF_ISOM_SAMPLE_ENTRY_AUDIO) return NULL;
+	if (entry->cfg_ac4->type != GF_ISOM_BOX_TYPE_DAC4) return NULL;
+
+	res = (GF_AC4Config*)gf_malloc(sizeof(GF_AC4Config));
+	if (res) {
+		gf_odf_ac4_cfg_deep_copy(res, &entry->cfg_ac4->cfg);
+		if (!res->channel_count) res->channel_count = entry->channel_count;
+	}
+	return res;
+}
+
 GF_EXPORT
 GF_Err gf_isom_flac_config_get(GF_ISOFile *the_file, u32 trackNumber, u32 StreamDescriptionIndex, u8 **dsi, u32 *dsi_size)
 {
 	u32 type;
 	GF_TrackBox *trak;
 	GF_MPEGAudioSampleEntryBox *entry;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (dsi) *dsi = NULL;
 	if (dsi_size) *dsi_size = 0;
 	if (!trak || !StreamDescriptionIndex) return GF_BAD_PARAM;
@@ -390,7 +417,7 @@ GF_Err gf_isom_truehd_config_get(GF_ISOFile *the_file, u32 trackNumber, u32 Stre
 {
 	GF_TrackBox *trak;
 	GF_MPEGAudioSampleEntryBox *entry;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !StreamDescriptionIndex) return GF_BAD_PARAM;
 
 	entry = (GF_MPEGAudioSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
@@ -405,6 +432,7 @@ GF_Err gf_isom_truehd_config_get(GF_ISOFile *the_file, u32 trackNumber, u32 Stre
 }
 
 #ifndef GPAC_DISABLE_ISOM_WRITE
+GF_EXPORT
 GF_Err gf_isom_truehd_config_new(GF_ISOFile *the_file, u32 trackNumber, char *URLname, char *URNname, u32 format, u32 peak_rate, u32 *outDescriptionIndex)
 {
 	GF_TrackBox *trak;
@@ -413,10 +441,10 @@ GF_Err gf_isom_truehd_config_new(GF_ISOFile *the_file, u32 trackNumber, char *UR
 	GF_MPEGAudioSampleEntryBox *entry;
 	GF_SampleDescriptionBox *stsd;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	//get or create the data ref
@@ -454,10 +482,10 @@ GF_Err gf_isom_opus_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_OpusCon
 	GF_SampleDescriptionBox *stsd;
 
 	if (!cfg) return GF_BAD_PARAM;
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	//get or create the data ref
@@ -490,7 +518,7 @@ static GF_Err gf_isom_opus_config_get_internal(GF_ISOFile *the_file, u32 trackNu
 	u32 type;
 	GF_TrackBox *trak;
 	GF_MPEGAudioSampleEntryBox *entry;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (dsi) *dsi = NULL;
 	if (dsi_size) *dsi_size = 0;
 	if (ocfg) memset(ocfg, 0, sizeof(GF_OpusConfig));
@@ -544,10 +572,10 @@ GF_Err gf_isom_3gp_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_3GPConfi
 	u32 cfg_type;
 	GF_SampleDescriptionBox *stsd;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media || !cfg) return GF_BAD_PARAM;
 
 	switch (cfg->type) {
@@ -637,10 +665,10 @@ GF_Err gf_isom_3gp_config_update(GF_ISOFile *the_file, u32 trackNumber, GF_3GPCo
 	GF_MPEGAudioSampleEntryBox *a_entry;
 	GF_MPEGVisualSampleEntryBox *v_entry;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media || !param || !DescriptionIndex) return GF_BAD_PARAM;
 
 	cfg = NULL;
@@ -677,10 +705,10 @@ GF_Err gf_isom_ac3_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_AC3Confi
 	u32 dataRefIndex;
 	GF_MPEGAudioSampleEntryBox *entry;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media || !cfg) return GF_BAD_PARAM;
 
 	//get or create the data ref
@@ -721,10 +749,10 @@ GF_Err gf_isom_ac3_config_update(GF_ISOFile *the_file, u32 trackNumber, u32 samp
 	GF_Err e;
 	GF_MPEGAudioSampleEntryBox *entry;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media || !cfg || !sampleDescriptionIndex) return GF_BAD_PARAM;
 
 	if (!the_file->keep_utc)
@@ -743,6 +771,70 @@ GF_Err gf_isom_ac3_config_update(GF_ISOFile *the_file, u32 trackNumber, u32 samp
 	return GF_OK;
 }
 
+GF_EXPORT
+GF_Err gf_isom_ac4_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_AC4Config *cfg, const char *URLname, const char *URNname, u32 *outDescriptionIndex)
+{
+	GF_TrackBox *trak;
+	GF_Err e;
+	u32 dataRefIndex;
+	GF_MPEGAudioSampleEntryBox *entry;
+
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
+	if (e) return e;
+
+	trak = gf_isom_get_track_box(the_file, trackNumber);
+	if (!trak || !trak->Media || !cfg) return GF_BAD_PARAM;
+
+	//get or create the data ref
+	e = Media_FindDataRef(trak->Media->information->dataInformation->dref, (char *)URLname, (char *)URNname, &dataRefIndex);
+	if (e) return e;
+	if (!dataRefIndex) {
+		e = Media_CreateDataRef(the_file, trak->Media->information->dataInformation->dref, (char *)URLname, (char *)URNname, &dataRefIndex);
+		if (e) return e;
+	}
+	if (!the_file->keep_utc)
+		trak->Media->mediaHeader->modificationTime = gf_isom_get_mp4time();
+
+	entry = (GF_MPEGAudioSampleEntryBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_AC4);
+	if (!entry) return GF_OUT_OF_MEM;
+	entry->cfg_ac4 = (GF_AC4ConfigBox *) gf_isom_box_new_parent(&entry->child_boxes, GF_ISOM_BOX_TYPE_DAC4);
+
+	if (!entry->cfg_ac4) {
+		gf_isom_box_del((GF_Box *) entry);
+		return GF_OUT_OF_MEM;
+	}
+	gf_odf_ac4_cfg_deep_copy(&entry->cfg_ac4->cfg, cfg);
+	entry->samplerate_hi = trak->Media->mediaHeader->timeScale;
+	entry->dataReferenceIndex = dataRefIndex;
+	e = gf_list_add(trak->Media->information->sampleTable->SampleDescription->child_boxes, entry);
+	*outDescriptionIndex = gf_list_count(trak->Media->information->sampleTable->SampleDescription->child_boxes);
+	return e;
+}
+
+GF_EXPORT
+GF_Err gf_isom_ac4_config_update(GF_ISOFile *the_file, u32 trackNumber, u32 sampleDescriptionIndex, GF_AC4Config *cfg)
+{
+	GF_TrackBox *trak;
+	GF_Err e;
+	GF_MPEGAudioSampleEntryBox *entry;
+
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
+	if (e) return e;
+
+	trak = gf_isom_get_track_box(the_file, trackNumber);
+	if (!trak || !trak->Media || !cfg || !sampleDescriptionIndex) return GF_BAD_PARAM;
+
+	if (!the_file->keep_utc)
+		trak->Media->mediaHeader->modificationTime = gf_isom_get_mp4time();
+
+	entry = gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, sampleDescriptionIndex-1);
+	if (!entry) return GF_BAD_PARAM;
+	if (!entry->cfg_ac4) return GF_BAD_PARAM;
+
+	gf_odf_ac4_cfg_deep_copy(&entry->cfg_ac4->cfg, cfg);
+	return GF_OK;
+}
+
 GF_EXPORT
 GF_Err gf_isom_flac_config_new(GF_ISOFile *the_file, u32 trackNumber, u8 *metadata, u32 metadata_size, const char *URLname, const char *URNname, u32 *outDescriptionIndex)
 {
@@ -751,10 +843,10 @@ GF_Err gf_isom_flac_config_new(GF_ISOFile *the_file, u32 trackNumber, u8 *metada
 	u32 dataRefIndex;
 	GF_MPEGAudioSampleEntryBox *entry;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	//get or create the data ref
@@ -776,8 +868,11 @@ GF_Err gf_isom_flac_config_new(GF_ISOFile *the_file, u32 trackNumber, u8 *metada
 		return GF_OUT_OF_MEM;
 	}
 	entry->cfg_flac->dataSize = metadata_size;
-	entry->cfg_flac->data = gf_malloc(sizeof(u8)*metadata_size);
-	memcpy(entry->cfg_flac->data, metadata, sizeof(u8)*metadata_size);
+	entry->cfg_flac->data = NULL;
+	if (metadata && metadata_size) {
+		entry->cfg_flac->data = gf_malloc(sizeof(u8)*metadata_size);
+		memcpy(entry->cfg_flac->data, metadata, sizeof(u8)*metadata_size);
+	}
 	entry->samplerate_hi = trak->Media->mediaHeader->timeScale;
 	entry->dataReferenceIndex = dataRefIndex;
 	e = gf_list_add(trak->Media->information->sampleTable->SampleDescription->child_boxes, entry);
@@ -794,10 +889,10 @@ GF_Err gf_isom_new_mj2k_description(GF_ISOFile *the_file, u32 trackNumber, const
 	GF_Err e;
 	u32 dataRefIndex=0;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	//get or create the data ref
@@ -837,7 +932,7 @@ GF_Err gf_isom_get_dims_description(GF_ISOFile *movie, u32 trackNumber, u32 desc
 {
 	GF_DIMSSampleEntryBox *dims;
 	GF_TrackBox *trak;
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !descriptionIndex || !desc) return GF_BAD_PARAM;
 
 	dims = (GF_DIMSSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, descriptionIndex-1);
@@ -871,10 +966,10 @@ GF_Err gf_isom_new_dims_description(GF_ISOFile *movie, u32 trackNumber, GF_DIMSD
 	u32 dataRefIndex;
 	GF_DIMSSampleEntryBox *dims;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	if (trak->Media->handler->handlerType != GF_ISOM_MEDIA_SCENE) return GF_BAD_PARAM;
@@ -923,10 +1018,10 @@ GF_Err gf_isom_update_dims_description(GF_ISOFile *movie, u32 trackNumber, GF_DI
 	GF_Err e;
 	GF_DIMSSampleEntryBox *dims;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media || !desc || !DescriptionIndex) return GF_BAD_PARAM;
 
 	dims = (GF_DIMSSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, DescriptionIndex-1);
@@ -975,7 +1070,7 @@ GF_EXPORT
 GF_Err gf_isom_get_udts_config(GF_ISOFile *movie, u32 trackNumber, u32 descriptionIndex, GF_UDTSConfig *cfg)
 {
 	GF_Box *sample_entry;
-	GF_TrackBox *trak = gf_isom_get_track_from_file(movie, trackNumber);
+	GF_TrackBox *trak = gf_isom_get_track_box(movie, trackNumber);
 	GF_UDTSSpecificBox *udts = NULL;
 
 	if (!trak || !descriptionIndex || !cfg) return GF_BAD_PARAM;
@@ -1002,7 +1097,7 @@ GF_Err gf_isom_get_udts_config(GF_ISOFile *movie, u32 trackNumber, u32 descripti
 
 GF_Err LSR_UpdateESD(GF_LASeRSampleEntryBox *lsr, GF_ESD *esd)
 {
-	GF_BitRateBox *btrt = gf_isom_sample_entry_get_bitrate((GF_SampleEntryBox *)lsr, GF_TRUE);
+	GF_BitRateBox *btrt = gf_isom_sample_entry_get_bitrate_box((GF_SampleEntryBox *)lsr, GF_TRUE);
 
 	if (lsr->descr) gf_isom_box_del_parent(&lsr->child_boxes, (GF_Box *) lsr->descr);
 	lsr->descr = NULL;
@@ -1076,7 +1171,7 @@ GF_Err gf_isom_get_xml_metadata_description(GF_ISOFile *file, u32 track, u32 sam
 	if (_namespace) *_namespace = NULL;
 	if (content_encoding) *content_encoding = NULL;
 	if (schema_loc) *schema_loc = NULL;
-	trak = gf_isom_get_track_from_file(file, track);
+	trak = gf_isom_get_track_box(file, track);
 	if (!trak || !sampleDescription) return GF_BAD_PARAM;
 	ptr = (GF_MetaDataSampleEntryBox*)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, sampleDescription-1);
 	if (!ptr) return GF_BAD_PARAM;
@@ -1089,6 +1184,7 @@ GF_Err gf_isom_get_xml_metadata_description(GF_ISOFile *file, u32 track, u32 sam
 
 #ifndef GPAC_DISABLE_ISOM_WRITE
 
+GF_EXPORT
 GF_Err gf_isom_new_xml_metadata_description(GF_ISOFile *movie, u32 trackNumber,
         const char *_namespace, const char *schema_loc, const char *content_encoding,
         u32 *outDescriptionIndex)
@@ -1100,10 +1196,10 @@ GF_Err gf_isom_new_xml_metadata_description(GF_ISOFile *movie, u32 trackNumber,
 	char *URLname = NULL;
 	char *URNname = NULL;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media || !_namespace)
 		return GF_BAD_PARAM;
 
@@ -1152,12 +1248,12 @@ GF_Err gf_isom_xml_subtitle_get_description(GF_ISOFile *the_file, u32 trackNumbe
 	if (xmlnamespace) *xmlnamespace = NULL;
 	if (xml_schema_loc) *xml_schema_loc = NULL;
 	if (mimes) *mimes = NULL;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !StreamDescriptionIndex) return GF_BAD_PARAM;
 
 	entry = (GF_MetaDataSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
 	if (!entry) return GF_BAD_PARAM;
- 
+
 	if ((entry->type!=GF_ISOM_BOX_TYPE_STPP) && (entry->type!=GF_ISOM_BOX_TYPE_METX)) {
 		return GF_BAD_PARAM;
 	}
@@ -1180,7 +1276,7 @@ const char *gf_isom_subtitle_get_mime(GF_ISOFile *the_file, u32 trackNumber, u32
 	GF_TrackBox *trak;
 	GF_TextConfigBox *mime;
 	GF_MetaDataSampleEntryBox *entry;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !StreamDescriptionIndex) return NULL;
 
 	entry = (GF_MetaDataSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
@@ -1202,10 +1298,10 @@ GF_Err gf_isom_subtitle_set_mime(GF_ISOFile *the_file, u32 trackNumber, u32 Stre
 	GF_TextConfigBox *mime;
 	GF_MetaDataSampleEntryBox *entry;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !StreamDescriptionIndex) return GF_BAD_PARAM;
 
 	entry = (GF_MetaDataSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
@@ -1238,10 +1334,10 @@ GF_Err gf_isom_new_xml_subtitle_description(GF_ISOFile  *movie, u32 trackNumber,
 	char *URLname = NULL;
 	char *URNname = NULL;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	switch (trak->Media->handler->handlerType) {
@@ -1294,7 +1390,7 @@ GF_Err gf_isom_stxt_get_description(GF_ISOFile *the_file, u32 trackNumber, u32 S
 	if (mime) *mime = NULL;
 	if (config) *config = NULL;
 	if (encoding) *encoding = NULL;
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !StreamDescriptionIndex) return GF_BAD_PARAM;
 
 	entry = (GF_MetaDataSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
@@ -1318,6 +1414,7 @@ GF_Err gf_isom_stxt_get_description(GF_ISOFile *the_file, u32 trackNumber, u32 S
 }
 
 #ifndef GPAC_DISABLE_ISOM_WRITE
+GF_EXPORT
 GF_Err gf_isom_new_stxt_description(GF_ISOFile *movie, u32 trackNumber, u32 type,
                                     const char *mime, const char *encoding, const char * config,
                                     u32 *outDescriptionIndex)
@@ -1329,10 +1426,10 @@ GF_Err gf_isom_new_stxt_description(GF_ISOFile *movie, u32 trackNumber, u32 type
 	char *URLname = NULL;
 	char *URNname = NULL;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	switch (trak->Media->handler->handlerType) {
@@ -1405,10 +1502,10 @@ GF_Err gf_isom_update_stxt_description(GF_ISOFile *movie, u32 trackNumber,
 	GF_Err e;
 	GF_MetaDataSampleEntryBox *sample_entry;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media || !DescriptionIndex) return GF_BAD_PARAM;
 
 	sample_entry = (GF_MetaDataSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, DescriptionIndex-1);
@@ -1453,7 +1550,7 @@ GF_WebVTTSampleEntryBox *gf_webvtt_isom_get_description(GF_ISOFile *movie, u32 t
 
 	if (!descriptionIndex) return NULL;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media) return NULL;
 
 	switch (trak->Media->handler->handlerType) {
@@ -1504,10 +1601,10 @@ GF_Err gf_isom_update_webvtt_description(GF_ISOFile *movie, u32 trackNumber, u32
 	GF_WebVTTSampleEntryBox *wvtt;
 	GF_TrackBox *trak;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return GF_BAD_PARAM;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	switch (trak->Media->handler->handlerType) {
@@ -1536,7 +1633,7 @@ GF_Err gf_isom_update_webvtt_description(GF_ISOFile *movie, u32 trackNumber, u32
 }
 #endif
 
-
+GF_EXPORT
 GF_Err gf_isom_new_webvtt_description(GF_ISOFile *movie, u32 trackNumber, const char *URLname, const char *URNname, u32 *outDescriptionIndex, const char *config)
 {
 	GF_TrackBox *trak;
@@ -1544,10 +1641,10 @@ GF_Err gf_isom_new_webvtt_description(GF_ISOFile *movie, u32 trackNumber, const
 	u32 dataRefIndex;
 	GF_WebVTTSampleEntryBox *wvtt;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	switch (trak->Media->handler->handlerType) {
@@ -1584,7 +1681,7 @@ GF_Err gf_isom_new_webvtt_description(GF_ISOFile *movie, u32 trackNumber, const
 #endif /*GPAC_DISABLE_VTT*/
 #endif //GPAC_DISABLE_ISOM_WRITE
 
-GF_BitRateBox *gf_isom_sample_entry_get_bitrate(GF_SampleEntryBox *ent, Bool create)
+GF_BitRateBox *gf_isom_sample_entry_get_bitrate_box(GF_SampleEntryBox *ent, Bool create)
 {
 	u32 i=0;
 	GF_BitRateBox *a;
@@ -1599,7 +1696,7 @@ GF_BitRateBox *gf_isom_sample_entry_get_bitrate(GF_SampleEntryBox *ent, Bool cre
 #ifndef GPAC_DISABLE_ISOM_WRITE
 
 GF_EXPORT
-GF_Err gf_isom_update_bitrate(GF_ISOFile *movie, u32 trackNumber, u32 sampleDescriptionIndex, u32 average_bitrate, u32 max_bitrate, u32 decode_buffer_size)
+GF_Err gf_isom_update_bitrate_ex(GF_ISOFile *movie, u32 trackNumber, u32 sampleDescriptionIndex, u32 average_bitrate, u32 max_bitrate, u32 decode_buffer_size, Bool forced_for_mpeg4)
 {
 	GF_BitRateBox *a;
 	GF_Err e;
@@ -1607,10 +1704,10 @@ GF_Err gf_isom_update_bitrate(GF_ISOFile *movie, u32 trackNumber, u32 sampleDesc
 	u32 i, count;
 	GF_TrackBox *trak;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return GF_BAD_PARAM;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
@@ -1653,12 +1750,13 @@ GF_Err gf_isom_update_bitrate(GF_ISOFile *movie, u32 trackNumber, u32 sampleDesc
 				if (decode_buffer_size)
 					esds->desc->decoderConfig->bufferSizeDB = decode_buffer_size;
 			}
-			continue;
+			if (!forced_for_mpeg4)
+				continue;
 		}
-		
+
 		//using BTRT
 		if (!max_bitrate && average_bitrate) max_bitrate = average_bitrate;
-		a = gf_isom_sample_entry_get_bitrate(ent, max_bitrate ? GF_TRUE : GF_FALSE);
+		a = gf_isom_sample_entry_get_bitrate_box(ent, max_bitrate ? GF_TRUE : GF_FALSE);
 
 		if (!max_bitrate) {
 			if (a) {
@@ -1674,6 +1772,13 @@ GF_Err gf_isom_update_bitrate(GF_ISOFile *movie, u32 trackNumber, u32 sampleDesc
 	return GF_OK;
 }
 
+GF_EXPORT
+GF_Err gf_isom_update_bitrate(GF_ISOFile *movie, u32 trackNumber, u32 sampleDescriptionIndex, u32 average_bitrate, u32 max_bitrate, u32 decode_buffer_size)
+{
+
+	return gf_isom_update_bitrate_ex(movie, trackNumber, sampleDescriptionIndex, average_bitrate, max_bitrate, decode_buffer_size, GF_FALSE);
+
+}
 
 GF_EXPORT
 GF_Err gf_isom_tmcd_config_new(GF_ISOFile *the_file, u32 trackNumber, u32 fps_num, u32 fps_den, s32 frames_per_counter_tick, Bool is_drop, Bool is_counter, u32 *outDescriptionIndex)
@@ -1686,10 +1791,10 @@ GF_Err gf_isom_tmcd_config_new(GF_ISOFile *the_file, u32 trackNumber, u32 fps_nu
 	GF_TimeCodeMediaInformationBox *tcmi;
 	GF_TimeCodeSampleEntryBox *entry;
 
-	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	trak = gf_isom_get_track_box(the_file, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	//get or create the data ref
@@ -1730,13 +1835,47 @@ GF_Err gf_isom_tmcd_config_new(GF_ISOFile *the_file, u32 trackNumber, u32 fps_nu
 	entry->timescale = fps_num;
 	entry->frame_duration = fps_den;
 	entry->frames_per_counter_tick = (u8) frames_per_counter_tick;
-	
+
 	entry->dataReferenceIndex = dataRefIndex;
 	e = gf_list_add(trak->Media->information->sampleTable->SampleDescription->child_boxes, entry);
 	*outDescriptionIndex = gf_list_count(trak->Media->information->sampleTable->SampleDescription->child_boxes);
 	return e;
 }
 
+GF_EXPORT
+GF_Err gf_isom_evte_config_new(GF_ISOFile *the_file, u32 trackNumber, u32 *outDescriptionIndex)
+{
+	GF_TrackBox *trak;
+	GF_Err e;
+	u32 dataRefIndex;
+	GF_EventMessageSampleEntryBox *entry;
+
+	e = gf_isom_can_access_movie(the_file, GF_ISOM_OPEN_WRITE);
+	if (e) return e;
+
+	trak = gf_isom_get_track_box(the_file, trackNumber);
+	if (!trak || !trak->Media) return GF_BAD_PARAM;
+
+	//get or create the data ref
+	e = Media_FindDataRef(trak->Media->information->dataInformation->dref, NULL, NULL, &dataRefIndex);
+	if (e) return e;
+	if (!dataRefIndex) {
+		e = Media_CreateDataRef(the_file, trak->Media->information->dataInformation->dref, NULL, NULL, &dataRefIndex);
+		if (e) return e;
+	}
+	if (!the_file->keep_utc)
+		trak->Media->mediaHeader->modificationTime = gf_isom_get_mp4time();
+
+	entry = (GF_EventMessageSampleEntryBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_EVTE);
+	if (!entry) return GF_OUT_OF_MEM;
+	entry->dataReferenceIndex = dataRefIndex;
+	gf_list_add(trak->Media->information->sampleTable->SampleDescription->child_boxes, entry);
+	if (outDescriptionIndex) *outDescriptionIndex = gf_list_count(trak->Media->information->sampleTable->SampleDescription->child_boxes);
+
+	return e;
+}
+
+GF_EXPORT
 GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const char *URLname, const char *URNname, u32 *outDescriptionIndex, u8 *dsi, u32 dsi_size, u32 mha_subtype)
 {
 	GF_TrackBox *trak;
@@ -1744,10 +1883,10 @@ GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const ch
 	u32 dataRefIndex;
 	GF_MPEGAudioSampleEntryBox *mpa;
 
-	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
+	e = gf_isom_can_access_movie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	switch (trak->Media->handler->handlerType) {
@@ -1786,6 +1925,9 @@ GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const ch
 	if (outDescriptionIndex) *outDescriptionIndex = gf_list_count(trak->Media->information->sampleTable->SampleDescription->child_boxes);
 
 	if (dsi) {
+		if (dsi_size <=5)
+			return GF_BAD_PARAM;
+
 		mpa->cfg_mha = (GF_MHAConfigBox *) gf_isom_box_new_parent(&mpa->child_boxes, GF_ISOM_BOX_TYPE_MHAC);
 		if (!mpa->cfg_mha) return GF_OUT_OF_MEM;
 		mpa->cfg_mha->configuration_version = dsi[0];
@@ -1794,6 +1936,10 @@ GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const ch
 		mpa->cfg_mha->mha_config_size = dsi[3];
 		mpa->cfg_mha->mha_config_size <<= 8;
 		mpa->cfg_mha->mha_config_size |= dsi[4];
+
+		if (mpa->cfg_mha->mha_config_size < dsi_size-5)
+			return GF_BAD_PARAM;
+
 		mpa->cfg_mha->mha_config = gf_malloc(sizeof(u8) * mpa->cfg_mha->mha_config_size);
 		if (!mpa->cfg_mha->mha_config) return GF_OUT_OF_MEM;
 		memcpy(mpa->cfg_mha->mha_config, dsi+5, dsi_size-5);
@@ -1808,7 +1954,7 @@ GF_Err gf_isom_get_tmcd_config(GF_ISOFile *movie, u32 trackNumber, u32 descripti
 {
 	GF_TimeCodeSampleEntryBox *tmcd;
 	GF_TrackBox *trak;
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !descriptionIndex) return GF_BAD_PARAM;
 
 	tmcd = (GF_TimeCodeSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, descriptionIndex-1);
@@ -1828,7 +1974,7 @@ GF_Err gf_isom_get_pcm_config(GF_ISOFile *movie, u32 trackNumber, u32 descriptio
 	GF_AudioSampleEntryBox *aent;
 	GF_PCMConfigBox *pcmC;
 	GF_TrackBox *trak;
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !descriptionIndex) return GF_BAD_PARAM;
 
 	aent = (GF_AudioSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, descriptionIndex-1);
@@ -1871,7 +2017,7 @@ GF_Err gf_isom_get_lpcm_config(GF_ISOFile *movie, u32 trackNumber, u32 descripti
 	GF_AudioSampleEntryBox *aent;
 	GF_TrackBox *trak;
 	GF_BitStream *bs;
-	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	trak = gf_isom_get_track_box(movie, trackNumber);
 	if (!trak || !descriptionIndex) return GF_BAD_PARAM;
 
 	aent = (GF_AudioSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, descriptionIndex-1);
