@@ -165,12 +165,12 @@ u32 gf_dolby_vision_level(u32 width, u32 height, u64 fps_num, u64 fps_den, u32 c
 		else if (level_check <= 3840*2160*30) dv_level = 7;
 		else if (level_check <= 3840*2160*48) dv_level = 8;
 		else if (level_check <= 3840*2160*60) dv_level = 9;
-        else if (level_check <= 3840*2160*120) {
-            if (width == 7680) dv_level = 11;
-            else dv_level = 10;
-        }
-        else if (level_check <= 7680*4320*60) dv_level = 12;
-        else dv_level = 13;
+		else if (level_check <= 3840*2160*120) {
+			if (width == 7680) dv_level = 11;
+			else dv_level = 10;
+		}
+		else if (level_check <= 7680*4320*60) dv_level = 12;
+		else dv_level = 13;
 	}
 	return dv_level;
 }
@@ -300,6 +300,7 @@ static GF_Err gf_import_isomedia_track(GF_MediaImporter *import)
 	GF_Err e;
 	u64 offset, sampDTS, duration, dts_offset;
 	Bool is_nalu_video=GF_FALSE;
+	u32 inject_rap_sample_group = 0;
 	u32 track, di, trackID, track_in, i, num_samples, mtype, w, h, sr, sbr_sr, ch, mstype, cur_extract_mode, cdur, bps;
 	u64 mtimescale;
 	s32 trans_x, trans_y;
@@ -383,51 +384,82 @@ static GF_Err gf_import_isomedia_track(GF_MediaImporter *import)
 	}
 	mtype = gf_isom_get_media_type(import->orig, track_in);
 	if (mtype==GF_ISOM_MEDIA_VISUAL) {
+		Bool skip_profile=GF_FALSE;
 		u8 PL = iod ? iod->visual_profileAndLevel : 0xFE;
 		gf_isom_get_visual_info(import->orig, track_in, 1, &w, &h);
-#ifndef GPAC_DISABLE_AV_PARSERS
 		/*for MPEG-4 visual, always check size (don't trust input file)*/
-		if (origin_esd
-			&& origin_esd->decoderConfig
-			&& (origin_esd->decoderConfig->objectTypeIndication==GF_CODECID_MPEG4_PART2)
-		) {
-			if (origin_esd->decoderConfig->decoderSpecificInfo) {
-				GF_M4VDecSpecInfo dsi;
-				gf_m4v_get_config(origin_esd->decoderConfig->decoderSpecificInfo->data, origin_esd->decoderConfig->decoderSpecificInfo->dataLength, &dsi);
-				w = dsi.width;
-				h = dsi.height;
-				PL = dsi.VideoPL;
-			} else {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("Missing DecoderSpecificInfo in MPEG-4 Visual (Part2) stream\n"));
+		if (origin_esd && origin_esd->decoderConfig) {
+			switch (origin_esd->decoderConfig->objectTypeIndication) {
+#ifndef GPAC_DISABLE_AV_PARSERS
+			case GF_CODECID_MPEG4_PART2:
+				if (origin_esd->decoderConfig->decoderSpecificInfo) {
+					GF_M4VDecSpecInfo dsi;
+					gf_m4v_get_config(origin_esd->decoderConfig->decoderSpecificInfo->data, origin_esd->decoderConfig->decoderSpecificInfo->dataLength, &dsi);
+					w = dsi.width;
+					h = dsi.height;
+					PL = dsi.VideoPL;
+				} else {
+					GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("Missing DecoderSpecificInfo in MPEG-4 Visual (Part2) stream\n"));
+				}
+				break;
+#endif
+			case GF_CODECID_MPEG1:
+			case GF_CODECID_MPEG2_422:
+			case GF_CODECID_MPEG2_SNR:
+			case GF_CODECID_MPEG2_HIGH:
+			case GF_CODECID_MPEG2_MAIN:
+			case GF_CODECID_MPEG2_SIMPLE:
+			case GF_CODECID_MPEG2_SPATIAL:
+			case GF_CODECID_AVC:
+			case GF_CODECID_SVC:
+			case GF_CODECID_MVC:
+				break;
+			default:
+				skip_profile=GF_TRUE;
+				break;
 			}
 		}
-#endif
-		gf_isom_set_pl_indication(import->dest, GF_ISOM_PL_VISUAL, PL);
+		if (!skip_profile)
+			gf_isom_set_pl_indication(import->dest, GF_ISOM_PL_VISUAL, PL);
 	}
 	else if (mtype==GF_ISOM_MEDIA_AUDIO) {
+		Bool skip_profile = GF_FALSE;
 		u8 PL = iod ? iod->audio_profileAndLevel : 0xFE;
 		bps = 16;
 		sr = ch = sbr_sr = 0;
 		sbr = GF_FALSE;
 		ps = GF_FALSE;
 		gf_isom_get_audio_info(import->orig, track_in, 1, &sr, &ch, &bps);
+		if (origin_esd && origin_esd->decoderConfig) {
+			switch (origin_esd->decoderConfig->objectTypeIndication) {
+			case GF_CODECID_AAC_MPEG4:
+			case GF_CODECID_AAC_MPEG2_MP:
+			case GF_CODECID_AAC_MPEG2_LCP:
+			case GF_CODECID_AAC_MPEG2_SSRP:
 #ifndef GPAC_DISABLE_AV_PARSERS
-		if (origin_esd && origin_esd->decoderConfig && (origin_esd->decoderConfig->objectTypeIndication==GF_CODECID_AAC_MPEG4)) {
-			if (origin_esd->decoderConfig->decoderSpecificInfo) {
-				GF_M4ADecSpecInfo dsi;
-				gf_m4a_get_config(origin_esd->decoderConfig->decoderSpecificInfo->data, origin_esd->decoderConfig->decoderSpecificInfo->dataLength, &dsi);
-				sr = dsi.base_sr;
-				if (dsi.has_sbr) sbr_sr = dsi.sbr_sr;
-				ch = dsi.nb_chan;
-				PL = dsi.audioPL;
-				sbr = dsi.has_sbr ? ((dsi.base_object_type==GF_M4A_AAC_SBR || dsi.base_object_type==GF_M4A_AAC_PS) ? 2 : 1) : GF_FALSE;
-				ps = dsi.has_ps;
-			} else {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("Missing DecoderSpecificInfo in MPEG-4 AAC stream\n"));
+				if (origin_esd->decoderConfig->decoderSpecificInfo) {
+					GF_M4ADecSpecInfo dsi;
+					gf_m4a_get_config(origin_esd->decoderConfig->decoderSpecificInfo->data, origin_esd->decoderConfig->decoderSpecificInfo->dataLength, &dsi);
+					sr = dsi.base_sr;
+					if (dsi.has_sbr) sbr_sr = dsi.sbr_sr;
+					ch = dsi.nb_chan;
+					PL = dsi.audioPL;
+					sbr = dsi.has_sbr ? ((dsi.base_object_type==GF_M4A_AAC_SBR || dsi.base_object_type==GF_M4A_AAC_PS) ? 2 : 1) : GF_FALSE;
+					ps = dsi.has_ps;
+				} else {
+					GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("Missing DecoderSpecificInfo in MPEG-4 AAC stream\n"));
+				}
+				break;
+#endif
+			case GF_CODECID_MPEG_AUDIO:
+				break;
+			default:
+				skip_profile=GF_TRUE;
+				break;
 			}
 		}
-#endif
-		gf_isom_set_pl_indication(import->dest, GF_ISOM_PL_AUDIO, PL);
+		if (!skip_profile)
+			gf_isom_set_pl_indication(import->dest, GF_ISOM_PL_AUDIO, PL);
 	}
 	else if (mtype==GF_ISOM_MEDIA_SUBPIC) {
 		w = h = 0;
@@ -500,12 +532,12 @@ static GF_Err gf_import_isomedia_track(GF_MediaImporter *import)
 	case GF_ISOM_MEDIA_VISUAL:
 		gf_import_message(import, GF_OK, "IsoMedia import %s - track ID %d - Video (size %d x %d)", orig_name, trackID, w, h);
 		break;
-    case GF_ISOM_MEDIA_AUXV:
-        gf_import_message(import, GF_OK, "IsoMedia import %s - track ID %d - Auxiliary Video (size %d x %d)", orig_name, trackID, w, h);
-        break;
-    case GF_ISOM_MEDIA_PICT:
-        gf_import_message(import, GF_OK, "IsoMedia import %s - track ID %d - Picture sequence (size %d x %d)", orig_name, trackID, w, h);
-        break;
+	case GF_ISOM_MEDIA_AUXV:
+		gf_import_message(import, GF_OK, "IsoMedia import %s - track ID %d - Auxiliary Video (size %d x %d)", orig_name, trackID, w, h);
+		break;
+	case GF_ISOM_MEDIA_PICT:
+		gf_import_message(import, GF_OK, "IsoMedia import %s - track ID %d - Picture sequence (size %d x %d)", orig_name, trackID, w, h);
+		break;
 	case GF_ISOM_MEDIA_AUDIO:
 	{
 		if (ps) {
@@ -575,7 +607,7 @@ static GF_Err gf_import_isomedia_track(GF_MediaImporter *import)
 				break;
 			case GF_ISOM_SUBTYPE_VVC1:
 				gf_isom_set_nalu_extract_mode(import->orig, track_in, GF_ISOM_NALU_EXTRACT_INSPECT | GF_ISOM_NALU_EXTRACT_INBAND_PS_FLAG);
-				//gf_isom_vvc_set_inband_config(import->dest, track, 1, (import->xps_inband==2) ? GF_TRUE : GF_FALSE);
+				gf_isom_vvc_set_inband_config(import->dest, track, 1, (import->xps_inband==2) ? GF_TRUE : GF_FALSE);
 				break;
 			}
 		}
@@ -599,6 +631,7 @@ static GF_Err gf_import_isomedia_track(GF_MediaImporter *import)
 	}
 	num_samples = gf_isom_get_sample_count(import->orig, track_in);
 
+
 	if (is_cenc) {
 		u32 container_type;
 		e = gf_isom_cenc_get_sample_aux_info(import->orig, track_in, 0, 1, &container_type, NULL, NULL);
@@ -626,11 +659,21 @@ static GF_Err gf_import_isomedia_track(GF_MediaImporter *import)
 				gf_isom_sample_del(&samp);
 				break;
 			}
+
+			if ((import->flags & GF_IMPORT_FORCE_SYNC) && ((samp->IsRAP==SAP_TYPE_3) || (samp->IsRAP==SAP_TYPE_4))) {
+				samp->IsRAP = SAP_TYPE_1;
+			}
+
 			e = gf_isom_add_sample_reference(import->dest, track, di, samp, offset);
 		} else {
 			samp = gf_isom_get_sample(import->orig, track_in, i+1, &di);
 			if (!samp) {
 				/*couldn't get the sample, but still move on*/
+				e = gf_isom_last_error(import->orig);
+				if (e==GF_ISOM_INCOMPLETE_FILE) {
+					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[ISOM import] Incomplete file detected, aborting track #%d import after %d / %d samples\n", track_in, i+1, num_samples));
+					e = GF_OK;
+				}
 				goto exit;
 			}
 			samp->DTS -= dts_offset;
@@ -661,11 +704,28 @@ static GF_Err gf_import_isomedia_track(GF_MediaImporter *import)
 				break;
 			}
 
+			if ((import->flags & GF_IMPORT_FORCE_SYNC) && ((samp->IsRAP==SAP_TYPE_3) || (samp->IsRAP==SAP_TYPE_4))) {
+				samp->IsRAP = SAP_TYPE_1;
+			}
+
 			e = gf_isom_add_sample(import->dest, track, di, samp);
 		}
 		sampDTS = samp->DTS;
 		if (samp->nb_pack)
 			i+= samp->nb_pack-1;
+
+		//when importing files indicating SAP3 as sync points, we will drop all SAP3 points - check if we need to add 'rap '
+		//sample to group info, ie if it was not present in source track - cf #2992
+		if (!(import->flags & GF_IMPORT_FORCE_SYNC) && (samp->IsRAP == SAP_TYPE_3)) {
+			if (!inject_rap_sample_group) {
+				u32 sgdi=0;
+				gf_isom_get_sample_to_group_info(import->orig, track_in, i+1, GF_ISOM_SAMPLE_GROUP_RAP, 0, &sgdi);
+				inject_rap_sample_group = sgdi ? 1 : 2;
+			}
+			if (inject_rap_sample_group == 2) {
+				e = gf_isom_set_sample_rap_group(import->dest, track, i+1, GF_TRUE, 0);
+			}
+		}
 		gf_isom_sample_del(&samp);
 
 		//this will also copy all sample to group mapping, including seig for CENC
@@ -835,6 +895,7 @@ GF_Err gf_media_import_chapters_file(GF_MediaImporter *import)
 	Bool found_chap = GF_FALSE;
 	u32 i, h, m, s, ms, fr, fps;
 	char line[1024];
+	char szTemp[1025];
 	char szTitle[1024];
 	FILE *f = gf_fopen(import->in_name, "rt");
 	if (!f) return GF_URL_ERROR;
@@ -904,7 +965,7 @@ GF_Err gf_media_import_chapters_file(GF_MediaImporter *import)
 		for (i=0; i<gf_isom_get_track_count(import->dest); i++) {
 			GF_ISOSample *samp;
 			u32 timescale, inc;
-            u32 mtype = gf_isom_get_media_type(import->dest, i+1);
+			u32 mtype = gf_isom_get_media_type(import->dest, i+1);
 			if (!gf_isom_is_video_handler_type(mtype)) continue;
 			if (gf_isom_get_sample_count(import->dest, i+1) < 20) continue;
 			samp = gf_isom_get_sample_info(import->dest, 1, 2, NULL, NULL);
@@ -953,7 +1014,7 @@ GF_Err gf_media_import_chapters_file(GF_MediaImporter *import)
 		if (!strnicmp(sL, "AddChapter(", 11)) {
 			u32 nb_fr;
 			sscanf(sL, "AddChapter(%u,%1023s)", &nb_fr, szTitle);
-			ts = gf_timestamp_rescale(nb_fr, 1000 * import->video_fps.den, import->video_fps.num);
+			ts = gf_timestamp_rescale(nb_fr, import->video_fps.num, 1000 * import->video_fps.den);
 			sL = strchr(sL, ',');
 			strcpy(szTitle, sL+1);
 			sL = strrchr(szTitle, ')');
@@ -986,7 +1047,7 @@ GF_Err gf_media_import_chapters_file(GF_MediaImporter *import)
 				ts = (h*3600 + m*60+s)*1000;
 			}
 			else {
-				char szTS[1025], *tok;
+				char *tok, *szTS = szTemp;
 				strncpy(szTS, sL, 1024);
 				szTS[1024]=0;
 				tok = strrchr(szTS, ' ');
@@ -1024,7 +1085,7 @@ GF_Err gf_media_import_chapters_file(GF_MediaImporter *import)
 		/*CHAPTERX= and CHAPTERXNAME=*/
 		else if (!strnicmp(sL, "CHAPTER", 7)) {
 			u32 idx;
-			char szTemp[1025], *str;
+			char *str;
 			strncpy(szTemp, sL, 1024);
 			szTemp[1024] = 0;
 			str = strrchr(szTemp, '=');
@@ -1094,7 +1155,7 @@ GF_Err gf_media_import_chapters(GF_ISOFile *file, char *chap_file, GF_Fraction i
 {
 	GF_Err e;
 	u32 i;
-	GF_MediaImporter import;
+	GF_MediaImporter *import = NULL;
 	//remove all chapter info
 	gf_isom_remove_chapter(file, 0, 0);
 
@@ -1111,17 +1172,18 @@ restart_check:
 		}
 	}
 
-	memset(&import, 0, sizeof(GF_MediaImporter));
-	import.dest = file;
-	import.in_name = chap_file;
-	import.video_fps = import_fps;
-	import.streamFormat = "CHAP";
-	e = gf_media_import(&import);
-	if (e) return e;
-
-	if (!import.final_trackID) return GF_OK;
+	GF_SAFEALLOC(import, GF_MediaImporter);
+	import->dest = file;
+	import->in_name = chap_file;
+	import->video_fps = import_fps;
+	import->streamFormat = "CHAP";
+	e = gf_media_import(import);
+	if (e || !import->final_trackID) {
+		gf_free(import);
+		return e;
+	}
 	if (use_qt) {
-		u32 chap_track = gf_isom_get_track_by_id(file, import.final_trackID);
+		u32 chap_track = gf_isom_get_track_by_id(file, import->final_trackID);
 		u32 nb_sdesc = gf_isom_get_sample_description_count(file, chap_track);
 		for (i=0; i<nb_sdesc; i++) {
 			gf_isom_set_media_subtype(file, chap_track, i+1, GF_ISOM_SUBTYPE_TEXT);
@@ -1135,10 +1197,11 @@ restart_check:
 		case GF_ISOM_MEDIA_AUXV:
 		case GF_ISOM_MEDIA_PICT:
 		case GF_ISOM_MEDIA_AUDIO:
-			gf_isom_set_track_reference(file, i+1, GF_ISOM_REF_CHAP, import.final_trackID);
+			gf_isom_set_track_reference(file, i+1, GF_ISOM_REF_CHAP, import->final_trackID);
 			break;
 		}
 	}
+	gf_free(import);
 	return GF_OK;
 }
 
@@ -1255,7 +1318,8 @@ GF_Err gf_media_import(GF_MediaImporter *importer)
 			p = gf_filter_pid_get_property(pid, GF_PROP_PID_CODECID);
 			tki->codecid = p ? p->value.uint : GF_CODECID_NONE;
 			p = gf_filter_pid_get_property(pid, GF_PROP_PID_LANGUAGE);
-			if (p && p->value.string) tki->lang = GF_4CC(p->value.string[0], p->value.string[1], p->value.string[2], ' ');
+			if (p && p->value.string && (strlen(p->value.string)>=3))
+				tki->lang = GF_4CC(p->value.string[0], p->value.string[1], p->value.string[2], ' ');
 			p = gf_filter_pid_get_property(pid, GF_PROP_PID_ID);
 			tki->track_num = p ? p->value.uint : 1;
 			p = gf_filter_pid_get_property(pid, GF_PROP_PID_ESID);
@@ -1283,7 +1347,7 @@ GF_Err gf_media_import(GF_MediaImporter *importer)
 					if (p->value.frac.den) tki->video_info.FPS /= p->value.frac.den;
 				}
 				p = gf_filter_pid_get_property(pid, GF_PROP_PID_SAR);
-				if (p) tki->video_info.par = (p->value.frac.num << 16) | p->value.frac.den;
+				if (p && (p->value.frac.num>0)) tki->video_info.par = (p->value.frac.num << 16) | p->value.frac.den;
 			}
 			p = gf_filter_pid_get_property(pid, GF_PROP_PID_SAMPLE_RATE);
 			if (p) {
@@ -1372,6 +1436,7 @@ GF_Err gf_media_import(GF_MediaImporter *importer)
 		e |= gf_dynstrcat(&args, szSubArg, ":");
 	}
 	if (importer->asemode==GF_IMPORT_AUDIO_SAMPLE_ENTRY_v0_2) { e |= gf_dynstrcat(&args, "ase=v0s", ":"); }
+	else if (importer->asemode==GF_IMPORT_AUDIO_SAMPLE_ENTRY_v0_BS) { e |= gf_dynstrcat(&args, "ase=v0bs", ":"); }
 	else if (importer->asemode==GF_IMPORT_AUDIO_SAMPLE_ENTRY_v1_MPEG) { e |= gf_dynstrcat(&args, "ase=v1", ":"); }
 	else if (importer->asemode==GF_IMPORT_AUDIO_SAMPLE_ENTRY_v1_QTFF) { e |= gf_dynstrcat(&args, "ase=v1qt", ":"); }
 
