@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / MPEG-4 ObjectDescriptor sub-project
@@ -10,21 +11,23 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 
 #include <gpac/internal/odf_dev.h>
 
+#ifndef GPAC_MINIMAL_ODF
+
 
 //from OD
 GF_Err gf_odf_write_descriptor_list(GF_BitStream *bs, GF_List *descList);
@@ -47,7 +50,7 @@ struct __tag_oci_event
 	GF_List *OCIDescriptors;
 };
 
-struct __tag_oci_codec 
+struct __tag_oci_codec
 {
 	//events
 	GF_List *OCIEvents;
@@ -62,7 +65,7 @@ OCIEvent *gf_oci_event_new(u16 EventID)
 {
 	OCIEvent *tmp;
 	if (EventID > MAX_OCIEVENT_ID) return NULL;
-	tmp = (OCIEvent *)malloc(sizeof(OCIEvent));
+	tmp = (OCIEvent *)gf_malloc(sizeof(OCIEvent));
 	if (!tmp) return NULL;
 	memset(tmp, 0, sizeof(OCIEvent));
 	tmp->EventID = EventID;
@@ -73,16 +76,15 @@ OCIEvent *gf_oci_event_new(u16 EventID)
 GF_EXPORT
 void gf_oci_event_del(OCIEvent *event)
 {
-	GF_Descriptor *desc;
 	if (!event) return;
 
 	while (gf_list_count(event->OCIDescriptors)) {
-		desc = (GF_Descriptor *)gf_list_get(event->OCIDescriptors, 0);
+		GF_Descriptor *desc = (GF_Descriptor *)gf_list_get(event->OCIDescriptors, 0);
 		gf_list_rem(event->OCIDescriptors, 0);
 		gf_odf_delete_descriptor(desc);
 	}
 	gf_list_del(event->OCIDescriptors);
-	free(event);	
+	gf_free(event);
 }
 
 GF_EXPORT
@@ -124,7 +126,7 @@ GF_Err gf_oci_event_add_desc(OCIEvent *event, GF_Descriptor *oci_desc)
 {
 	if (!event || !oci_desc) return GF_BAD_PARAM;
 	if (!OCI_IsOCIDesc(oci_desc)) return GF_ODF_INVALID_DESCRIPTOR;
-	
+
 	gf_list_add(event->OCIDescriptors, oci_desc);
 	return GF_OK;
 }
@@ -140,7 +142,7 @@ GF_Err gf_oci_event_get_id(OCIEvent *event, u16 *ID)
 GF_EXPORT
 GF_Err gf_oci_event_get_start_time(OCIEvent *event, u8 *Hours, u8 *Minutes, u8 *Seconds, u8 *HundredSeconds, u8 *IsAbsoluteTime)
 {
-	if (!event || !Hours || !Minutes || !Seconds || !HundredSeconds || !IsAbsoluteTime) 
+	if (!event || !Hours || !Minutes || !Seconds || !HundredSeconds || !IsAbsoluteTime)
 		return GF_BAD_PARAM;
 
 	*IsAbsoluteTime = event->AbsoluteTimeFlag;
@@ -154,7 +156,7 @@ GF_Err gf_oci_event_get_start_time(OCIEvent *event, u8 *Hours, u8 *Minutes, u8 *
 GF_EXPORT
 GF_Err gf_oci_event_get_duration(OCIEvent *event, u8 *Hours, u8 *Minutes, u8 *Seconds, u8 *HundredSeconds)
 {
-	if (!event || !Hours || !Minutes || !Seconds || !HundredSeconds) 
+	if (!event || !Hours || !Minutes || !Seconds || !HundredSeconds)
 		return GF_BAD_PARAM;
 
 	*Hours = event->duration[0];
@@ -192,7 +194,7 @@ OCICodec *gf_oci_codec_new(u8 IsEncoder, u8 Version)
 {
 	OCICodec *tmp;
 	if (Version != 0x01) return NULL;
-	tmp = (OCICodec *)malloc(sizeof(OCICodec));
+	tmp = (OCICodec *)gf_malloc(sizeof(OCICodec));
 	if (!tmp) return NULL;
 	tmp->Mode = IsEncoder ? 1 : 0;
 	tmp->Version = 0x01;
@@ -203,16 +205,15 @@ OCICodec *gf_oci_codec_new(u8 IsEncoder, u8 Version)
 GF_EXPORT
 void gf_oci_codec_del(OCICodec *codec)
 {
-	OCIEvent *ev;
 	if (!codec) return;
 
 	while (gf_list_count(codec->OCIEvents)) {
-		ev = (OCIEvent *)gf_list_get(codec->OCIEvents, 0);
+		OCIEvent *ev = (OCIEvent *)gf_list_get(codec->OCIEvents, 0);
 		gf_oci_event_del(ev);
 		gf_list_rem(codec->OCIEvents, 0);
 	}
 	gf_list_del(codec->OCIEvents);
-	free(codec);
+	gf_free(codec);
 }
 
 GF_EXPORT
@@ -229,16 +230,16 @@ GF_Err WriteSevenBitLength(GF_BitStream *bs, u32 size)
 	unsigned char vals[4];
 
 	if (!bs || !size) return GF_BAD_PARAM;
-	
+
 	length = size;
 	vals[3] = (unsigned char) (length & 0x7f);
 	length >>= 7;
-	vals[2] = (unsigned char) ((length & 0x7f) | 0x80); 
+	vals[2] = (unsigned char) ((length & 0x7f) | 0x80);
 	length >>= 7;
-	vals[1] = (unsigned char) ((length & 0x7f) | 0x80); 
+	vals[1] = (unsigned char) ((length & 0x7f) | 0x80);
 	length >>= 7;
 	vals[0] = (unsigned char) ((length & 0x7f) | 0x80);
-	
+
 	if (size < 0x00000080) {
 		gf_bs_write_int(bs, vals[3], 8);
 	} else if (size < 0x00004000) {
@@ -260,13 +261,13 @@ GF_Err WriteSevenBitLength(GF_BitStream *bs, u32 size)
 }
 
 GF_EXPORT
-GF_Err gf_oci_codec_encode(OCICodec *codec, char **outAU, u32 *au_length)
+GF_Err gf_oci_codec_encode(OCICodec *codec, u8 **outAU, u32 *au_length)
 {
 	GF_BitStream *bs;
 	u32 i, size, desc_size;
 	GF_Err e;
 	OCIEvent *ev;
-	
+
 	if (!codec || !codec->Mode || *outAU) return GF_BAD_PARAM;
 
 	bs = NULL;
@@ -297,7 +298,7 @@ GF_Err gf_oci_codec_encode(OCICodec *codec, char **outAU, u32 *au_length)
 		gf_bs_write_int(bs, ev->AbsoluteTimeFlag, 1);
 		gf_bs_write_data(bs, ev->StartingTime, 4);
 		gf_bs_write_data(bs, ev->duration, 4);
-		
+
 		e = gf_odf_write_descriptor_list(bs, ev->OCIDescriptors);
 		gf_oci_event_del(ev);
 		if (e) goto err_exit;
@@ -322,12 +323,12 @@ err_exit:
 
 
 GF_EXPORT
-GF_Err gf_oci_codec_decode(OCICodec *codec, char *au, u32 au_length)
+GF_Err gf_oci_codec_decode(OCICodec *codec, u8 *au, u32 au_length)
 {
 	OCIEvent *ev;
 	GF_BitStream *bs;
 	u32 size, hdrS, desc_size, tot_size, tmp_size, val;
-	GF_Descriptor *tmp;	
+	GF_Descriptor *tmp;
 	GF_Err e;
 
 	//must be decoder
@@ -343,7 +344,7 @@ GF_Err gf_oci_codec_decode(OCICodec *codec, char *au, u32 au_length)
 			e = GF_OUT_OF_MEM;
 			goto err_exit;
 		}
-		
+
 
 		//FIX IM1
 		gf_bs_read_int(bs, 8);
@@ -356,7 +357,7 @@ GF_Err gf_oci_codec_decode(OCICodec *codec, char *au, u32 au_length)
 			size <<= 7;
 			size |= val & 0x7F;
 		} while ( val & 0x80 );
-		
+
 		//parse event vars
 		ev->EventID = gf_bs_read_int(bs, 15);
 		ev->AbsoluteTimeFlag = gf_bs_read_int(bs, 1);
@@ -377,7 +378,7 @@ GF_Err gf_oci_codec_decode(OCICodec *codec, char *au, u32 au_length)
 			gf_list_add(ev->OCIDescriptors, tmp);
 			desc_size += tmp_size + gf_odf_size_field_size(tmp_size);
 		}
-		
+
 		if (desc_size != size - 10) {
 			e = GF_CORRUPTED_DATA;
 			goto err_exit;
@@ -420,3 +421,5 @@ OCIEvent *gf_oci_codec_get_event(OCICodec *codec)
 	gf_list_rem(codec->OCIEvents, 0);
 	return ev;
 }
+
+#endif /*GPAC_MINIMAL_ODF*/
