@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2023
+ *			Copyright (c) Telecom ParisTech 2000-2026
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Management sub-project
@@ -47,7 +47,7 @@ enum
 		if (strchr(__sep, __str[_len])) __str[_len] = 0;	\
 		else break;	\
 	}	\
- 
+
 
 static GF_Err gf_text_guess_format(char *filename, u32 *fmt)
 {
@@ -170,55 +170,61 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 
 		if (sOK) REM_TRAIL_MARKS(szLine, "\r\n\t ")
 
-			if (!sOK || !strlen(szLine)) {
-				state = 0;
-				if (au) {
-					/*if italic or underscore do it*/
-					if (font && (italic || underlined || bold)) {
-						com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
-						com->node = font;
-						gf_node_register(font, NULL);
-						inf = gf_sg_command_field_new(com);
-						inf->fieldIndex = style.fieldIndex;
-						inf->fieldType = style.fieldType;
-						inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
-						sfstr = (SFString *)inf->field_ptr;
-						if (bold && italic && underlined) sfstr->buffer = gf_strdup("BOLDITALIC UNDERLINED");
-						else if (italic && underlined) sfstr->buffer = gf_strdup("ITALIC UNDERLINED");
-						else if (bold && underlined) sfstr->buffer = gf_strdup("BOLD UNDERLINED");
-						else if (underlined) sfstr->buffer = gf_strdup("UNDERLINED");
-						else if (bold && italic) sfstr->buffer = gf_strdup("BOLDITALIC");
-						else if (bold) sfstr->buffer = gf_strdup("BOLD");
-						else sfstr->buffer = gf_strdup("ITALIC");
-						gf_list_add(au->commands, com);
-					}
-
-					au = gf_sm_stream_au_new(srt, end, 0, 1);
+		if (!sOK || !strlen(szLine)) {
+			state = 0;
+			if (au) {
+				/*if italic or underscore do it*/
+				if (font && (italic || underlined || bold)) {
 					com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
-					com->node = text;
-					gf_node_register(text, NULL);
+					com->node = font;
+					gf_node_register(font, NULL);
 					inf = gf_sg_command_field_new(com);
-					inf->fieldIndex = string.fieldIndex;
-					inf->fieldType = string.fieldType;
-					inf->field_ptr = gf_sg_vrml_field_pointer_new(string.fieldType);
+					inf->fieldIndex = style.fieldIndex;
+					inf->fieldType = style.fieldType;
+					inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
+					sfstr = (SFString *)inf->field_ptr;
+					if (bold && italic && underlined) sfstr->buffer = gf_strdup("BOLDITALIC UNDERLINED");
+					else if (italic && underlined) sfstr->buffer = gf_strdup("ITALIC UNDERLINED");
+					else if (bold && underlined) sfstr->buffer = gf_strdup("BOLD UNDERLINED");
+					else if (underlined) sfstr->buffer = gf_strdup("UNDERLINED");
+					else if (bold && italic) sfstr->buffer = gf_strdup("BOLDITALIC");
+					else if (bold) sfstr->buffer = gf_strdup("BOLD");
+					else sfstr->buffer = gf_strdup("ITALIC");
 					gf_list_add(au->commands, com);
-					/*reset font styles so that all AUs are true random access*/
-					if (font) {
-						com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
-						com->node = font;
-						gf_node_register(font, NULL);
-						inf = gf_sg_command_field_new(com);
-						inf->fieldIndex = style.fieldIndex;
-						inf->fieldType = style.fieldType;
-						inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
-						gf_list_add(au->commands, com);
-					}
-					au = NULL;
 				}
-				inf = NULL;
-				if (!sOK) break;
-				continue;
+
+				au = gf_sm_stream_au_new(srt, end, 0, 1);
+				com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
+				com->node = text;
+				gf_node_register(text, NULL);
+				inf = gf_sg_command_field_new(com);
+				inf->fieldIndex = string.fieldIndex;
+				inf->fieldType = string.fieldType;
+				inf->field_ptr = gf_sg_vrml_field_pointer_new(string.fieldType);
+				gf_list_add(au->commands, com);
+				/*reset font styles so that all AUs are true random access*/
+				if (font) {
+					com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
+					com->node = font;
+					gf_node_register(font, NULL);
+					inf = gf_sg_command_field_new(com);
+					inf->fieldIndex = style.fieldIndex;
+					inf->fieldType = style.fieldType;
+					inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
+					gf_list_add(au->commands, com);
+				}
+				au = NULL;
 			}
+			inf = NULL;
+			if (!sOK) break;
+			continue;
+		}
+
+		if (!gf_utf8_is_legal(sOK, (u32)strlen(szLine))) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[srt->bifs] Illegal UTF8 data\n"));
+			e = GF_CORRUPTED_DATA;
+			goto exit;
+		}
 
 		switch (state) {
 		case 0:
@@ -309,6 +315,7 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 			len = 0;
 			for (i=0; i<strlen(ptr); i++) {
 				/*FIXME - UTF16 support !!*/
+				if (len >= GF_ARRAY_LENGTH(szText)) break;
 				if (ptr[i] & 0x80) {
 					/*non UTF8 (likely some win-CP)*/
 					if ((ptr[i+1] & 0xc0) != 0x80) {
@@ -323,10 +330,11 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 						i++;
 					}
 				}
+				if (len >= GF_ARRAY_LENGTH(szText)) break;
 				szText[len] = ptr[i];
 				len++;
 			}
-			szText[len] = 0;
+			szText[MIN(len, GF_ARRAY_LENGTH(szText)-1)] = 0;
 			sfstr->buffer = gf_strdup(szText);
 			break;
 		}
@@ -437,10 +445,16 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 			break;
 		}
 		while (szLine[i+1] && szLine[i+1]!='}') {
+			if (i>=GF_ARRAY_LENGTH(szTime)) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[sub->bifs] Bad frame (line %d): expected \"}\" before %d chars after \"{\"\n", line, GF_ARRAY_LENGTH(szTime)));
+				e = GF_NON_COMPLIANT_BITSTREAM;
+				szTime[0] = 0;
+				goto exit;
+			}
 			szTime[i] = szLine[i+1];
 			i++;
 		}
-		szTime[i] = 0;
+		szTime[MIN(i, GF_ARRAY_LENGTH(szTime)-1)] = 0;
 		start = atoi(szTime);
 		if (start<end) {
 			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[sub->bifs] corrupted SUB frame (line %d) - starts (at %d ms) before end of previous one (%d ms) - adjusting time stamps\n", line, start, end));
@@ -454,10 +468,16 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 			break;
 		}
 		while (szLine[i+1+j] && szLine[i+1+j]!='}') {
+			if (i>=GF_ARRAY_LENGTH(szTime)) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[sub->bifs] Bad frame (line %d): expected \"}\" before %d chars after \"{\"\n", line, GF_ARRAY_LENGTH(szTime)));
+				e = GF_NON_COMPLIANT_BITSTREAM;
+				szTime[0] = 0;
+				goto exit;
+			}
 			szTime[i] = szLine[i+1+j];
 			i++;
 		}
-		szTime[i] = 0;
+		szTime[MIN(i, GF_ARRAY_LENGTH(szTime)-1)] = 0;
 		end = atoi(szTime);
 		j+=i+2;
 
@@ -517,6 +537,7 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 			sfstr->buffer = gf_strdup(szText);
 		}
 	}
+exit:
 	gf_fclose(sub_in);
 	return e;
 }
@@ -539,4 +560,3 @@ GF_Err gf_sm_import_bifs_subtitle(GF_SceneManager *ctx, GF_ESD *src, GF_MuxInfo
 	return GF_NOT_SUPPORTED;
 #endif
 }
-
