@@ -91,7 +91,7 @@ GF_SceneGraph *gf_sg_new()
 		gf_node_parent_of(NULL, NULL);
 		gf_sg_get_parent(tmp);
 		gf_node_get_attribute_count(NULL);
-		gf_xml_node_clone(NULL, NULL, NULL, NULL, 0);
+		gf_sg_xml_node_clone(NULL, NULL, NULL, NULL, 0);
 		gf_dom_flatten_textContent(NULL);
 		gf_smil_timing_pause(NULL);
 		gf_smil_timing_resume(NULL);
@@ -629,7 +629,7 @@ void remove_node_id(GF_SceneGraph *sg, GF_Node *node)
 {
 	NodeIDedItem *reg_node = sg->id_node;
 	if (!reg_node) return;
-	
+
 	if (reg_node->node==node) {
 		sg->id_node = reg_node->next;
 		if (sg->id_node_last==reg_node)
@@ -1783,6 +1783,8 @@ void gf_node_init(GF_Node *node)
 	gf_assert(pSG);
 	/*no user-defined init, consider the scenegraph is only used for parsing/encoding/decoding*/
 	if (!pSG->NodeCallback) return;
+	//some broken content may trigger twice an update
+	if (node->sgprivate->UserPrivate) return;
 
 	/*internal nodes*/
 #ifndef GPAC_DISABLE_VRML
@@ -1944,7 +1946,7 @@ const char *gf_node_get_class_name(GF_Node *node)
 		if (ns == full->ns) return full->name;
 		xmlns = (char *) gf_sg_get_namespace_qname(node->sgprivate->scenegraph, full->ns);
 		if (!xmlns) return full->name;
-		sprintf(node->sgprivate->scenegraph->szNameBuffer, "%s:%s", xmlns, full->name);
+		snprintf(node->sgprivate->scenegraph->szNameBuffer, 99, "%s:%s", xmlns, full->name);
 		return node->sgprivate->scenegraph->szNameBuffer;
 	}
 #ifndef GPAC_DISABLE_SVG
@@ -2083,7 +2085,7 @@ GF_Err gf_node_get_field_by_name(GF_Node *node, char *name, GF_FieldInfo *field)
 {
 	s32 res = -1;
 
-	if (node->sgprivate->tag==TAG_UndefinedNode) return GF_BAD_PARAM;
+	if (!node || !node->sgprivate || node->sgprivate->tag==TAG_UndefinedNode) return GF_BAD_PARAM;
 #ifndef GPAC_DISABLE_VRML
 	else if (node->sgprivate->tag == TAG_ProtoNode) {
 		res = gf_sg_proto_get_field_index_by_name(NULL, node, name);
@@ -2220,7 +2222,7 @@ GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_pa
 		return NULL;
 	} else {
 #ifndef GPAC_DISABLE_SVG
-		return gf_xml_node_clone(inScene, orig, cloned_parent, id, deep);
+		return gf_sg_xml_node_clone(inScene, orig, cloned_parent, id, deep);
 #endif
 	}
 	return NULL;
@@ -2251,7 +2253,7 @@ GF_Err gf_sg_add_namespace(GF_SceneGraph *sg, char *name, char *qname)
 
 	GF_SAFEALLOC(ns, GF_XMLNS);
 	if (!ns) return GF_OUT_OF_MEM;
-	
+
 	ns->xmlns_id = id ? id : gf_crc_32(name, (u32) strlen(name));
 	ns->name = gf_strdup(name);
 
