@@ -1,8 +1,9 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Authors: Cyril Concolato - Jean Le Feuvre
- *    Copyright (c)2008-200X ENST - All rights reserved
+ *			Authors: Jean Le Feuvre - Cyril Concolato
+ *			Copyright (c) Telecom ParisTech 2000-2022
+ *					All rights reserved
  *
  *  This file is part of GPAC / SVG Scene Graph sub-project
  *
@@ -14,7 +15,7 @@
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY, TYPE, 0 }, without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Lesser General Public License for more details.	
+ *  GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library, TYPE, 0 }, see the file COPYING.  If not, write to
@@ -41,9 +42,44 @@ enum
 	GF_SVG_ATTOPT_CURSOR = 3,
 	/*attribute only valid for listener*/
 	GF_SVG_ATTOPT_LISTENER = 4,
+	/*attribute only valid for filters*/
+	GF_SVG_ATTOPT_FILTER = 5,
 } GF_SVGAttOption;
 
-static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts; u32 xmlns; } xml_attributes [] = 
+/* unused XBL tags (deprecated)
+
+	TAG_XBL_ATT_RANGE_FIRST = TAG_SVG_ATT_RANGE_FIRST + 256,
+	TAG_XBL_ATT_id = TAG_XBL_ATT_RANGE_FIRST,
+	TAG_XBL_ATT_extends,
+	TAG_XBL_ATT_display,
+	TAG_XBL_ATT_inheritstyle,
+	TAG_XBL_ATT_includes,
+	TAG_XBL_ATT_name,
+	TAG_XBL_ATT_implements,
+	TAG_XBL_ATT_type,
+	TAG_XBL_ATT_readonly,
+	TAG_XBL_ATT_onget,
+	TAG_XBL_ATT_onset,
+	TAG_XBL_ATT_event,
+	TAG_XBL_ATT_action,
+	TAG_XBL_ATT_phase,
+	TAG_XBL_ATT_button,
+	TAG_XBL_ATT_modifiers,
+	TAG_XBL_ATT_keycode,
+	TAG_XBL_ATT_key,
+	TAG_XBL_ATT_charcode,
+	TAG_XBL_ATT_clickcount,
+	TAG_XBL_ATT_command,
+	TAG_XBL_ATT_preventdefault,
+	TAG_XBL_ATT_src,
+*/
+static const struct xml_att_def {
+	const char *name;
+	u32 tag;
+	u32 type;
+	u32 opts;
+	u32 xmlns;
+} xml_attributes [] =
 {
 	/*XML base*/
 	{ "id", TAG_XML_ATT_id, SVG_ID_datatype, 0, GF_XMLNS_XML } ,
@@ -69,7 +105,7 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "target", TAG_XMLEV_ATT_target, XML_IDREF_datatype, 0, GF_XMLNS_XMLEV },
 	{ "handler", TAG_XMLEV_ATT_handler, XMLRI_datatype, 0, GF_XMLNS_XMLEV },
 
-	
+
 	{ "id", TAG_SVG_ATT_id, SVG_ID_datatype, 0, GF_XMLNS_SVG } ,
 	{ "class", TAG_SVG_ATT__class, DOM_String_datatype, 0, GF_XMLNS_SVG },
 	{ "requiredFeatures", TAG_SVG_ATT_requiredFeatures, XMLRI_List_datatype, 0, GF_XMLNS_SVG },
@@ -86,7 +122,7 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "audio-level", TAG_SVG_ATT_audio_level, SVG_Number_datatype, 0, GF_XMLNS_SVG },
 	{ "viewport-fill", TAG_SVG_ATT_viewport_fill, SVG_Paint_datatype, 0, GF_XMLNS_SVG },
 	{ "viewport-fill-opacity", TAG_SVG_ATT_viewport_fill_opacity, SVG_Number_datatype, 0, GF_XMLNS_SVG },
-	{ "overflow", TAG_SVG_ATT_overflow, DOM_String_datatype, 0, GF_XMLNS_SVG },
+	{ "overflow", TAG_SVG_ATT_overflow, SVG_Overflow_datatype, 0, GF_XMLNS_SVG },
 	{ "fill-opacity", TAG_SVG_ATT_fill_opacity, SVG_Number_datatype, 0, GF_XMLNS_SVG },
 	{ "stroke-opacity", TAG_SVG_ATT_stroke_opacity, SVG_Number_datatype, 0, GF_XMLNS_SVG },
 	{ "fill-rule", TAG_SVG_ATT_fill_rule, SVG_FillRule_datatype, 0, GF_XMLNS_SVG },
@@ -127,7 +163,7 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "nav-down-left", TAG_SVG_ATT_nav_down_left, SVG_Focus_datatype, 0, GF_XMLNS_SVG },
 	{ "nav-left", TAG_SVG_ATT_nav_left, SVG_Focus_datatype, 0, GF_XMLNS_SVG },
 	{ "nav-up-left", TAG_SVG_ATT_nav_up_left, SVG_Focus_datatype, 0, GF_XMLNS_SVG },
-	{ "transform", TAG_SVG_ATT_transform, SVG_Transform_datatype, 0, GF_XMLNS_SVG },	
+	{ "transform", TAG_SVG_ATT_transform, SVG_Transform_datatype, 0, GF_XMLNS_SVG },
 	{ "target", TAG_SVG_ATT_target, DOM_String_datatype, 0, GF_XMLNS_SVG },
 	{ "attributeName", TAG_SVG_ATT_attributeName, SMIL_AttributeName_datatype, 0, GF_XMLNS_SVG },
 	{ "attributeType", TAG_SVG_ATT_attributeType, SMIL_AttributeType_datatype, 0, GF_XMLNS_SVG },
@@ -208,6 +244,7 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "x2", TAG_SVG_ATT_x2, SVG_Coordinate_datatype, 0, GF_XMLNS_SVG },
 	{ "y2", TAG_SVG_ATT_y2, SVG_Coordinate_datatype, 0, GF_XMLNS_SVG },
 	{ "gradientUnits", TAG_SVG_ATT_gradientUnits, SVG_GradientUnit_datatype, 0, GF_XMLNS_SVG },
+	{ "filterUnits", TAG_SVG_ATT_filterUnits, SVG_GradientUnit_datatype, 0, GF_XMLNS_SVG },
 	{ "spreadMethod", TAG_SVG_ATT_spreadMethod, SVG_SpreadMethod_datatype, 0, GF_XMLNS_SVG },
 	{ "gradientTransform", TAG_SVG_ATT_gradientTransform, SVG_Transform_datatype, 0, GF_XMLNS_SVG },
 	{ "pathLength", TAG_SVG_ATT_pathLength, SVG_Number_datatype, 0, GF_XMLNS_SVG },
@@ -220,8 +257,8 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "fx", TAG_SVG_ATT_fx, SVG_Coordinate_datatype, 0, GF_XMLNS_SVG },
 	{ "fy", TAG_SVG_ATT_fy, SVG_Coordinate_datatype, 0, GF_XMLNS_SVG },
 	{ "size", TAG_SVG_ATT_size, LASeR_Size_datatype, 0, GF_XMLNS_SVG },
-	{ "choice", TAG_SVG_ATT_choice, LASeR_Choice_datatype, 0, GF_XMLNS_SVG },
-	{ "delta", TAG_SVG_ATT_delta, LASeR_Size_datatype, 0, GF_XMLNS_SVG },
+	{ "choice", TAG_SVG_ATT_choice, LASeR_Choice_datatype, 0, GF_XMLNS_LASER },
+	{ "delta", TAG_SVG_ATT_delta, LASeR_Size_datatype, 0, GF_XMLNS_LASER },
 	{ "offset", TAG_SVG_ATT_offset, SVG_Number_datatype, 0, GF_XMLNS_SVG },
 	{ "syncBehaviorDefault", TAG_SVG_ATT_syncBehaviorDefault, SMIL_SyncBehavior_datatype, 0, GF_XMLNS_SVG },
 	{ "syncToleranceDefault", TAG_SVG_ATT_syncToleranceDefault, SMIL_SyncTolerance_datatype, 0, GF_XMLNS_SVG },
@@ -237,16 +274,23 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "overlay", TAG_SVG_ATT_overlay, SVG_Overlay_datatype, 0, GF_XMLNS_SVG },
 	{ "fullscreen", TAG_SVG_ATT_fullscreen, SVG_Boolean_datatype, 0, GF_XMLNS_SVG },
 	{ "motionTransform", TAG_SVG_ATT_motionTransform, SVG_Motion_datatype, 0, GF_XMLNS_SVG },
+	{ "clip-path", TAG_SVG_ATT_clip_path, SVG_ClipPath_datatype, 0, GF_XMLNS_SVG },
+
+
 	/*SMIL anim fill*/
 	{ "fill", TAG_SVG_ATT_smil_fill, SMIL_Fill_datatype, GF_SVG_ATTOPT_SMIL, GF_XMLNS_SVG },
 	/*regular paint fill*/
 	{ "fill", TAG_SVG_ATT_fill, SVG_Paint_datatype, 0, GF_XMLNS_SVG },
+	/*filter*/
+	{ "filter", TAG_SVG_ATT_filter, SVG_Paint_datatype, 0, GF_XMLNS_SVG },
 	/*text rotate*/
 	{ "rotate", TAG_SVG_ATT_text_rotate, SVG_Numbers_datatype, GF_SVG_ATTOPT_TEXT, GF_XMLNS_SVG },
 	/*regular matrix rotate*/
 	{ "rotate", TAG_SVG_ATT_rotate, SVG_Rotate_datatype, 0, GF_XMLNS_SVG },
 	/*SMIL anim type*/
 	{ "type", TAG_SVG_ATT_transform_type, SVG_TransformType_datatype, GF_SVG_ATTOPT_SMIL, GF_XMLNS_SVG },
+	/*Filter componentTransfer type*/
+	{ "type", TAG_SVG_ATT_filter_transfer_type, SVG_Filter_TransferType_datatype, GF_SVG_ATTOPT_FILTER, GF_XMLNS_SVG },
 	/*regular content type*/
 	{ "type", TAG_SVG_ATT_type, SVG_ContentType_datatype, 0, GF_XMLNS_SVG },
 	/*text x*/
@@ -258,6 +302,12 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	/*regular y position*/
 	{ "y", TAG_SVG_ATT_y, SVG_Coordinate_datatype, 0, GF_XMLNS_SVG },
 
+	/*filters*/
+	{ "tableValues", TAG_SVG_ATT_filter_table_values, SVG_Numbers_datatype, 0, GF_XMLNS_SVG },
+	{ "intercept", TAG_SVG_ATT_filter_intercept, SVG_Number_datatype, 0, GF_XMLNS_SVG },
+	{ "amplitude", TAG_SVG_ATT_filter_amplitude, SVG_Number_datatype, 0, GF_XMLNS_SVG },
+	{ "exponent", TAG_SVG_ATT_filter_exponent, SVG_Number_datatype, 0, GF_XMLNS_SVG },
+
 	/*LASeR*/
 	{ "enabled", TAG_LSR_ATT_enabled, SVG_Boolean_datatype, 0, GF_XMLNS_LASER },
 	/*cursor x*/
@@ -266,6 +316,7 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "y", TAG_SVG_ATT_cursorManager_y, SVG_Length_datatype, GF_SVG_ATTOPT_CURSOR, GF_XMLNS_LASER },
 
 	/*XBL*/
+#if 0
 	{ "id", TAG_XBL_ATT_id, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "extends", TAG_XBL_ATT_extends, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "display", TAG_XBL_ATT_display, DOM_String_datatype, 0, GF_XMLNS_XBL },
@@ -289,6 +340,12 @@ static const struct xml_att_def { const char *name; u32 tag; u32 type; u32 opts;
 	{ "command", TAG_XBL_ATT_command, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "preventdefault", TAG_XBL_ATT_preventdefault, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "src", TAG_XBL_ATT_src, DOM_String_datatype, 0, GF_XMLNS_XBL },
+#endif
+	/*GPAC SVG Extensions*/
+	{ "use-as-primary", TAG_GSVG_ATT_useAsPrimary, SVG_Boolean_datatype, 0, GF_XMLNS_SVG_GPAC_EXTENSION},
+	{ "depthOffset", TAG_GSVG_ATT_depthOffset, SVG_Number_datatype, 0, GF_XMLNS_SVG_GPAC_EXTENSION},
+	{ "depthGain", TAG_GSVG_ATT_depthGain, SVG_Number_datatype, 0, GF_XMLNS_SVG_GPAC_EXTENSION},
+
 };
 
 void gf_xml_push_namespaces(GF_DOMNode *elt)
@@ -322,18 +379,50 @@ void gf_xml_pop_namespaces(GF_DOMNode *elt)
 }
 
 
-u32 gf_xml_get_attribute_tag(GF_Node *elt, char *attribute_name, u32 ns)
+static u32 gf_xml_get_namespace(GF_DOMNode *elt, const char *attribute_name)
+{
+	GF_DOMAttribute *att = elt->attributes;
+	while (att) {
+		if (att->tag==TAG_DOM_ATT_any) {
+			GF_DOMFullAttribute *datt = (GF_DOMFullAttribute*)att;
+			if (datt->name && !strncmp(datt->name, "xmlns:", 6) && !strcmp(datt->name+6, attribute_name)) {
+				return gf_xml_get_namespace_id(*(DOM_String *)  datt->data);
+			}
+		}
+		att = att->next;
+	}
+	if (!elt->sgprivate->parents) return 0;
+	return gf_xml_get_namespace((GF_DOMNode*)elt->sgprivate->parents->node, attribute_name);
+}
+
+static char *gf_xml_get_namespace_qname(GF_DOMNode *elt, u32 ns)
+{
+	GF_DOMAttribute *att = elt->attributes;
+	while (att) {
+		if (att->tag==TAG_DOM_ATT_any) {
+			GF_DOMFullAttribute *datt = (GF_DOMFullAttribute*)att;
+			if (datt->name && !strncmp(datt->name, "xmlns", 5) && (gf_xml_get_namespace_id(*(DOM_String *)  datt->data)==ns)) {
+				if (datt->name[5]) return datt->name+6;
+				return NULL;
+			}
+		}
+		att = att->next;
+	}
+	if (!elt->sgprivate->parents) return NULL;
+	return gf_xml_get_namespace_qname((GF_DOMNode*)elt->sgprivate->parents->node, ns);
+}
+
+u32 gf_xml_get_attribute_tag(GF_Node *elt, char *attribute_name, GF_NamespaceType ns)
 {
-	char *ns_sep;
 	u32 i, count;
 	count = sizeof(xml_attributes) / sizeof(struct xml_att_def);
 
 	if (!ns) {
-		ns_sep = strchr(attribute_name, ':');
-		ns = 0;
+		char *ns_sep = strchr(attribute_name, ':');
 		if (ns_sep) {
 			ns_sep[0] = 0;
 			ns = gf_sg_get_namespace_code(elt->sgprivate->scenegraph, attribute_name);
+			if (ns==GF_XMLNS_UNDEFINED) ns = gf_xml_get_namespace((GF_DOMNode*)elt, attribute_name);
 			ns_sep[0] = ':';
 			attribute_name = ++ns_sep;
 		} else {
@@ -354,9 +443,13 @@ u32 gf_xml_get_attribute_tag(GF_Node *elt, char *attribute_name, u32 ns)
 			case TAG_SVG_animateMotion:
 			case TAG_SVG_animateTransform:
 			case TAG_SVG_animation:
+			case TAG_SVG_set:
+				return xml_attributes[i].tag;
 			case TAG_SVG_audio:
 			case TAG_SVG_video:
-			case TAG_SVG_set:
+				//"type" on audio and video is mime type, not smil anim type
+				if (xml_attributes[i].tag == TAG_SVG_ATT_transform_type)
+					break;
 				return xml_attributes[i].tag;
 			default:
 				break;
@@ -374,10 +467,42 @@ u32 gf_xml_get_attribute_tag(GF_Node *elt, char *attribute_name, u32 ns)
 			if (elt->sgprivate->tag == TAG_SVG_listener)
 				return xml_attributes[i].tag;
 			break;
+		case GF_SVG_ATTOPT_FILTER:
+			switch (elt->sgprivate->tag) {
+			case TAG_SVG_filter:
+			case TAG_SVG_feDistantLight:
+			case TAG_SVG_fePointLight:
+			case TAG_SVG_feSpotLight:
+			case TAG_SVG_feBlend:
+			case TAG_SVG_feColorMatrix:
+			case TAG_SVG_feComponentTransfer:
+			case TAG_SVG_feFuncR:
+			case TAG_SVG_feFuncG:
+			case TAG_SVG_feFuncB:
+			case TAG_SVG_feFuncA:
+			case TAG_SVG_feComposite:
+			case TAG_SVG_feConvolveMatrix:
+			case TAG_SVG_feDiffuseLighting:
+			case TAG_SVG_feDisplacementMap:
+			case TAG_SVG_feFlood:
+			case TAG_SVG_feGaussianBlur:
+			case TAG_SVG_feImage:
+			case TAG_SVG_feMerge:
+			case TAG_SVG_feMorphology:
+			case TAG_SVG_feOffset:
+			case TAG_SVG_feSpecularLighting:
+			case TAG_SVG_feTile:
+			case TAG_SVG_feTurbulence:
+				return xml_attributes[i].tag;
+			default:
+				break;
+			}
+			break;
 		default:
 			return xml_attributes[i].tag;
 		}
 	}
+
 	return TAG_DOM_ATT_any;
 }
 
@@ -404,9 +529,9 @@ const char*gf_svg_get_attribute_name(GF_Node *node, u32 tag)
 			if (ns == xml_attributes[i].xmlns)
 				return xml_attributes[i].name;
 
-			xmlns = (char *) gf_sg_get_namespace_qname(node->sgprivate->scenegraph, xml_attributes[i].xmlns);
+			xmlns = (char *) gf_xml_get_namespace_qname((GF_DOMNode*)node, xml_attributes[i].xmlns);
 			if (xmlns) {
-				sprintf(node->sgprivate->scenegraph->szNameBuffer, "%s:%s", xmlns, xml_attributes[i].name);
+				snprintf(node->sgprivate->scenegraph->szNameBuffer, 99, "%s:%s", xmlns, xml_attributes[i].name);
 				return node->sgprivate->scenegraph->szNameBuffer;
 			}
 			return xml_attributes[i].name;
@@ -420,7 +545,11 @@ GF_DOMAttribute *gf_xml_create_attribute(GF_Node *node, u32 tag)
 	return gf_node_create_attribute_from_datatype(type, tag);
 }
 
-static const struct xml_elt_def { const char *name; u32 tag; u32 xmlns; } xml_elements [] = 
+static const struct xml_elt_def {
+	const char *name;
+	u32 tag;
+	u32 xmlns;
+} xml_elements [] =
 {
 	{ "listener", TAG_SVG_listener, GF_XMLNS_XMLEV},
 	/*SVG*/
@@ -431,6 +560,7 @@ static const struct xml_elt_def { const char *name; u32 tag; u32 xmlns; } xml_el
 	{ "animateTransform", TAG_SVG_animateTransform, GF_XMLNS_SVG },
 	{ "animation", TAG_SVG_animation, GF_XMLNS_SVG },
 	{ "audio", TAG_SVG_audio, GF_XMLNS_SVG },
+	{ "clipPath", TAG_SVG_clip_path, GF_XMLNS_SVG },
 	{ "circle", TAG_SVG_circle, GF_XMLNS_SVG },
 	{ "defs", TAG_SVG_defs, GF_XMLNS_SVG },
 	{ "desc", TAG_SVG_desc, GF_XMLNS_SVG },
@@ -470,6 +600,30 @@ static const struct xml_elt_def { const char *name; u32 tag; u32 xmlns; } xml_el
 	{ "tspan", TAG_SVG_tspan, GF_XMLNS_SVG },
 	{ "use", TAG_SVG_use, GF_XMLNS_SVG },
 	{ "video", TAG_SVG_video, GF_XMLNS_SVG },
+	{ "filter", TAG_SVG_filter, GF_XMLNS_SVG },
+	{ "feDistantLight", TAG_SVG_feDistantLight, GF_XMLNS_SVG },
+	{ "fePointLight", TAG_SVG_fePointLight, GF_XMLNS_SVG },
+	{ "feSpotLight", TAG_SVG_feSpotLight, GF_XMLNS_SVG },
+	{ "feBlend", TAG_SVG_feBlend, GF_XMLNS_SVG },
+	{ "feColorMatrix", TAG_SVG_feColorMatrix, GF_XMLNS_SVG },
+	{ "feComponentTransfer", TAG_SVG_feComponentTransfer, GF_XMLNS_SVG },
+	{ "feFuncR", TAG_SVG_feFuncR, GF_XMLNS_SVG },
+	{ "feFuncG", TAG_SVG_feFuncG, GF_XMLNS_SVG },
+	{ "feFuncB", TAG_SVG_feFuncB, GF_XMLNS_SVG },
+	{ "feFuncA", TAG_SVG_feFuncA, GF_XMLNS_SVG },
+	{ "feComposite", TAG_SVG_feComposite, GF_XMLNS_SVG },
+	{ "feConvolveMatrix", TAG_SVG_feConvolveMatrix, GF_XMLNS_SVG },
+	{ "feDiffuseLighting", TAG_SVG_feDiffuseLighting, GF_XMLNS_SVG },
+	{ "feDisplacementMap", TAG_SVG_feDisplacementMap, GF_XMLNS_SVG },
+	{ "feFlood", TAG_SVG_feFlood, GF_XMLNS_SVG },
+	{ "feGaussianBlur", TAG_SVG_feGaussianBlur, GF_XMLNS_SVG },
+	{ "feImage", TAG_SVG_feImage, GF_XMLNS_SVG },
+	{ "feMerge", TAG_SVG_feMerge, GF_XMLNS_SVG },
+	{ "feMorphology", TAG_SVG_feMorphology, GF_XMLNS_SVG },
+	{ "feOffset", TAG_SVG_feOffset, GF_XMLNS_SVG },
+	{ "feSpecularLighting", TAG_SVG_feSpecularLighting, GF_XMLNS_SVG },
+	{ "feTile", TAG_SVG_feTile, GF_XMLNS_SVG },
+	{ "feTurbulence", TAG_SVG_feTurbulence, GF_XMLNS_SVG },
 
 	/*LASeR*/
 	{ "conditional", TAG_LSR_conditional, GF_XMLNS_LASER },
@@ -477,33 +631,15 @@ static const struct xml_elt_def { const char *name; u32 tag; u32 xmlns; } xml_el
 	{ "cursorManager", TAG_LSR_cursorManager, GF_XMLNS_LASER },
 	{ "selector", TAG_LSR_selector, GF_XMLNS_LASER },
 	{ "simpleLayout", TAG_LSR_simpleLayout, GF_XMLNS_LASER },
+	{ "updates", TAG_LSR_updates, GF_XMLNS_LASER },
 
-	/*XBL*/
-	{ "bindings", TAG_XBL_bindings, GF_XMLNS_XBL },
-	{ "binding", TAG_XBL_binding, GF_XMLNS_XBL },
-	{ "content", TAG_XBL_content, GF_XMLNS_XBL },
-	{ "children", TAG_XBL_children, GF_XMLNS_XBL },
-	{ "implementation", TAG_XBL_implementation, GF_XMLNS_XBL },
-	{ "constructor", TAG_XBL_constructor, GF_XMLNS_XBL },
-	{ "destructor", TAG_XBL_destructor, GF_XMLNS_XBL },
-	{ "field", TAG_XBL_field, GF_XMLNS_XBL },
-	{ "property", TAG_XBL_property, GF_XMLNS_XBL },
-	{ "getter", TAG_XBL_getter, GF_XMLNS_XBL },
-	{ "setter", TAG_XBL_setter, GF_XMLNS_XBL },
-	{ "method", TAG_XBL_method, GF_XMLNS_XBL },
-	{ "parameter", TAG_XBL_parameter, GF_XMLNS_XBL },
-	{ "body", TAG_XBL_body, GF_XMLNS_XBL },
-	{ "handlers", TAG_XBL_handlers, GF_XMLNS_XBL },
-	{ "handler", TAG_XBL_handler, GF_XMLNS_XBL },
-	{ "resources", TAG_XBL_resources, GF_XMLNS_XBL },
-	{ "stylesheet", TAG_XBL_stylesheet, GF_XMLNS_XBL },
-	{ "image", TAG_XBL_image, GF_XMLNS_XBL },
 };
 
+GF_EXPORT
 u32 gf_xml_get_element_tag(const char *element_name, u32 ns)
 {
 	u32 i, count;
-	if (!element_name) return TAG_UndefinedNode;
+//	if (!element_name) return TAG_UndefinedNode;
 	count = sizeof(xml_elements) / sizeof(struct xml_elt_def);
 	for (i=0; i<count; i++) {
 		if (!strcmp(xml_elements[i].name, element_name)) {
@@ -521,14 +657,14 @@ const char *gf_xml_get_element_name(GF_Node *n)
 	ns = n ? gf_sg_get_namespace_code(n->sgprivate->scenegraph, NULL) : 0;
 	count = sizeof(xml_elements) / sizeof(struct xml_elt_def);
 	for (i=0; i<count; i++) {
-		if (n->sgprivate->tag==xml_elements[i].tag) {
+		if (n && n->sgprivate && (n->sgprivate->tag==xml_elements[i].tag)) {
 			char *xmlns;
-			if (!n || (ns == xml_elements[i].xmlns)) 
+			if (ns == xml_elements[i].xmlns)
 				return xml_elements[i].name;
 
 			xmlns = (char *) gf_sg_get_namespace_qname(n->sgprivate->scenegraph, xml_elements[i].xmlns);
 			if (xmlns) {
-				sprintf(n->sgprivate->scenegraph->szNameBuffer, "%s:%s", xmlns, xml_elements[i].name);
+				snprintf(n->sgprivate->scenegraph->szNameBuffer, 99, "%s:%s", xmlns, xml_elements[i].name);
 				return n->sgprivate->scenegraph->szNameBuffer;
 			}
 			return xml_elements[i].name;
@@ -537,7 +673,7 @@ const char *gf_xml_get_element_name(GF_Node *n)
 	return "UndefinedNode";
 }
 
-u32 gf_xml_get_element_namespace(GF_Node *n)
+GF_NamespaceType gf_xml_get_element_namespace(GF_Node *n)
 {
 	u32 i, count;
 	if (n->sgprivate->tag==TAG_DOMFullNode) {
@@ -549,15 +685,15 @@ u32 gf_xml_get_element_namespace(GF_Node *n)
 	for (i=0; i<count; i++) {
 		if (n->sgprivate->tag==xml_elements[i].tag) return xml_elements[i].xmlns;
 	}
-	return GF_XMLNS_NONE;
+	return GF_XMLNS_UNDEFINED;
 }
 
-
 u32 gf_node_get_attribute_count(GF_Node *node)
 {
 	u32 count = 0;
-	GF_DOMNode *dom = (GF_DOMNode *)node;
-	GF_DOMAttribute *atts = dom->attributes;
+	GF_DOMAttribute *atts;
+	if (!node) return 0;
+	atts =  ((GF_DOMNode *)node)->attributes;
 	while (atts) {
 		count++;
 		atts = atts->next;
@@ -565,7 +701,7 @@ u32 gf_node_get_attribute_count(GF_Node *node)
 	return count;
 }
 
-GF_Err gf_node_get_attribute_info(GF_Node *node, GF_FieldInfo *info) 
+GF_Err gf_node_get_attribute_info(GF_Node *node, GF_FieldInfo *info)
 {
 	GF_DOMNode *dom = (GF_DOMNode *)node;
 	GF_DOMAttribute *atts = dom->attributes;
@@ -584,16 +720,16 @@ GF_Err gf_node_get_attribute_info(GF_Node *node, GF_FieldInfo *info)
 
 void gf_node_delete_attributes(GF_Node *node)
 {
-	GF_DOMAttribute *tmp;
 	GF_DOMAttribute *att = ((GF_DOMNode*)node)->attributes;
 	while(att) {
+		GF_DOMAttribute *tmp;
 		gf_svg_delete_attribute_value(att->data_type, att->data, node->sgprivate->scenegraph);
 		tmp = att;
 		att = att->next;
 		if (tmp->tag==TAG_DOM_ATT_any) {
-			free( ((GF_DOMFullAttribute*)tmp)->name);
+			gf_free( ((GF_DOMFullAttribute*)tmp)->name);
 		}
-		free(tmp);
+		gf_free(tmp);
 	}
 }
 
@@ -603,12 +739,14 @@ SVGAttribute *gf_node_create_attribute_from_datatype(u32 data_type, u32 attribut
 	if (!data_type) return NULL;
 
 	GF_SAFEALLOC(att, SVGAttribute);
+	if (!att) return NULL;
 	att->data_type = (u16) data_type;
 	att->tag = (u16) attribute_tag;
 	att->data = gf_svg_create_attribute_value(att->data_type);
 	return att;
 }
 
+GF_EXPORT
 GF_Err gf_node_get_attribute_by_name(GF_Node *node, char *name, u32 xmlns_code, Bool create_if_not_found, Bool set_default, GF_FieldInfo *field)
 {
 	u32 attribute_tag = gf_xml_get_attribute_tag(node, name, xmlns_code);
@@ -618,12 +756,12 @@ GF_Err gf_node_get_attribute_by_name(GF_Node *node, char *name, u32 xmlns_code,
 		SVGAttribute *last_att = NULL;
 		GF_DOMFullAttribute *att = (GF_DOMFullAttribute *) ((SVG_Element*)node)->attributes;
 		if (xmlns_code) ns = gf_sg_get_namespace_qname(node->sgprivate->scenegraph, xmlns_code);
-		if (ns) len = strlen(ns);
+		if (ns) len = (u32) strlen(ns);
 
 		while (att) {
-			if (((u32) att->tag == TAG_DOM_ATT_any) && 
-				((!ns && !strcmp(name, att->name)) || (!strncmp(att->name, ns, len) && !strcmp(att->name+len+1, name)))
-			) {
+			if (((u32) att->tag == TAG_DOM_ATT_any) &&
+			        ((!ns && !strcmp(name, att->name)) || (ns && !strncmp(att->name, ns, len) && att->name[len] && !strcmp(att->name+len+1, name)))
+			   ) {
 				field->fieldIndex = att->tag;
 				field->fieldType = att->data_type;
 				field->far_ptr = att->data;
@@ -634,11 +772,12 @@ GF_Err gf_node_get_attribute_by_name(GF_Node *node, char *name, u32 xmlns_code,
 		}
 		if (create_if_not_found) {
 			GF_SAFEALLOC(att, GF_DOMFullAttribute);
+			if (!att) return GF_OUT_OF_MEM;
 			att->data_type = (u16) DOM_String_datatype;
 			att->tag = (u16) TAG_DOM_ATT_any;
 			att->data = gf_svg_create_attribute_value(att->data_type);
 
-			att->name = strdup(name);
+			att->name = gf_strdup(name);
 			if (!xmlns_code)
 				att->xmlns = gf_xml_get_element_namespace(node);
 			else
@@ -658,8 +797,9 @@ GF_Err gf_node_get_attribute_by_name(GF_Node *node, char *name, u32 xmlns_code,
 }
 
 
-static void attributes_set_default_value(u32 node_tag, SVGAttribute *att)
+static void attributes_set_default_value(GF_Node *node, SVGAttribute *att)
 {
+	u32 node_tag = node->sgprivate->tag;
 	switch (att->tag) {
 	case TAG_SVG_ATT_width:
 	case TAG_SVG_ATT_height:
@@ -684,9 +824,9 @@ static void attributes_set_default_value(u32 node_tag, SVGAttribute *att)
 		}
 		break;
 	case TAG_SVG_ATT_dur:
-		if (node_tag == TAG_SVG_video || 
-			node_tag == TAG_SVG_audio ||
-			node_tag == TAG_SVG_animation)
+		if (node_tag == TAG_SVG_video ||
+		        node_tag == TAG_SVG_audio ||
+		        node_tag == TAG_SVG_animation)
 		{
 			((SMIL_Duration *)att->data)->type = SMIL_DURATION_MEDIA;
 		} else {
@@ -762,18 +902,18 @@ static void attributes_set_default_value(u32 node_tag, SVGAttribute *att)
 	case TAG_SVG_ATT_zoomAndPan:
 		*((SVG_ZoomAndPan *)att->data) = SVG_ZOOMANDPAN_MAGNIFY;
 		break;
-	case TAG_SVG_ATT_stroke_linecap: 
-		*(SVG_StrokeLineCap*)att->data = SVG_STROKELINECAP_INHERIT; 
+	case TAG_SVG_ATT_stroke_linecap:
+		*(SVG_StrokeLineCap*)att->data = SVG_STROKELINECAP_INHERIT;
 		break;
-	case TAG_SVG_ATT_stroke_linejoin: 
-		*(SVG_StrokeLineJoin*)att->data = SVG_STROKELINEJOIN_INHERIT; 
+	case TAG_SVG_ATT_stroke_linejoin:
+		*(SVG_StrokeLineJoin*)att->data = SVG_STROKELINEJOIN_INHERIT;
 		break;
 
-	case TAG_SVG_ATT_transform: 
+	case TAG_SVG_ATT_transform:
 		gf_mx2d_init(((SVG_Transform*)att->data)->mat);
 		break;
 
-		
+
 	/*all default=0 values (don't need init)*/
 	case TAG_SVG_ATT_font_family:
 	case TAG_SVG_ATT_font_style:
@@ -783,7 +923,7 @@ static void attributes_set_default_value(u32 node_tag, SVGAttribute *att)
 		break;
 
 	default:
-		GF_LOG(GF_LOG_ERROR, GF_LOG_SCENE, ("[Scene] Cannot create default value for SVG attribute %s\n", gf_svg_get_attribute_name(NULL, att->tag)));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_SCENE, ("[Scene] Cannot create default value for SVG attribute %s\n", gf_svg_get_attribute_name(node, att->tag)));
 	}
 }
 
@@ -815,8 +955,8 @@ GF_Err gf_node_get_attribute_by_tag(GF_Node *node, u32 attribute_tag, Bool creat
 			field->fieldType = att->data_type;
 			field->fieldIndex = att->tag;
 			/* attribute name should not be called, if needed use gf_svg_get_attribute_name(att->tag);*/
-			field->name = NULL; 
-			if (set_default) attributes_set_default_value(node->sgprivate->tag, att);
+			field->name = NULL;
+			if (set_default) attributes_set_default_value(node, att);
 			return GF_OK;
 		}
 	}
@@ -834,6 +974,7 @@ void gf_node_register_iri(GF_SceneGraph *sg, XMLRI *target)
 #endif
 }
 
+GF_EXPORT
 void gf_node_unregister_iri(GF_SceneGraph *sg, XMLRI *target)
 {
 #ifndef GPAC_DISABLE_SVG
@@ -841,10 +982,12 @@ void gf_node_unregister_iri(GF_SceneGraph *sg, XMLRI *target)
 #endif
 }
 
-GF_Node *gf_xml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_parent, char *inst_id, Bool deep)
+GF_Node *gf_sg_xml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_parent, char *inst_id, Bool deep)
 {
 	GF_DOMAttribute *att;
-	GF_Node *clone = gf_node_new(inScene, orig->sgprivate->tag);
+	GF_Node *clone;
+	if (!orig) return NULL;
+	clone = gf_node_new(inScene, orig->sgprivate->tag);
 	if (!clone) return NULL;
 
 	if (orig->sgprivate->tag == TAG_DOMText) {
@@ -852,16 +995,16 @@ GF_Node *gf_xml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *clone
 		n_src = (GF_DOMText *)orig;
 		n_dst = (GF_DOMText *)clone;
 		n_dst->type = n_src->type;
-		n_dst->textContent = strdup(n_src->textContent);
+		n_dst->textContent = gf_strdup(n_src->textContent);
 	} else {
 		if (orig->sgprivate->tag == TAG_DOMFullNode) {
 			GF_DOMFullNode *n_src,*n_dst;
 			n_src = (GF_DOMFullNode *)orig;
 			n_dst = (GF_DOMFullNode *)clone;
 			n_dst->ns = n_src->ns;
-			n_dst->name = strdup(n_dst->name);
-		} 
-		
+			n_dst->name = gf_strdup(n_src->name);
+		}
+
 		att = ((GF_DOMNode *)orig)->attributes;
 		while (att) {
 			GF_FieldInfo dst, src;
@@ -884,7 +1027,7 @@ GF_Node *gf_xml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *clone
 				}
 			}
 			att = att->next;
-		}	
+		}
 	}
 	if (cloned_parent) {
 		gf_node_list_add_child( & ((GF_ParentNode*)cloned_parent)->children, clone);
@@ -909,31 +1052,37 @@ GF_Node *gf_xml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *clone
 static u32 check_existing_file(char *base_file, char *ext, char *data, u32 data_size, u32 idx)
 {
 	char szFile[GF_MAX_PATH];
-	u32 fsize;
+	u64 fsize;
 	FILE *f;
-	
-	sprintf(szFile, "%s%04X%s", base_file, idx, ext);
-	
-	f = fopen(szFile, "rb");
+	int concatres;
+
+	concatres = snprintf(szFile, GF_MAX_PATH, "%s%04X%s", base_file, idx, ext);
+	if (concatres<0) {
+		GF_LOG(GF_LOG_WARNING, GF_LOG_CORE, ("Path too long (limit %d) when trying to concatenate %s and %s\n", GF_MAX_PATH, base_file, ext));
+	}
+
+	f = gf_fopen(szFile, "rb");
 	if (!f) return 0;
 
-	fseek(f, 0, SEEK_END);
-	fsize = ftell(f);
+	fsize = gf_fsize(f);
 	if (fsize==data_size) {
 		u32 offset=0;
 		char cache[1024];
-		fseek(f, 0, SEEK_SET);
+
 		while (fsize) {
-			u32 read = fread(cache, 1, 1024, f);
+			u32 read = (u32) gf_fread(cache, 1024, f);
+			if ((s32) read < 0) return 0;
 			fsize -= read;
 			if (memcmp(cache, data+offset, sizeof(char)*read)) break;
 			offset+=read;
 		}
-		fclose(f);
+		gf_fclose(f);
+		f = NULL;
 		/*same file*/
 		if (!fsize) return 2;
 	}
-	fclose(f);
+	if (f)
+		gf_fclose(f);
 	return 1;
 }
 
@@ -941,30 +1090,32 @@ static u32 check_existing_file(char *base_file, char *ext, char *data, u32 data_
 GF_EXPORT
 GF_Err gf_node_store_embedded_data(XMLRI *iri, const char *cache_dir, const char *base_filename)
 {
-	char szFile[GF_MAX_PATH], buf[20], *sep, *data, *ext;
-	u32 data_size, idx;
+	char szFile[GF_MAX_PATH], buf[20], *sep, *data=NULL, *ext;
+	u32 data_size=0, idx;
 	Bool existing;
-	FILE *f;
 
-	if (!cache_dir || !base_filename || !iri || !iri->string || strncmp(iri->string, "data:", 5)) return GF_OK;
+	if (!cache_dir || !base_filename || !iri || !iri->string) return GF_OK;
+	if (strncmp(iri->string, "data:", 5)) return GF_OK;
+	//used in test suite
+	if (!strcmp(iri->string, "data:,void")) return GF_OK;
 
 	/*handle "data:" scheme when cache is specified*/
 	strcpy(szFile, cache_dir);
-	data_size = strlen(szFile);
+	data_size = (u32) strlen(szFile);
 	if (szFile[data_size-1] != GF_PATH_SEPARATOR) {
 		szFile[data_size] = GF_PATH_SEPARATOR;
 		szFile[data_size+1] = 0;
 	}
-	if (base_filename) {
-		sep = strrchr(base_filename, GF_PATH_SEPARATOR);
+
+	sep = strrchr(base_filename, GF_PATH_SEPARATOR);
 #ifdef WIN32
-		if (!sep) sep = strrchr(base_filename, '/');
+	if (!sep) sep = strrchr(base_filename, '/');
 #endif
-		if (!sep) sep = (char *) base_filename;
-		else sep += 1;
-		strcat(szFile, sep);
-	}
-	sep = strrchr(szFile, '.');
+	if (!sep) sep = (char *) base_filename;
+	else sep += 1;
+	strcat(szFile, sep);
+
+	sep = gf_file_ext_start(szFile);
 	if (sep) sep[0] = 0;
 	strcat(szFile, "_img_");
 
@@ -972,29 +1123,30 @@ GF_Err gf_node_store_embedded_data(XMLRI *iri, const char *cache_dir, const char
 	sep = (char *)iri->string + 5;
 	if (!strncmp(sep, "image/jpg", 9) || !strncmp(sep, "image/jpeg", 10)) ext = ".jpg";
 	else if (!strncmp(sep, "image/png", 9)) ext = ".png";
-	else return GF_OK;
+	else if (!strncmp(sep, "image/svg+xml", 13)) ext = ".svg";
+	else return GF_BAD_PARAM;
 
 
 	data = NULL;
-	sep = strchr(iri->string, ';');
+	sep = strstr(iri->string, ";base");
 	if (!strncmp(sep, ";base64,", 8)) {
 		sep += 8;
-		data_size = 2*strlen(sep);
-		data = (char*)malloc(sizeof(char)*data_size);
+		data_size = 2 * (u32) strlen(sep);
+		data = (char*)gf_malloc(sizeof(char)*data_size);
 		if (!data) return GF_OUT_OF_MEM;
-		data_size = gf_base64_decode(sep, strlen(sep), data, data_size);
+		data_size = gf_base64_decode(sep, (u32) strlen(sep), data, data_size);
 	}
 	else if (!strncmp(sep, ";base16,", 8)) {
-		data_size = 2*strlen(sep);
-		data = (char*)malloc(sizeof(char)*data_size);
+		data_size = 2 * (u32) strlen(sep);
+		data = (char*)gf_malloc(sizeof(char)*data_size);
 		if (!data) return GF_OUT_OF_MEM;
 		sep += 8;
-		data_size = gf_base16_decode(sep, strlen(sep), data, data_size);
+		data_size = gf_base16_decode(sep, (u32) strlen(sep), data, data_size);
 	}
-	if (!data_size) return GF_OK;
-	
+	if (!data || !data_size) return GF_OK;
+
 	iri->type = XMLRI_STRING;
-	
+
 	existing = 0;
 	idx = 0;
 	while (1) {
@@ -1010,23 +1162,25 @@ GF_Err gf_node_store_embedded_data(XMLRI *iri, const char *cache_dir, const char
 	strcat(szFile, buf);
 	strcat(szFile, ext);
 
+	GF_Err e = GF_OK;
 	if (!existing) {
-		f = fopen(szFile, "wb");
+		FILE *f = gf_fopen(szFile, "wb");
 		if (!f) {
-			free(data);
-			free(iri->string);
+			gf_free(data);
+			gf_free(iri->string);
 			iri->string = NULL;
 			return GF_IO_ERR;
 		}
-		fwrite(data, data_size, 1, f);
-		fclose(f);
+		if (gf_fwrite(data, data_size, f)!=data_size) e = GF_IO_ERR;
+		gf_fclose(f);
 	}
-	free(data);
-	free(iri->string);
-	iri->string = strdup(szFile);
-	return GF_OK;
+	gf_free(data);
+	gf_free(iri->string);
+	iri->string = gf_strdup(szFile);
+	if (!iri->string) e = GF_OUT_OF_MEM;
+
+	return e;
 }
 
 
 #endif /*GPAC_DISABLE_SVG*/
-
