@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2024
+ *			Copyright (c) Telecom ParisTech 2000-2025
  *					All rights reserved
  *
  *  This file is part of GPAC / common tools sub-project
@@ -140,7 +140,7 @@ u32 gf_sys_clock()
 {
 	struct timeval now;
 	gettimeofday(&now, NULL);
-	return (u32) ( ( (now.tv_sec)*1000 + (now.tv_usec) / 1000) - sys_start_time );
+	return (u32) ( ( (u64)(now.tv_sec)*1000 + (now.tv_usec) / 1000) - sys_start_time );
 }
 
 GF_EXPORT
@@ -148,7 +148,7 @@ u64 gf_sys_clock_high_res()
 {
 	struct timeval now;
 	gettimeofday(&now, NULL);
-	return (now.tv_sec)*1000000 + (now.tv_usec) - sys_start_time_hr;
+	return (u64)(now.tv_sec)*1000000 + (now.tv_usec) - sys_start_time_hr;
 }
 
 #endif
@@ -170,7 +170,8 @@ u64 gf_sys_clock_high_res()
 
 #endif
 
-Bool gf_sys_enable_remotery(Bool start, Bool is_shutdown);
+GF_Err gf_sys_enable_rmtws(Bool start);
+GF_Err gf_sys_enable_userws(Bool start);
 
 static Bool gpac_disable_rti = GF_FALSE;
 
@@ -178,7 +179,6 @@ static Bool gpac_disable_rti = GF_FALSE;
 GF_EXPORT
 void gf_sleep(u32 ms)
 {
-	gf_rmt_begin(sleep, GF_RMT_AGGREGATE);
 
 #ifdef WIN32
 	Sleep(ms);
@@ -216,8 +216,6 @@ void gf_sleep(u32 ms)
 	} while ( sel_err && (errno == EINTR) );
 #endif
 
-	gf_rmt_end();
-
 }
 
 #ifndef gettimeofday
@@ -243,13 +241,13 @@ struct tm
 {
 	int tm_sec;     /* seconds after the minute - [0,59] */
 	int tm_min;     /* minutes after the hour - [0,59] */
-	int tm_hour;	/* hours since midnight - [0,23] */
-	int tm_mday;	/* day of the month - [1,31] */
+	int tm_hour;    /* hours since midnight - [0,23] */
+	int tm_mday;    /* day of the month - [1,31] */
 	int tm_mon;     /* months since January - [0,11] */
-	int tm_year;	/* years since 1900 */
-	int tm_wday;	/* days since Sunday - [0,6] */
-	int tm_yday;	/* days since January 1 - [0,365] */
-	int tm_isdst;	/* daylight savings time flag */
+	int tm_year;    /* years since 1900 */
+	int tm_wday;    /* days since Sunday - [0,6] */
+	int tm_yday;    /* days since January 1 - [0,365] */
+	int tm_isdst;   /* daylight savings time flag */
 };
 #define _TM_DEFINED
 #endif /* _TM_DEFINED */
@@ -341,6 +339,8 @@ s32 __gettimeofday(struct timeval *tp, void *tz)
 
 #elif defined(WIN32)
 
+#include <WinSock2.h>
+
 static s32 gettimeofday(struct timeval *tp, void *tz)
 {
 	struct _timeb timebuffer;
@@ -531,7 +531,7 @@ void gf_prompt_set_echo_off(Bool echo_off)
 GF_EXPORT
 GF_Err gf_prompt_get_size(u32 *width, u32 *height)
 {
-    CONSOLE_SCREEN_BUFFER_INFO info;
+	CONSOLE_SCREEN_BUFFER_INFO info;
 	HANDLE hStdin = GetStdHandle(STD_INPUT_HANDLE);
 	BOOL ret = GetConsoleScreenBufferInfo(hStdin, &info);
 
@@ -633,12 +633,12 @@ GF_EXPORT
 GF_Err gf_prompt_get_size(u32 *width, u32 *height)
 {
 #if defined(TIOCGWINSZ)
-    struct winsize ws;
-    if (ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) != 0) return GF_IO_ERR;
+	struct winsize ws;
+	if (ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) != 0) return GF_IO_ERR;
 
-    if (width) *width = ws.ws_col;
-    if (height) *height = ws.ws_row;
-    return GF_OK;
+	if (width) *width = ws.ws_col;
+	if (height) *height = ws.ws_row;
+	return GF_OK;
 #elif defined(WIOCGETD)
 	struct uwdata w;
 	if (ioctl(2, WIOCGETD, &w) != 0) return GF_IO_ERR;
@@ -647,9 +647,9 @@ GF_Err gf_prompt_get_size(u32 *width, u32 *height)
 		*width = w.uw_width / w.uw_hs;
 	if (height && (w.uw_height > 0))
 		*height = w.uw_height / w.uw_vs;
-    return GF_OK;
+	return GF_OK;
 #else
-    return GF_NOT_SUPPORTED;
+	return GF_NOT_SUPPORTED;
 #endif
 }
 
@@ -743,6 +743,8 @@ u64 gf_sys_clock_high_res()
 
 #ifdef WIN32
 
+#include <timeapi.h>
+
 static u32 OS_GetSysClockHIGHRES()
 {
 	LARGE_INTEGER now;
@@ -858,6 +860,8 @@ static Bool gpac_discard_config = GF_FALSE;
 extern FILE *gpac_log_file;
 extern Bool gpac_log_time_start;
 extern Bool gpac_log_utc_time;
+void gf_logs_init();
+void gf_logs_close();
 #endif
 
 GF_EXPORT
@@ -1065,7 +1069,7 @@ GF_Err gf_sys_set_args(s32 argc, const char **argv)
 		gf_log_reset_file();
 #endif
 		if (gf_opts_get_bool("core", "rmt"))
-			gf_sys_enable_remotery(GF_TRUE, GF_FALSE);
+			gf_sys_enable_rmtws(GF_TRUE);
 
 		if (gpac_quiet) {
 			if (gpac_quiet==2) gf_log_set_tool_level(GF_LOG_ALL, GF_LOG_QUIET);
@@ -1161,191 +1165,140 @@ const char *gf_sys_find_global_arg(const char *arg)
 }
 
 
-#ifndef GPAC_DISABLE_REMOTERY
-Remotery *remotery_handle=NULL;
-
-//commented out as it put quite some load on the browser
-
-gf_log_cbk gpac_prev_default_logs = NULL;
-
-const char *gf_log_tool_name(GF_LOG_Tool log_tool);
-const char *gf_log_level_name(GF_LOG_Level log_level);
-
-void gpac_rmt_log_callback(void *cbck, GF_LOG_Level level, GF_LOG_Tool tool, const char *fmt, va_list vlist)
-{
-#ifndef GPAC_DISABLE_LOG
-
-#define RMT_LOG_SIZE	5000
-	char szMsg[RMT_LOG_SIZE];
-	u32 len;
-	sprintf(szMsg, "{ \"type\": \"logs\", \"level\": \"%s\" \"tool\": \"%s\", \"value\": \"", gf_log_level_name(level), gf_log_tool_name(tool));
-
-	len = (u32) strlen(szMsg);
-	vsnprintf(szMsg, RMT_LOG_SIZE - len - 3, fmt, vlist);
-	strcat(szMsg, "\"}");
-
-	rmt_LogText(szMsg);
-
-#undef RMT_LOG_SIZE
-
+#ifndef GPAC_DISABLE_RMTWS
+RMT_WS* rmtws_handle=NULL;
+RMT_WS* userws_handle=NULL;
 #endif
 
-}
-
-static void *rmt_udta = NULL;
-gf_rmt_user_callback rmt_usr_cbk = NULL;
-
-static void gpac_rmt_input_handler(const char* text, void* context)
-{
-	if (text && rmt_usr_cbk)
-		rmt_usr_cbk(rmt_udta, text);
-}
+GF_EXPORT
+void* gf_sys_get_rmtws() {
+#ifndef GPAC_DISABLE_RMTWS
+	return (void*)rmtws_handle;
+#else
+	return NULL;
 #endif
+}
 
-Bool gf_sys_enable_remotery(Bool start, Bool is_shutdown)
-{
-#ifndef GPAC_DISABLE_REMOTERY
-	if (start && !remotery_handle) {
-		rmtSettings *rmcfg = rmt_Settings();
+GF_EXPORT
+void* gf_sys_get_userws() {
+#ifndef GPAC_DISABLE_RMTWS
+	return (void*)userws_handle;
+#else
+	return NULL;
+#endif
+}
+
+
+GF_EXPORT
+GF_Err gf_sys_enable_rmtws(Bool start) {
+#ifndef GPAC_DISABLE_RMTWS
+	if (start && !rmtws_handle) {
+
+		rmtws_handle = rmt_ws_new();
+		if (!rmtws_handle) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[core] unable to initialize RMT websocket server\n"));
+			return GF_OUT_OF_MEM;
+		}
+
+		RMT_Settings *rmcfg = gf_rmt_get_settings(rmtws_handle);
 
 		rmcfg->port = gf_opts_get_int("core", "rmt-port");
-		rmcfg->reuse_open_port = gf_opts_get_bool("core", "rmt-reuse");
 		rmcfg->limit_connections_to_localhost = gf_opts_get_bool("core", "rmt-localhost");
 		rmcfg->msSleepBetweenServerUpdates = gf_opts_get_int("core", "rmt-sleep");
-		rmcfg->maxNbMessagesPerUpdate = gf_opts_get_int("core", "rmt-nmsg");
-		rmcfg->messageQueueSizeInBytes = gf_opts_get_int("core", "rmt-qsize");
-		rmcfg->input_handler = gpac_rmt_input_handler;
+		rmcfg->cert = gf_opts_get_key("core", "rmt-cert");
+		rmcfg->pkey = gf_opts_get_key("core", "rmt-pkey");
 
-		rmtError rme = rmt_CreateGlobalInstance(&remotery_handle);
-		if (rme != RMT_ERROR_NONE) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[core] unable to initialize Remotery profiler: error %d\n", rme));
-			return GF_FALSE;
+		rmt_ws_run(rmtws_handle);
+
+
+	} else if (!start && rmtws_handle) {
+
+		rmt_ws_del(rmtws_handle);
+		rmtws_handle=NULL;
+
+	}
+	return GF_OK;
+#else
+	return GF_NOT_SUPPORTED;
+#endif
+}
+
+GF_EXPORT
+GF_Err gf_sys_enable_userws(Bool start) {
+#ifndef GPAC_DISABLE_RMTWS
+	if (start && !userws_handle) {
+
+		userws_handle = rmt_ws_new();
+		if (!userws_handle) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[core] unable to initialize RMT websocket server\n"));
+			return GF_OUT_OF_MEM;
 		}
-		//OpenGL binding is done upon loading of the driver, otherwise crashes on windows
 
-		if (gf_opts_get_bool("core", "rmt-log")) {
-			gpac_prev_default_logs = gf_log_set_callback(NULL, gpac_rmt_log_callback);
-		}
-#ifdef GPAC_ENABLE_COVERAGE
-		if (gf_sys_is_cov_mode()) {
-			gpac_rmt_input_handler(NULL, NULL);
-		}
-#endif
+		RMT_Settings *rmcfg = gf_rmt_get_settings(userws_handle);
 
-	} else if (!start && remotery_handle) {
-		if (gf_opts_get_bool("core", "rmt-ogl"))
-			rmt_UnbindOpenGL();
+		rmcfg->port = gf_opts_get_int("core", "userws-port");
+		rmcfg->limit_connections_to_localhost = gf_opts_get_bool("core", "userws-localhost");
+		rmcfg->msSleepBetweenServerUpdates = gf_opts_get_int("core", "userws-sleep");
+		rmcfg->cert = gf_opts_get_key("core", "userws-cert");
+		rmcfg->pkey = gf_opts_get_key("core", "userws-pkey");
 
-		rmt_DestroyGlobalInstance(remotery_handle);
+		rmt_ws_run(userws_handle);
+
+
+	} else if (!start && userws_handle) {
+
+		rmt_ws_del(userws_handle);
+		userws_handle=NULL;
 
-		remotery_handle=NULL;
-		if (gpac_prev_default_logs != NULL)
-			gf_log_set_callback(NULL, gpac_prev_default_logs);
 	}
-	return GF_TRUE;
+	return GF_OK;
 #else
 	return GF_NOT_SUPPORTED;
 #endif
 }
 
-GF_EXPORT
-GF_Err gf_sys_profiler_set_callback(void *udta, gf_rmt_user_callback usr_cbk)
-{
-#ifndef GPAC_DISABLE_REMOTERY
-	if (remotery_handle) {
-		rmt_udta = udta;
-		rmt_usr_cbk = usr_cbk;
-		return GF_OK;
-	}
-	return GF_BAD_PARAM;
-#else
-	return GF_NOT_SUPPORTED;
-#endif
-}
-
-GF_EXPORT
-GF_Err gf_sys_profiler_log(const char *msg)
-{
-#ifndef GPAC_DISABLE_REMOTERY
-	if (remotery_handle) {
-		rmt_LogText(msg);
-		return GF_OK;
-	}
-	return GF_BAD_PARAM;
-#else
-	return GF_NOT_SUPPORTED;
-#endif
-}
-
-GF_EXPORT
-GF_Err gf_sys_profiler_send(const char *msg)
-{
-#ifndef GPAC_DISABLE_REMOTERY
-	if (remotery_handle) {
-		rmt_SendText(msg);
-		return GF_OK;
-	}
-	return GF_BAD_PARAM;
-#else
-	return GF_NOT_SUPPORTED;
-#endif
-}
-
-GF_EXPORT
-void gf_sys_profiler_enable_sampling(Bool enable)
-{
-#ifndef GPAC_DISABLE_REMOTERY
-	if (remotery_handle) {
-		rmt_EnableSampling(enable);
-	}
-#endif
-}
-
-GF_EXPORT
-Bool gf_sys_profiler_sampling_enabled()
-{
-#ifndef GPAC_DISABLE_REMOTERY
-	if (remotery_handle) {
-		return rmt_SamplingEnabled();
-	}
-#endif
-	return GF_FALSE;
-}
 
 #include <gpac/list.h>
 GF_List *all_blobs = NULL;
 
 GF_EXPORT
-GF_Err gf_blob_get(const char *blob_url, u8 **out_data, u32 *out_size, u32 *out_flags)
+GF_Err gf_blob_get_ex(GF_Blob *blob, u8 **out_data, u32 *out_size, u32 *out_flags)
 {
-	GF_Blob *blob = NULL;
-	if (strncmp(blob_url, "gmem://", 7)) return GF_BAD_PARAM;
-	if (sscanf(blob_url, "gmem://%p", &blob) != 1) return GF_BAD_PARAM;
 	if (!blob)
 		return GF_BAD_PARAM;
 	if (gf_list_find(all_blobs, blob)<0)
 		return GF_URL_REMOVED;
-	if (blob->data && blob->mx)
-		gf_mx_p(blob->mx);
+	gf_mx_p(blob->mx);
 	if (out_data) *out_data = blob->data;
 	if (out_size) *out_size = blob->size;
 	if (out_flags) *out_flags = blob->flags;
 	return GF_OK;
 }
+GF_EXPORT
+GF_Err gf_blob_get(const char *blob_url, u8 **out_data, u32 *out_size, u32 *out_flags)
+{
+	GF_Blob *blob = NULL;
+	if (sscanf(blob_url, "gmem://%p", &blob) != 1) return GF_BAD_PARAM;
+	return gf_blob_get_ex(blob, out_data, out_size, out_flags);
+}
+
+GF_EXPORT
+GF_Err gf_blob_release_ex(GF_Blob *blob)
+{
+	if (!blob)
+		return GF_BAD_PARAM;
+	if (gf_list_find(all_blobs, blob)<0)
+		return GF_URL_REMOVED;
+	gf_mx_v(blob->mx);
+	return GF_OK;
+}
 
 GF_EXPORT
 GF_Err gf_blob_release(const char *blob_url)
 {
-    GF_Blob *blob = NULL;
-    if (strncmp(blob_url, "gmem://", 7)) return GF_BAD_PARAM;
-    if (sscanf(blob_url, "gmem://%p", &blob) != 1) return GF_BAD_PARAM;
-    if (!blob)
-		return GF_BAD_PARAM;
-	if (gf_list_find(all_blobs, blob)<0)
-		return GF_URL_REMOVED;
-    if (blob->data && blob->mx)
-        gf_mx_v(blob->mx);
-    return GF_OK;
+	GF_Blob *blob = NULL;
+	if (sscanf(blob_url, "gmem://%p", &blob) != 1) return GF_BAD_PARAM;
+	return gf_blob_release_ex(blob);
 }
 
 GF_EXPORT
@@ -1376,7 +1329,6 @@ void gf_blob_unregister(GF_Blob *blob)
 GF_Blob *gf_blob_from_url(const char *blob_url)
 {
 	GF_Blob *blob = NULL;
-	if (strncmp(blob_url, "gmem://", 7)) return NULL;
 	if (sscanf(blob_url, "gmem://%p", &blob) != 1) return NULL;
 	if (!blob)
 		return NULL;
@@ -1386,6 +1338,17 @@ GF_Blob *gf_blob_from_url(const char *blob_url)
 }
 #endif
 
+
+GF_EXPORT
+GF_BlobRangeStatus gf_blob_query_range(GF_Blob *blob, Bool check_when_complete, u64 start_offset, u32 size)
+{
+	if (!blob) return GF_BLOB_RANGE_CORRUPTED;
+	if (blob->range_valid) return blob->range_valid(blob, check_when_complete, start_offset, &size);
+
+	if (blob->flags & GF_BLOB_IN_TRANSFER) return GF_BLOB_RANGE_IN_TRANSFER;
+	return GF_BLOB_RANGE_VALID;
+}
+
 void gf_init_global_config(const char *profile);
 void gf_uninit_global_config(Bool discard_config);
 
@@ -1488,6 +1451,7 @@ GF_Err gf_sys_init(GF_MemTrackerType mem_tracker_type, const char *profile)
 #endif
 
 #ifndef GPAC_DISABLE_LOG
+		gf_logs_init();
 		/*by default log subsystem is initialized to error on all tools, and info on console to debug scripts*/
 		gf_log_set_tool_level(GF_LOG_ALL, GF_LOG_WARNING);
 		gf_log_set_tool_level(GF_LOG_APP, GF_LOG_INFO);
@@ -1609,6 +1573,9 @@ GF_Err gf_sys_init(GF_MemTrackerType mem_tracker_type, const char *profile)
 void gf_net_close_capture();
 #endif
 
+extern GF_List *gfio_delete_handlers;
+extern GF_List *allocated_gfios;
+
 GF_EXPORT
 void gf_sys_close()
 {
@@ -1630,7 +1597,8 @@ void gf_sys_close()
 		psapi_hinst = NULL;
 #endif
 
-		gf_sys_enable_remotery(GF_FALSE, GF_TRUE);
+		gf_sys_enable_rmtws(GF_FALSE);
+		gf_sys_enable_userws(GF_FALSE);
 
 #ifdef GPAC_HAS_QJS
 		void gf_js_delete_runtime();
@@ -1640,10 +1608,7 @@ void gf_sys_close()
 		gf_uninit_global_config(gpac_discard_config);
 
 #ifndef GPAC_DISABLE_LOG
-		if (gpac_log_file) {
-			gf_fclose(gpac_log_file);
-			gpac_log_file = NULL;
-		}
+		gf_logs_close();
 #endif
 		if (gpac_lang_file) gf_cfg_del(gpac_lang_file);
 		gpac_lang_file = NULL;
@@ -1668,6 +1633,9 @@ void gf_sys_close()
 		gf_list_del(all_blobs);
 		all_blobs = NULL;
 
+		gf_list_del(gfio_delete_handlers);
+		gf_list_del(allocated_gfios);
+
 #if !defined(GPAC_DISABLE_NETCAP) && !defined(GPAC_DISABLE_NETWORK)
 		gf_net_close_capture();
 #endif
@@ -2098,16 +2066,16 @@ Bool gf_sys_get_rti_os(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 Bool gf_sys_get_rti_os(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 {
 	u64 u_k_time = 0;
-    kern_return_t kr;
-    task_info_data_t tinfo;
-    mach_msg_type_number_t task_info_count;
-    task_basic_info_t basic_info;
-    thread_array_t thread_list;
-    mach_msg_type_number_t thread_count;
-    thread_info_data_t thinfo;
-    mach_msg_type_number_t thread_info_count;
-    thread_basic_info_t basic_info_th;
-    u32 j, tot_cpu = 0, nb_threads = 0;
+	kern_return_t kr;
+	task_info_data_t tinfo;
+	mach_msg_type_number_t task_info_count;
+	task_basic_info_t basic_info;
+	thread_array_t thread_list;
+	mach_msg_type_number_t thread_count;
+	thread_info_data_t thinfo;
+	mach_msg_type_number_t thread_info_count;
+	thread_basic_info_t basic_info_th;
+	u32 j, tot_cpu = 0, nb_threads = 0;
 
 	u32 entry_time = gf_sys_clock();
 	if (last_update_time && (entry_time - last_update_time < refresh_time_ms)) {
@@ -2115,49 +2083,49 @@ Bool gf_sys_get_rti_os(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 		return GF_FALSE;
 	}
 
-    task_info_count = TASK_INFO_MAX;
-    kr = task_info(mach_task_self(), TASK_BASIC_INFO, (task_info_t)tinfo, &task_info_count);
-    if (kr != KERN_SUCCESS) {
-        return GF_FALSE;
-    }
+	task_info_count = TASK_INFO_MAX;
+	kr = task_info(mach_task_self(), TASK_BASIC_INFO, (task_info_t)tinfo, &task_info_count);
+	if (kr != KERN_SUCCESS) {
+		return GF_FALSE;
+	}
 
-    basic_info = (task_basic_info_t)tinfo;
-    the_rti.gpac_memory = the_rti.process_memory = basic_info->resident_size;
+	basic_info = (task_basic_info_t)tinfo;
+	the_rti.gpac_memory = the_rti.process_memory = basic_info->resident_size;
 
-    // get threads in the task
-    kr = task_threads(mach_task_self(), &thread_list, &thread_count);
-    if (kr != KERN_SUCCESS) {
-        return GF_FALSE;
-    }
-    if (thread_count > 0)
-        nb_threads = (u32) thread_count;
+	// get threads in the task
+	kr = task_threads(mach_task_self(), &thread_list, &thread_count);
+	if (kr != KERN_SUCCESS) {
+		return GF_FALSE;
+	}
+	if (thread_count > 0)
+		nb_threads = (u32) thread_count;
 
-    for (j = 0; j < nb_threads; j++)   {
-        thread_info_count = THREAD_INFO_MAX;
-        kr = thread_info(thread_list[j], THREAD_BASIC_INFO,
-                         (thread_info_t)thinfo, &thread_info_count);
-        if (kr != KERN_SUCCESS) {
+	for (j = 0; j < nb_threads; j++)   {
+		thread_info_count = THREAD_INFO_MAX;
+		kr = thread_info(thread_list[j], THREAD_BASIC_INFO,
+						 (thread_info_t)thinfo, &thread_info_count);
+		if (kr != KERN_SUCCESS) {
 			vm_deallocate(mach_task_self(), (vm_offset_t)thread_list, thread_count * sizeof(thread_t));
-            return GF_FALSE;
-        }
+			return GF_FALSE;
+		}
 
-        basic_info_th = (thread_basic_info_t)thinfo;
+		basic_info_th = (thread_basic_info_t)thinfo;
 
-        if (!(basic_info_th->flags & TH_FLAGS_IDLE)) {
-            u64 tot_sec = basic_info_th->user_time.seconds + basic_info_th->system_time.seconds;
-            tot_sec *= 1000000;
-            tot_sec += basic_info_th->user_time.microseconds + basic_info_th->system_time.microseconds;
-            u_k_time += tot_sec;
+		if (!(basic_info_th->flags & TH_FLAGS_IDLE)) {
+			u64 tot_sec = basic_info_th->user_time.seconds + basic_info_th->system_time.seconds;
+			tot_sec *= 1000000;
+			tot_sec += basic_info_th->user_time.microseconds + basic_info_th->system_time.microseconds;
+			u_k_time += tot_sec;
 			tot_cpu += basic_info_th->cpu_usage;
-        }
+		}
 
-    } // for each thread
+	} // for each thread
 
-    the_rti.process_cpu_usage = (tot_cpu * 100) / TH_USAGE_SCALE;
+	the_rti.process_cpu_usage = (tot_cpu * 100) / TH_USAGE_SCALE;
 	the_rti.total_cpu_usage = the_rti.process_cpu_usage;
 
-    kr = vm_deallocate(mach_task_self(), (vm_offset_t)thread_list, thread_count * sizeof(thread_t));
-    gf_assert(kr == KERN_SUCCESS);
+	kr = vm_deallocate(mach_task_self(), (vm_offset_t)thread_list, thread_count * sizeof(thread_t));
+	gf_assert(kr == KERN_SUCCESS);
 
 
 	if (last_update_time) {
@@ -2301,6 +2269,9 @@ Bool gf_sys_get_rti_os(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 	the_rti.physical_memory = EM_ASM_INT(return HEAP8.length);
 	s_mallinfo mi = mallinfo();
 	the_rti.physical_memory_avail = the_rti.physical_memory - (unsigned int)sbrk(0) + mi.fordblks;
+#elif defined(GPAC_CONFIG_FREEBSD)
+	/* FreeBSD doesn't have /proc/meminfo, would need sysctl to get memory info */
+	the_rti.physical_memory = the_rti.physical_memory_avail = 0;
 #else
 	the_rti.physical_memory = the_rti.physical_memory_avail = 0;
 	f = gf_fopen("/proc/meminfo", "r");
@@ -2513,7 +2484,7 @@ GF_GlobalLock * gf_create_PID_file( const char * resourceName )
 		}
 		*pid = '\0';
 	}
-	int fd = open(pidfile, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+	int fd = open(pidfile, O_RDWR | O_CREAT , S_IRUSR | S_IWUSR);
 	if (fd == -1)
 		goto exit;
 	/* Get the flags */
@@ -2742,7 +2713,6 @@ s32 gf_net_get_ntp_diff_ms(u64 ntp)
 	return (s32) (local - remote);
 }
 
-#if 0
 /*!
 
 Adds or remove a given amount of microseconds to an NTP timestamp
@@ -2751,7 +2721,7 @@ Adds or remove a given amount of microseconds to an NTP timestamp
 \return adjusted NTP timestamp
  */
 GF_EXPORT
-u64 gf_net_add_usec(u64 ntp, s32 usec)
+u64 gf_net_ntp_add_usec(u64 ntp, s32 usec)
 {
 	u64 sec, frac;
 	s64 usec_ntp;
@@ -2772,7 +2742,6 @@ u64 gf_net_add_usec(u64 ntp, s32 usec)
 	ntp |= (sec<<32);
 	return ntp;
 }
-#endif
 
 
 GF_EXPORT
@@ -2921,6 +2890,7 @@ u64 gf_net_parse_date(const char *val)
 	year = month = day = h = m = s = 0;
 	oh = om = 0;
 	secs = 0;
+	Bool has_sep = strchr(val, ':') ? GF_TRUE : GF_FALSE;
 
 	if (sscanf(val, "%d-%d-%dT%d:%d:%g-%d:%d", &year, &month, &day, &h, &m, &secs, &oh, &om) == 8) {
 		neg_time_zone = GF_TRUE;
@@ -2929,6 +2899,8 @@ u64 gf_net_parse_date(const char *val)
 	}
 	else if (sscanf(val, "%d-%d-%dT%d:%d:%gZ", &year, &month, &day, &h, &m, &secs) == 6) {
 	}
+	else if (sscanf(val, "%d/%d/%dT%d:%d:%gZ", &year, &month, &day, &h, &m, &secs) == 6) {
+	}
 	else if (sscanf(val, "%3s, %d %3s %d %d:%d:%d", szDay, &day, szMonth, &year, &h, &m, &s)==7) {
 		secs  = (Float) s;
 	}
@@ -2938,11 +2910,11 @@ u64 gf_net_parse_date(const char *val)
 	else if (sscanf(val, "%3s %3s %d %02d:%02d:%02d %d", szDay, szMonth, &day, &year, &h, &m, &s)==7) {
 		secs  = (Float) s;
 	}
-	else if (sscanf(val, LLU, &current_time) == 1 && current_time > 1000000000 && current_time < GF_INT_MAX) {
-		return current_time * 1000; // guessed raw duration since UTC0 in seconds
-	}
-	else if (sscanf(val, LLU, &current_time) == 1 && current_time > 1000000000000ULL && current_time < GF_INT_MAX * 1000ULL) {
+	else if (!has_sep && (sscanf(val, LLU, &current_time) == 1) && current_time > 1000000000000ULL && current_time < GF_INT_MAX * 1000ULL) {
 		return current_time; // guessed duration since UTC0 in milliseconds
+	}
+	else if (!has_sep && (sscanf(val, LLU, &current_time) == 1) && current_time < GF_INT_MAX) {
+		return current_time * 1000; // guessed raw duration since UTC0 in seconds
 	} else {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[Core] Cannot parse date string %s\n", val));
 		return 0;
@@ -3178,6 +3150,9 @@ GF_Err gf_file_load_data(const char *file_name, u8 **out_data, u32 *out_size)
 	FILE *file = gf_fopen(file_name, "rb");
 
 	if (!file) {
+		if (!gf_file_exists(file_name))
+			return GF_URL_ERROR;
+
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[Core] Cannot open file %s\n", file_name));
 		return GF_IO_ERR;
 	}
@@ -3188,11 +3163,18 @@ GF_Err gf_file_load_data(const char *file_name, u8 **out_data, u32 *out_size)
 
 #ifndef WIN32
 #include <unistd.h>
+#include <fcntl.h>
+
 GF_EXPORT
 u32 gf_sys_get_process_id()
 {
 	return getpid ();
 }
+GF_EXPORT
+Bool gf_sys_check_process_id(u32 pid)
+{
+	return (getpgid(pid) == -1) ? GF_FALSE : GF_TRUE;
+}
 #else
 #include <windows.h>
 GF_EXPORT
@@ -3200,8 +3182,70 @@ u32 gf_sys_get_process_id()
 {
 	return GetCurrentProcessId();
 }
+GF_EXPORT
+Bool gf_sys_check_process_id(u32 pid)
+{
+	HANDLE processHandle = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pid);
+	Bool ret = GF_FALSE;
+	if (processHandle) {
+		CloseHandle(processHandle);
+		ret = GF_TRUE;
+	}
+	return ret;
+}
 #endif
 
+GF_EXPORT
+GF_LockStatus gf_sys_create_lockfile(const char *lockname)
+{
+	char szPID[20];
+	u32 retry = 10;
+	sprintf(szPID, "%u", gf_sys_get_process_id());
+	u32 len = (u32)strlen(szPID);
+
+	while (retry) {
+		retry--;
+#ifdef GPAC_HAS_FD
+		s32 fd = open(lockname, O_CREAT | O_EXCL | O_WRONLY | O_BINARY, S_IRUSR|S_IWUSR);
+		if (fd != -1) {
+			s32 wlen = write(fd, szPID, len);
+			close(fd);
+			if (wlen == (s32)len) return GF_LOCKFILE_NEW;
+			continue;
+		}
+#else
+#if defined __STDC_VERSION__
+		FILE *f = fopen(lockname, "wx");
+#else
+		FILE *f = gf_file_exists(lockname) ? NULL : fopen(lockname, "w");
+#endif
+		if (f) {
+			s32 wlen = (s32) fwrite(szPID, 1, len, f);
+			fclose(f);
+			if (wlen == (s32)len) return GF_LOCKFILE_NEW;
+			continue;
+		}
+#endif
+
+		//existing, check pid
+		u8 *data=NULL;
+		u32 size=0, pid;
+		GF_Err e = gf_file_load_data(lockname, &data, &size);
+		if (!data || !size || e!=GF_OK) continue;
+
+		sscanf(data, "%u", &pid);
+		gf_free(data);
+		//already locked by ourselves
+		if (pid == gf_sys_get_process_id()) return GF_LOCKFILE_REUSE;
+		//pid is alive, lock fail
+		if (gf_sys_check_process_id(pid)) return GF_LOCKFILE_FAILED;
+
+		//file exists but pid dead, grab the lock
+		gf_file_delete(lockname);
+	}
+	return GF_LOCKFILE_FAILED;
+}
+
 #ifndef WIN32
 #include <termios.h>
 GF_EXPORT
